﻿using System;
using System.Linq;

/// @author vesal
/// @version ..2018
/// <summary>
/// Esimerkki matriisin palauttamsiesta
/// </summary>
public class Kertominen
{
    /// <summary>
    /// Kerrotaan matriisin alkiot
    /// </summary>
    public static void Main()
    {
        double[,] a = {
            {1,2,3,4},
            {2,2,2,3},
            {4,2,3,5}
        };

        double[,] m = Kerro(a, 2);
        Console.WriteLine(String.Join(" ", m.Cast<double>()));
    }


    /// <summary>
    /// Kerrotaan matriisin jokainen alkio kertoimella.  Palautetaan
    /// uusi matriisi, alkuperäinen ei muutu.
    /// </summary>
    /// <param name="a">Kerrottava matriisi</param>
    /// <param name="kerroin">luku jolla jokainen alkio kerrotaan</param>
    /// <returns>kerrottu matriisi</returns>
    /// <example>
    /// <pre name="test">
    ///   double[,] a = {{1,2,3}, {2,2,2}, {4,2,3}};
    ///   double [,] m = Kerro(a, 2);
    ///   String.Join(" ",a.Cast<double>()) === "1 2 3 2 2 2 4 2 3";
    ///   String.Join(" ",m.Cast<double>()) === "2 4 6 4 4 4 8 4 6";
    ///   m = Kerro(m, 0.5);
    ///   String.Join(" ",m.Cast<double>()) === "1 2 3 2 2 2 4 2 3";
    /// </pre>
    /// </example>
    public static double[,] Kerro(double[,] a, double kerroin)
    {
        int riveja = a.GetLength(0);
        int sarakkeita = a.GetLength(1);
        double[,] m = new double[riveja, sarakkeita];
        for (int iy = 0; iy < riveja; iy++)
            for (int ix = 0; ix < sarakkeita; ix++)
                m[iy, ix] = kerroin * a[iy, ix];
        return m;
    }
}
