﻿using System;
using System.Text;
using System.Linq;
using System.Collections.Generic;

/// @author vesal
/// @version 24.10.2018
/// <summary>
///   Tulostetaan taulukossa olevat vektorit
/// </summary>
public class VektoreitaTaulukossa
{
    /// <summary>
    ///  Tulostetaan taulukossa olevat vektorit
    /// </summary>
    public static void Main()
    {
        Vector[] pisteet = { new Vector(3, 1), new Vector(9, 2), new Vector(2, 9), new Vector(12, 5) };
        TulostaPisteet(pisteet);
        int indeksiLahella = EtsiPisteenPaikka(pisteet, new Vector(8,3), 2.0);
        if ( indeksiLahella < 0 ) Console.WriteLine("Ei löydy");
        else Console.WriteLine($"Löytyi piste {pisteet[indeksiLahella]}");
    }


    /// <summary>
    /// Etsitään taulukosta ensimmäisen pisteen paikka, joka on etsittävästä korkeintaan etaisyyden paassa.
    /// </summary>
    /// <param name="pisteet">taulokko josta etsitään</param>
    /// <param name="etsittava">piste jonka lähellä olevaa etsitään</param>
    /// <param name="etaisyys">etäisyys jonka alle oleva piste hyväksytään</param>
    /// <returns>lähellä oleva pisteen indkesi jos löytyy, muuten -1</returns>
    /// <example>
    /// <pre name="test">
    ///  Vector[] pisteet = { new Vector(3, 1), new Vector(9, 2), new Vector(2, 9), new Vector(12, 5) };
    ///  EtsiPisteenPaikka(pisteet, new Vector(3,1), 0.1) === 0;
    ///  EtsiPisteenPaikka(pisteet, new Vector(8,3), 2.0) === 1;
    ///  EtsiPisteenPaikka(pisteet, new Vector(3,8), 2.0) === 2;
    ///  EtsiPisteenPaikka(pisteet, new Vector(11,5), 1.5) === 3;
    ///  EtsiPisteenPaikka(new Vector[0], new Vector(8,3), 2.0) === -1;
    /// </pre>
    /// </example>
    public static int EtsiPisteenPaikka(Vector[] pisteet, Vector etsittava, double etaisyys)
    {
        for (int i = 0; i < pisteet.Length; i++)
        {
            Vector p = pisteet[i];
            if (Vector.Distance(p, etsittava) <= etaisyys) return i;
        }
        return -1;
    }


    /// <summary>
    /// Tulostetaan taulukossa olevat pisteet kukin omalle rivilleen
    /// </summary>
    /// <param name="pisteet">Tulostettavat pisteet</param>
    private static void TulostaPisteet(Vector[] pisteet)
    {
        for (int i = 0; i < pisteet.Length; i++)
        {
            Vector p = pisteet[i];
            Console.WriteLine(p);
        }
    }
}


public class Vector
{
    private double x, y;
    public Vector(double x, double y) { this.x = x; this.y = y; }
    override public string ToString() { return $"({x,5:0.00},{y,5:0.00})"; }
    public static double Distance(Vector p1, Vector p2) {
        double dx = (p1.x - p2.x), dy = (p1.y - p2.y); return Math.Sqrt(dx * dx + dy * dy);
    } 
}