using System;
using System.Collections.Generic;
using MinJypeli;

namespace Demo7
{
    /// @author  Vesa Lappalainen
    /// @author  Enni Stylman
    /// @version 30.10.2019
    /// <summary>
    /// Käsitellään pistejoukkoja piste-olioiden avulla. 
    /// </summary>
    public class Pisteet2
    {
        private const double x1 = -5;
        private const double x2 = 20;
        private const double y1 = -5;
        private const double y2 = 20;


        /// <summary>
        /// Alustetaan pistetaulukko ja kysellään käyttäjältä pisteitä ja tulostetaan
        /// aina sitä lähimmän pisteen koordinaatit.
        /// Ohjelma loppuu kun käyttäjä vastaa pelkän return.
        /// </summary>
        public static void Main()
        {
            Vector[] taulukkoVektoreita;
            taulukkoVektoreita = ArvoPisteet(1, x1, y1, x2, y2);

            Piste pisteOtus = LuoHienoPiste(taulukkoVektoreita[0]);
            Piste punainenPiste = LuoPunainenPiste();

            TulostaPisteetE4(pisteOtus); //tulostaa ensimmäisen pisteotuksen tiedot
            Console.WriteLine("...................."); //tulostaa pisterivin
            TulostaPisteetE4(punainenPiste); //tulostaa punaisen pisteen tiedot
        }

        // BYCODEBEGIN
        /// <summary>
        /// Luodaan piste-olio, jolla on muitakin ominaisuuksia kuin vain
        /// sijainti vektorina. Tämä on todella hieno piste.
        /// </summary>
        /// <param name="vektori">pisteen sijaintitiedot vektorina</param>
        /// <returns>piste</returns>
        public static Piste LuoHienoPiste(Vector vektori)
        {
            Piste piste = new Piste(vektori);
            piste.nimi = "Todella hieno piste";

            return piste;
        }

        /// <summary>
        /// Luodaan piste-olio, joka on punainen.
        /// Pisteellä on muitakin ominaisuuksia kuin sijainti vektorina.
        /// </summary>
        /// <returns>piste</returns>
        public static Piste LuoPunainenPiste()
        {
            Piste pallero = new Piste(new Vector(0, 20));
            pallero.vari = "punainen";
            pallero.nimi = "Punainen pallero";

            return pallero;
        }


        /// <summary>
        /// Luodaan piste.
        /// TÄYDENNÄ
        /// </summary>
        /// <returns>piste</returns>
        public static Piste LuoPiste(/*LISÄÄ PARAMATRIT, JOS TARPEEN*/)
        {
            //TÄYDENNÄ
            return new Piste();
        }


        /// <summary>
        /// Tulostetaan halutun pisteen tiedot konsoliin.
        /// Tuloste on tyyliä: 
        /// "Pisteen nimi: Punainen pallero | Pisteen väri : punainen |  Pisteen vektori (0,0,20,0)"
        /// </summary>
        /// <param name="piste">tulostettava piste</param>

        public static void TulostaPisteetE4(Piste piste)
        {
            Console.WriteLine(piste);
        }


        //BYCODEEND
        /// <summary>
        /// Arvotaan joukko 2D-pisteitä halutulle alueelle
        /// </summary>
        /// <param name="n">montako pistettä arvotaan</param>
        /// <param name="x1">alueen vasemman alakulman x</param>
        /// <param name="y1">alueen vasemman alakulman y</param>
        /// <param name="x2">alueen oikean yläkulman x</param>
        /// <param name="y2">alueen oikean yläkulman y</param>
        /// <returns></returns>
        public static Vector[] ArvoPisteet(int n, double x1, double y1, double x2, double y2)
        {
            // Tee n-kokoinen taulukko Vector-olioista.
            // Arvo kullekin alkiolle sattunnainen x ja y koordinaatti
            // parametrina tuleviin rajoihin käyttäen RandomGen-luokkaa
            // Poista alla oleva koodi joka tekee aina 4 samaa pistettä
            // TODO: Tehtävä 2
            Vector[] pisteet = new Vector[n];
            for (int i = 0; i < pisteet.Length; i++)
            {
                double x = RandomGen.NextDouble(x1, x2);
                double y = RandomGen.NextDouble(y1, y2);
                pisteet[i] = new Vector(x, y);
            }
            return pisteet;
        }
    }
}

namespace MinJypeli
{
    /// <summary>
    /// Pieni luokka piste-olioiden luomista varten. 
    /// </summary>
    public class Piste
    {
        private Vector vektori;
        public String nimi = "";
        public String vari = "";


        /// <summary>
        /// Pisteen muodostaja.
        /// </summary>
        public Piste()
        {
            this.vektori = new Vector(0, 0);
        }


        /// <summary>
        /// Pisteen muodostaja.
        /// </summary>
        public Piste(Vector v)
        {
            this.vektori = v;
        }


        /// <summary>
        /// Pisteen muodostaja.
        /// </summary>
        public Piste(Vector v, String n)
        {
            this.vektori = v;
            this.nimi = n;
        }


        /// <summary>
        /// Pisteen muodostaja.
        /// </summary>
        public Piste(Vector v, String n, String vari)
        {
            this.vektori = v;
            this.nimi = n;
            this.vari = vari;
        }


        /// <summary>
        /// Aliohjelman olioin tietojen muuttamiseksi tekstiksi. 
        /// </summary>
        /// <returns>Olion kaikki tiedot merkkijonona.</returns>
        override
        public String ToString()
        {
            return "Pisteen nimi: " + this.nimi + " | Pisteen väri : " + this.vari + " | Pisteen vektori: " + this.vektori;
        }

    }
    /// <summary>
    /// Satunnaisgeneraattori. Luo satunnaisia arvoja, mm. lukuja, vektoreita sekä kulmia.
    /// </summary>
    public static class RandomGen
    {
        private static Random rand = new Random();

        /// <summary>
        /// Palauttaa satunnaisen liukuluvun parametrien <code>min</code> ja <code>max</code> väliltä.
        /// </summary>
        /// <param name="min">Pienin arvo jonka luku voi saada</param>
        /// <param name="max">Suurin arvo jonka luku voi saada</param>
        /// <returns>Satunnainen reaaliluku</returns>
        public static double NextDouble(double min, double max)
        {
            return min + rand.NextDouble() * (max - min);
        }


        /// <summary>
        /// Palauttaa satunnaisen vektorin.
        /// </summary>
        /// <param name="minX">Pienin sallittu X-koordinaatti</param>
        /// <param name="minY">Pienin sallittu Y-koordinaatti</param>
        /// <param name="maxX">Suurin sallittu X-koordinaatti</param>
        /// <param name="maxY">Suurin sallittu Y-koordinaatti</param>
        /// <returns>Satunnainen vektori</returns>
        public static Vector NextVector(double minX, double minY, double maxX, double maxY)
        {
            return new Vector(NextDouble(minX, maxX), NextDouble(minY, maxY));
        }
    }

    public struct Vector
    {
        /// <summary>
        /// Nollavektori.
        /// </summary>
        public static readonly Vector Zero = new Vector(0.0, 0.0);


        /// <summary>
        /// Etäisyys kahden pisteen välillä.
        /// </summary>
        public static double Distance(Vector p1, Vector p2)
        {
            double x, y;
            x = p1.X - p2.X;
            y = p1.Y - p2.Y;
            return Math.Sqrt((x * x + y * y));
        }

        public double X;
        public double Y;

        /// <summary>
        /// Vektorin pituus.
        /// </summary>
        public double Magnitude
        {
            get { return Math.Sqrt(MagnitudeSquared); }
        }

        /// <summary>
        /// Vektorin pituuden neliö.
        /// </summary>
        public double MagnitudeSquared
        {
            get { return X * X + Y * Y; }
        }

        /// <summary>
        /// Luo uuden vektorin komponenteista.
        /// </summary>
        /// <param name="X">X-komponentti</param>
        /// <param name="Y">Y-komponentti</param>
        public Vector(double X, double Y)
        {
            this.X = X;
            this.Y = Y;
        }


        /// <summary>
        /// Vektori merkkijonona, esim (2.3,4.1)
        /// </summary>
        /// <returns>lukupari merkkijonona</returns>
        public override string ToString()
        {
            return String.Format("({0:0.0},{1:0.0})", X, Y);
        }


        public static Vector Parse(string vectorStr)
        {
            if (vectorStr == null || vectorStr.Trim() == "") return new Vector(0, 0);

            string s = vectorStr.Replace("(", "").Replace(")", "").Replace(",", " ").Replace("  ", " ").Trim();
            string[] splitStr = s.Split(' ');

            if (splitStr.Length != 2)
                throw new FormatException("Not a vector string: " + vectorStr);

            double x = double.Parse(splitStr[0]);
            double y = double.Parse(splitStr[1]);

            return new Vector(x, y);
        }
    }
}
