﻿using System;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
// using Jypeli.Effects;
using Jypeli.Widgets;
using System.Collections.Generic;

using XnaRectangle = Microsoft.Xna.Framework.Rectangle;
using XnaV2 = Microsoft.Xna.Framework.Vector2;
using XnaColor = Microsoft.Xna.Framework.Color;
using JyColor = Jypeli.Color;

namespace Demo9
{
    /// @author  Vesa Lappalainen
    /// @version 30.10.2011
    /// <summary>
    /// Palapeli, jossa järjestellään ruutuja.
    /// </summary>
    public class Palapeli : PhysicsGame
    {
        private const int siirtymaNopeus = 500;
        private Vector ensimmaisenPalanPaikka = new Vector(-200, 150);

        private List<GameObject> palat;
        private GameObject odotuspaikalla = null;
        private Vector vapaapaikka;
        private Vector odotuspaikka;

        /// <summary>
        /// Aloitetaan palapeli.  Ladataan kuva, pilkotaan ja laitetaan
        /// palat ruudulle.
        /// </summary>
        public override void Begin()
        {
            // Image.SetLineCorrection(0); // Linux käyttäjät, ottakaa tämä kommenteista
            ClearAll();
            int nx = 5;
            int ny = 3;
            Image kuva = LoadImage("vesa");
            //Image kuva = LoadImage("numerot");

            List<Image> kuvat = LuoKuvat(kuva, nx, ny);
            RandomGen.Shuffle(kuvat);

            odotuspaikka = ensimmaisenPalanPaikka - new Vector(kuvat[0].Width + 40, 0);
            GameObject odotuspaikanMerkki = new GameObject(kuvat[0].Width, kuvat[0].Height);
            odotuspaikanMerkki.Position = odotuspaikka;
            odotuspaikanMerkki.Color = Color.LightGray;
            Add(odotuspaikanMerkki, -1);

            palat = LisaaPalatRuutuun(this, kuvat, nx, ny, ensimmaisenPalanPaikka);

            Camera.ZoomToAllObjects(100);
            AsetaOhjaimet();
        }



        /// <summary>
        /// Asetetaan ohjaimet varsinaista peliä varten
        /// </summary>
        private void AsetaOhjaimet()
        {
            Keyboard.Listen(Key.F1, ButtonState.Pressed, ShowControlHelp, "Show help");
            Keyboard.Listen(Key.F5, ButtonState.Pressed, Begin, "New game");
            Keyboard.Listen(Key.Escape, ButtonState.Pressed, Exit, "Exit game");

            Mouse.IsCursorVisible = true;
            Mouse.Listen(MouseButton.Left, ButtonState.Pressed, delegate () { ValitseKuva(Mouse.PositionOnWorld); }, "Valitse kuva");
        }


        /// <summary>
        /// Valitaan p:n kohdalla oleva kuva.
        /// Jos kukaan ei ole odotuspaikalla, siirretään valittu kuva sinne.
        /// Muuten siirretään valittu kuva vapaaseen paikkaan ja odotuspaikalla
        /// oleva kuva valitun kohdalle.
        /// </summary>
        /// <param name="p">Paikka jonka kohdalla oleva kuvan pala valitaan siirrettäväksi</param>
        private void ValitseKuva(Vector p)
        {
            GameObject klikattuPalanen = EtsiPala(palat, p);
            if (klikattuPalanen == null) return;

            Remove(klikattuPalanen); Add(klikattuPalanen, 1);  // siirretään 1-tasolle kulkemaan
            if (odotuspaikalla == null)
            { // siirretään nyt klikattu pala odotuspaikkaan
                odotuspaikalla = klikattuPalanen;
                vapaapaikka = klikattuPalanen.Position;
                klikattuPalanen.MoveTo(odotuspaikka, siirtymaNopeus);
                return;
            }


            if (odotuspaikalla != klikattuPalanen)
            { // siirretään odotuspaikkalla oleva pala nyt klikatun paikalle
                Vector p2 = klikattuPalanen.Position;
                odotuspaikalla.MoveTo(p2, siirtymaNopeus);
            }
            klikattuPalanen.MoveTo(vapaapaikka, siirtymaNopeus); // nyt klikattu siirretään vapaaseen paikkaan
            odotuspaikalla = null;
        }


        /// <summary>
        /// Etsitään pala, joka on pisteen p kohdalla.  Jos moni pala sattuu kohdalle,
        /// palautetaan 1. löytynyt.
        /// </summary>
        /// <param name="palat">palojen lista, josta etsitään</param>
        /// <param name="p">paikka jota tutkitaan</param>
        /// <returns>löytynyt pala tai null jos ei löydy</returns>
        public static GameObject EtsiPala(IEnumerable<GameObject> palat, Vector p)
        {
            foreach (GameObject palanen in palat)
                if (palanen.IsInside(p)) return palanen;
            return null;
        }


        /// <summary>
        /// Lisätään listassa olevat palat näyttöön.
        /// Palat otetaan listasta niin, että nx ensimmäistä palaa lisätään
        /// 1. riville alkaen paikasta (x,y), aina kuvanpalan leveyden verran seuraavaan paikkaan.
        /// nx seuraavaa palaa seuraavalle riville alkaen kuvan kuvan palaan korkeuden alempaa.
        /// jne. ny kertaa.
        /// Listassa on oltava nx*ny palaa.
        /// Lisätyt palat lisätään myös GameObject-listaan, joka lopuksi palautetaan.
        /// Jos listassa olevat palat olisivat
        ///   0 1 2 3 4 5 6 7 8 9 10 11
        /// niin siitä tulisi kuva  
        /// <example>
        ///  0  1  2  3
        ///  4  5  6  7
        ///  8  9 10 11
        /// </example>
        /// </summary>
        /// <param name="peli">mihin peliin lisätään</param>
        /// <param name="kuvat">kuvalista</param>
        /// <param name="nx">kuvien määrä x-suunnassa</param>
        /// <param name="ny">kuvien määrä y-suunnassa</param>
        /// <param name="aloitus">ensimmäisen kuvan keskipisteen koordinaatti</param>
        /// <returns>lista luoduista olioista</returns>
        public static List<GameObject> LisaaPalatRuutuun(Game peli, List<Image> kuvat, int nx, int ny, Vector aloitus)
        {

            /// TODO: Tehtävä 6-7: Korjaa kunnolla toimivaksi, nyt laittaa vain kolme palaa.
            List<GameObject> palat = new List<GameObject>();
            GameObject palanen;

            palanen = new GameObject(kuvat[0]);
            palanen.Position = aloitus;
            palat.Add(palanen);
            peli.Add(palanen);

            palanen = new GameObject(kuvat[1]);
            palanen.Position = aloitus + new Vector(palanen.Width, 0);
            palat.Add(palanen);
            peli.Add(palanen);

            palanen = new GameObject(kuvat[2]);
            palanen.Position = aloitus + new Vector(2 * palanen.Width, 0);
            palat.Add(palanen);
            peli.Add(palanen);

            return palat;
        }


        /// <summary>
        /// Luodaan kuvasta lista kuvan palasia.
        /// Eli kuva jaetaan ny*nx osaan.  Ensimmäiseksi listaan
        /// laitetaan nx kpl 1. rivin palasia, sitten nx kpl seuraavan
        /// rivin jne... Jos kuva olisi
        /// <example>
        ///  0  1  2  3
        ///  4  5  6  7
        ///  8  9 10 11
        /// </example>
        /// niin palat laitettaisiin listaan järjestyksessä
        ///   0 1 2 3 4 5 6 7 8 9 10 11
        /// </summary>
        /// <param name="kuva">kuva joka pilkotaan</param>
        /// <param name="nx">moneenko osaan x-suunnassa</param>
        /// <param name="ny">moneenko osaan y-suunnassa</param>
        /// <returns>Lista luoduista kuvista</returns>
        public static List<Image> LuoKuvat(Image kuva, int nx, int ny)
        {
            // TODO: Tehtävä 6-7:  korjaa tämä niin että tulee ny * nx palasta
            // nyt tehdään vain kolmea palasta (1x3)
            int h = kuva.Height / 1;
            int w = kuva.Width / 3;
            List<Image> kuvat = new List<Image>();
            Image kuvanpala;
            uint[,] intpala;

            kuvanpala = new Image(w, h, Color.Black);
            intpala = kuva.GetDataUInt(0, 0, w, h);
            kuvanpala.SetData(intpala);
            kuvat.Add(kuvanpala);

            kuvanpala = new Image(w, h, Color.Black);
            intpala = kuva.GetDataUInt(w, 0, w, h);
            kuvanpala.SetData(intpala);
            kuvat.Add(kuvanpala);

            kuvanpala = new Image(w, h, Color.Black);
            intpala = kuva.GetDataUInt(2 * w, 0, w, h);
            kuvanpala.SetData(intpala);
            kuvat.Add(kuvanpala);

            return kuvat;
        }

    }
}