using System;
using System.Collections.Generic;
using MinJypeli;

namespace Demo9
{
    public class PortaatYlosAlas
    {
        public static List<Shape> layer = new List<Shape> { };
        public static double LAATIKON_KOKO = 40.0;

        /// BYCODEBEGIN
        public static void Main()
        {
            Vector piste = new Vector(0, 0);
            piste = PiirraLaatikko(piste);
            piste = PiirraLaatikko(piste);
            piste = PiirraLaatikko(piste);
            piste -= new Vector(0, LAATIKON_KOKO);
            piste = PiirraLaatikkoAlas(piste);
            piste = PiirraLaatikkoAlas(piste);

            Console.WriteLine("Laatikoiden vasemmat alakulmat ja oikeat yläkulmat:");
            Print(layer);
        }

        // TODO: Kopioi tähän edellisen tehtävän aliohjelma PiirraLaatikko
        // TODO: Määrittele uusi aliohjelma PiirraLaatikkoAlas, jotta ohjelma toimii
        // TODO: Täysiä pisteitä varten määrittele myös aliohjelma PiirraPortaat,
        // jolle viedään parametrina, mistä pisteestä portaiden luominen aloitetaan
        // ja montako porrasta tulee ylös ja montako alas.

        /// BYCODEEND

        /// <summary>
        ///  Tulostaa kunkin muodon vasemman alakulman koordinaatit ja oikean yläkulman koordinaatit
        /// </summary>
        public static void Print(IEnumerable<Shape> list)
        {
            foreach (Shape s in list)
            {
                Console.WriteLine($"({s.Left}, {s.Bottom}), ({s.Right}, {s.Top})");
            }
        }
    }
}

/// @author  Vesa Lappalainen
/// @author  Enni Stylman
/// @author  Aleksi Tarvainen
/// @version 3.11.2019
namespace MinJypeli
{

    /// <summary>
    /// Kantaluokka kaksiulotteisille muodoille
    /// </summary>
    public abstract class Shape
    {
        public Vector Position = Vector.Zero;
        public double X
        {
            get { return Position.X; }
            set { Position = new Vector(value, Y); }
        }

        public double Y
        {
            get { return Position.Y; }
            set { Position = new Vector(X, value); }
        }

        public virtual double Left { get; }
        public virtual double Right { get; }
        public virtual double Top { get; }
        public virtual double Bottom { get; }
    }

    /// <summary>
    /// Tavallinen suorakulmio
    /// </summary>
    public class Rectangle : Shape
    {
        public double Width;
        public double Height;
        public Rectangle(double width, double height, Vector pos = new Vector())
        {
            Width = width;
            Height = height;
            Position = pos;
        }

        public override double Left { get { return X - Width / 2; } }
        public override double Right { get { return X + Width / 2; } }
        public override double Top { get { return Y + Height / 2; } }
        public override double Bottom { get { return Y - Height / 2; } }
    }

    /// <summary>
    /// Satunnaisgeneraattori. Luo satunnaisia arvoja, mm. lukuja, vektoreita sekä kulmia.
    /// </summary>
    public static class RandomGen
    {
        private static Random rand = new Random();

        /// <summary>
        /// Palauttaa satunnaisen liukuluvun parametrien <code>min</code> ja <code>max</code> väliltä.
        /// </summary>
        /// <param name="min">Pienin arvo jonka luku voi saada</param>
        /// <param name="max">Suurin arvo jonka luku voi saada</param>
        /// <returns>Satunnainen reaaliluku</returns>
        public static double NextDouble(double min, double max)
        {
            return min + rand.NextDouble() * (max - min);
        }


        /// <summary>
        /// Palauttaa satunnaisen vektorin.
        /// </summary>
        /// <param name="minX">Pienin sallittu X-koordinaatti</param>
        /// <param name="minY">Pienin sallittu Y-koordinaatti</param>
        /// <param name="maxX">Suurin sallittu X-koordinaatti</param>
        /// <param name="maxY">Suurin sallittu Y-koordinaatti</param>
        /// <returns>Satunnainen vektori</returns>
        public static Vector NextVector(double minX, double minY, double maxX, double maxY)
        {
            return new Vector(NextDouble(minX, maxX), NextDouble(minY, maxY));
        }
    }

    public struct Vector
    {
        /// <summary>
        /// Nollavektori.
        /// </summary>
        public static readonly Vector Zero = new Vector(0.0, 0.0);


        /// <summary>
        /// Etäisyys kahden pisteen välillä.
        /// </summary>
        public static double Distance(Vector p1, Vector p2)
        {
            double x, y;
            x = p1.X - p2.X;
            y = p1.Y - p2.Y;
            return Math.Sqrt((x * x + y * y));
        }

        public double X;
        public double Y;

        /// <summary>
        /// Vektorin pituus.
        /// </summary>
        public double Magnitude
        {
            get { return Math.Sqrt(MagnitudeSquared); }
        }

        /// <summary>
        /// Vektorin pituuden neliö.
        /// </summary>
        public double MagnitudeSquared
        {
            get { return X * X + Y * Y; }
        }

        /// <summary>
        /// Luo uuden vektorin komponenteista.
        /// </summary>
        /// <param name="X">X-komponentti</param>
        /// <param name="Y">Y-komponentti</param>
        public Vector(double X, double Y)
        {
            this.X = X;
            this.Y = Y;
        }


        /// <summary>
        /// Vektori merkkijonona, esim (2.3,4.1)
        /// </summary>
        /// <returns>lukupari merkkijonona</returns>
        public override string ToString()
        {
            return String.Format("({0:0.0},{1:0.0})", X, Y);
        }


        public static Vector Parse(string vectorStr)
        {
            if (vectorStr == null || vectorStr.Trim() == "") return new Vector(0, 0);

            string s = vectorStr.Replace("(", "").Replace(")", "").Replace(",", " ").Replace("  ", " ").Trim();
            string[] splitStr = s.Split(' ');

            if (splitStr.Length != 2)
                throw new FormatException("Not a vector string: " + vectorStr);

            double x = double.Parse(splitStr[0]);
            double y = double.Parse(splitStr[1]);

            return new Vector(x, y);
        }


        /// <summary>
        /// Vektorien yhteenlasku
        /// </summary>
        /// <param name="a">Vasen yhteenlaskettava vektori</param>
        /// <param name="b">Oikea yhteenlaskettava vektori</param>
        /// <returns>Yhteenlaskettu vektori</returns>
        public static Vector operator +(Vector a, Vector b)
        {
            return new Vector(a.X + b.X, a.Y + b.Y);
        }


        /// <summary>
        /// Vektorien erotus
        /// </summary>
        /// <param name="a">Vasen erotuksen vektori</param>
        /// <param name="b">Oikea erotuksen vektori</param>
        /// <returns>Vektorien erotus</returns>
        public static Vector operator -(Vector a, Vector b)
        {
            return new Vector(a.X - b.X, a.Y - b.Y);
        }
    }
}