// Otetaan käyttöön Jyväskylän yliopiston Jypeli-kirjasto
using Jypeli;
using System;

/// @author  Vesa Lappalainen, Antti-Jussi Lakanen
/// @version 22.12.2011
///
///
/// <summary>
/// Luokka, jossa harjoitellaan piirtämistä lisäämällä ympyröitä ruudulle
/// </summary>
public class LumiukkoAli : PhysicsGame
{

    /// <summary>
    /// Piirretään oliot ja zoomataan kamera niin että kenttä näkyy kokonaan.
    /// </summary>
    public override void Begin()
    {
        Camera.ZoomToLevel();
        Level.Background.Color = Color.Black;

        PiirraLumiukko(this, -200, Level.Bottom + 200.0);
        PiirraLumiukko(this, 200, Level.Bottom + 200.0);
        // Gravity = new Vector(0, -200);
    }


    /// <summary>
    /// Aliohjelma piirtää lumiukon
    /// annettuun paikkaan.
    /// </summary>
    /// <param name="peli">Peli, johon lumiukko tehdään.</param>
    /// <param name="x">Lumiukon alimman pallon x-koordinaatti.</param>
    /// <param name="y">Lumiukon alimman pallon y-koordinaatti.</param>
    public static void PiirraLumiukko(PhysicsGame peli, double x, double y)
    {
        double r1 = 200;
        double r2 = 0.5*r1;
        double r3 = 0.3*r1;

        PhysicsObject p1 = PiirraPallo(peli, x, y, r1);
        PhysicsObject p2 = PiirraPallo(peli, x, p1.Y + r1 + r2,  r2);
        PhysicsObject p3 = PiirraPallo(peli, x, p2.Y + r2 + r3, r3);
    }


    /// <summary>
    /// Piirtää yhden pallon haluttuun peliin
    /// </summary>
    /// <param name="peli">pali johon piirretään</param>
    /// <param name="x">x-koordinaatti</param>
    /// <param name="y">y-koordinaatti</param>
    /// <param name="r">pallon säde</param>
    /// <returns>viitteen piirrettyyn palloon</returns>
    public static PhysicsObject PiirraPallo(PhysicsGame peli, double x, double y, double r)
    {
        PhysicsObject p = new PhysicsObject(2 * r, 2 * r, Shape.Circle);
        p.X = x;
        p.Y = y;
        peli.Add(p);
        return p;  
    }

}