using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Effects;
using Jypeli.Widgets;

/// @author vesal
/// @version 16.10.2018
/// <summary>
/// Piirretään 4 lumiukkoa
/// </summary>
public class LumiukotTaulukolla : PhysicsGame
{
    /// <summary>
    /// Kolme lumiukkoa
    /// </summary>
    // BYCODEBEGIN
    public override void Begin()
    {
        PhysicsObject[] u1;
        PhysicsObject[] u3;

        Level.Size = Screen.Size;
        Level.Background.Color = Color.Black;


        PiirraLumiukko(this, -100, Level.Bottom + 200.0);
        u1 = PiirraLumiukko(this, 200, Level.Bottom + 400.0);
        u1[0].Color = Color.Yellow;
        u1[1].Color = Color.Red;
        PiirraLumiukko(this, -200, Level.Bottom + 600, 40);
        u3 = PiirraLumiukko(this, 400, Level.Bottom + 600, 10);

        Level.CreateBorders();
        //  Gravity = new Vector(10, -400);

        PhoneBackButton.Listen(ConfirmExit, "Lopeta peli");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
        Keyboard.Listen(Key.F1, ButtonState.Pressed, ShowControlHelp, "Avustus");

        Keyboard.Listen(Key.Left, ButtonState.Pressed, LyoPalloa, "Lyö päätä vasemmalle", u1[2], new Vector(-1000, 0));
        Keyboard.Listen(Key.Up, ButtonState.Pressed, LyoPalloa, "Lyö keskipalloa ylöspäin", u3[1], new Vector(0, 1000));
    }



    /// <summary>
    /// Piirretään yksi lumiukko, jonka "origo" on alapallon keskipisteessä
    /// </summary>
    /// <param name="peli">peli johon lumiukko lisätään</param>
    /// <param name="x">alapallon keskipisteen x</param>
    /// <param name="y">alapallon keskipisteen y</param>
    /// <param name="r">alapallon säde</param>
    /// <returns>pallot taulukossa</returns>
    public static PhysicsObject[] PiirraLumiukko(PhysicsGame peli, double x, double y, double r)
    {
        double r1 = r;
        double r2 = r1 / 2;
        double r3 = r1 * 0.3;
        PhysicsObject[] p = new PhysicsObject[3]; // int t = new int[3];
        p[0] = PiirraPallo(peli, x, y, r1);
        p[1] = PiirraPallo(peli, x, p[0].Y + r1 + r2, r2);
        p[2] = PiirraPallo(peli, x, p[1].Y + r2 + r3, r3);
        return p;
    }
    // BYCODEEND


    /// <summary>
    /// Piirretään yksi lumiukko, jonka "origo" on alapallon keskipisteessä
    /// </summary>
    /// <param name="peli">peli johon lumiukko lisätään</param>
    /// <param name="x">alapallon keskipisteen x</param>
    /// <param name="y">alapallon keskipisteen y</param>
    /// <returns>pallot taulukossa</returns>
    public static PhysicsObject[] PiirraLumiukko(PhysicsGame peli, double x, double y)
    {
        return PiirraLumiukko(peli, x, y, 100);
    }


    /// <summary>
    /// Piirretään yksi pallo peliin, jonka keskipiste on (x,y) ja säde r
    /// </summary>
    /// <param name="peli">peli johon piirretään</param>
    /// <param name="x">pallon x-koordinaatti</param>
    /// <param name="y">pallon y-koordinaatti</param>
    /// <param name="r">pallon säde</param>
    /// <returns>lisätty pallo</returns>
    public static PhysicsObject PiirraPallo(PhysicsGame peli, double x, double y, double r)
    {
        PhysicsObject pallo = new PhysicsObject(2 * r, 2 * r, Shape.Circle);
        pallo.X = x;
        pallo.Y = y;
        peli.Add(pallo);
        return pallo;
    }


    /// <summary>
    /// Lyö valittua palloa haluutuun suuntaan
    /// </summary>
    /// <param name="pallo">pallo jota lyödään</param>
    /// <param name="suunta">suuntavektori lyönnille</param>
    public static void LyoPalloa(PhysicsObject pallo, Vector suunta)
    {
        pallo.Hit(suunta);
    }
}


