using System;

/// @author  Vesa Lappalainen
/// @version 10.10.2016
///
/// <summary>
/// Esimerkkejä matriisista
/// </summary>
public class Matriisit
{
    /// <summary>
    /// Esimerkkejä matriiseista
    /// </summary>
    /// <param name="args">ei käytössä</param>
    public static void Main(string[] args)
    {
        // int[] luvut = int[3];
        int[] luvut = { 3, 5, 1, 9 };

        int[,] matriisi1 = new int[6, 5];

        int[,] matriisi = {
                                  {1,2,3},
                                  {4,5,6},
                                  {7,8,9},
                                  {10,11,12}
                              };

        Tulosta(matriisi);

        Console.WriteLine("Dimensio: " + matriisi.Rank);
        Console.WriteLine("Rivejä: " + matriisi.GetLength(0));
        Console.WriteLine("Sarakkeita: " + matriisi.GetLength(1));
        Console.WriteLine("Paikassa 2,1: " + matriisi[2, 1]);

        int rivi = MiltaRiviltaLoytyy(matriisi, 8);
        Console.WriteLine(rivi);

        matriisi1[4, 2] = 99;

        Tulosta(matriisi1);

        int[,] talo = {
                        { 2, 3, 1, 4 },
                        { 1, 3, 1, 2 },
                        { 0, 2, 2, 3 },
                        { 2, 5, 1, 0 },
                        { 2, 2, 2, 2 },
                      };

        Tulosta(talo);

        int kerros = MiltaRiviltaLoytyy(talo, 5);
        Console.WriteLine(kerros);

        int asukkaita = Summa(talo);
        Console.WriteLine(asukkaita);
    }


    /// <summary>
    /// Etsitään miltä riviltä löytyy etsittävä luku
    /// </summary>
    /// <param name="mat">Matriisi josta etsitään</param>
    /// <param name="etsittava">luku jota etsitään</param>
    /// <returns>rivi, jolta luku löytyy.  Jos usealla rivillä, palautetaan 1. indeksi.
    /// jos ei löydy lainkaan, palautetaan -1.</returns>
    /// <example>
    /// <pre name="test">
    /// int[,] m = {{2,4,1},{9,2,0},{5,6,1},{0,12,3}};
    /// MiltaRiviltaLoytyy(m,1) === 0;
    /// MiltaRiviltaLoytyy(m,0) === 1;
    /// MiltaRiviltaLoytyy(m,12) === 3;
    /// MiltaRiviltaLoytyy(m,11) === -1;
    /// </pre>
    /// </example>
    public static int MiltaRiviltaLoytyy(int[,] mat, int etsittava)
    {
        int riveja = mat.GetLength(0);
        int sarkkeita = mat.GetLength(1);

        for (int iy = 0; iy < riveja; iy++)
        {
            for (int ix = 0; ix < sarkkeita; ix++)
            {
                int luku = mat[iy, ix];
                if (luku == etsittava)
                    return iy;
            }
        }
        return -1;
    }


    /// <summary>
    /// Lasketaan matriisin kaikkien alkioiden summa
    /// </summary>
    /// <param name="mat">Matriisi jonka summa lasketaan</param>
    /// <returns>alkioiden summa</returns>
    /// <example>
    /// <pre name="test">
    /// int[,] m = {{2,4,1},{9,2,0},{5,6,1},{0,12,3}};
    /// Summa(m) === 45;
    /// Summa(new int[,]{{1,2},{3,4}}) === 10;
    /// Summa(new int[,]{{1,2}}) === 3;
    /// Summa(new int[,]{{2},{4}}) === 6;
    /// Summa(new int[,]{{9}}) === 9;
    /// Summa(new int[,]{{}}) === 0;
    /// </pre>
    /// </example>
    public static int Summa(int[,] mat)
    {
        int riveja = mat.GetLength(0);
        int sarkkeita = mat.GetLength(1);
        int summa = 0;

        for (int iy = 0; iy < riveja; iy++)
        {
            for (int ix = 0; ix < sarkkeita; ix++)
            {
                int luku = mat[iy, ix];
                summa += luku;
            }
        }
        return summa;
    }


    /// <summary>
    /// Tulostetaan matriisin alkiot riveittäin.
    /// Kukin alkio vie kahden merkin kokoisen tilan.
    /// </summary>
    /// <param name="mat">tulostettava matriisi</param>
    public static void Tulosta(int[,] mat)
    {
        int riveja = mat.GetLength(0);
        int sarkkeita = mat.GetLength(1);

        for (int iy = 0; iy < riveja; iy++)
        {
            for (int ix = 0; ix < sarkkeita; ix++)
            {
                int luku = mat[iy, ix];
                Console.Write("{0,2} ", luku);
            }
            Console.WriteLine();
        }
        Console.WriteLine();
    }

}