using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Widgets;
using System;
using System.Collections.Generic;

/// @author  Vesa Lappalainen
/// @version 18.10.2021
/// <summary>
/// Peli, jossa vihaisia Legoja tiputellaan toisten päälle
/// </summary>
public class AngryLego : PhysicsGame
{
    private static readonly String[] lines = {
                  "                        ",
                  "                        ",
                  "                        ",
                  "                        ",
                  "/                       ",
                  "                        ",
                  "                        ",
                  "                        ",
                  "                        ",
                  "                        ",
                  "                        ",
                  "                        ",
                  "                        ",
                  "                        ",
                  "                        ",
                  "                        ",
                  "                        ",
                  "        =======         ",
                  "        Y* Y  Y         ",
                  "        X  X  X         ",
                  "   *    X  X  X     *   ",
                  "        X  X  X         ",
                  "        X *X *X         ",
                  };

    private static readonly int tileWidth = 800 / lines[0].Length;
    private static readonly int tileHeight = 480 / lines.Length;


    /// <summary>
    /// Aloitetaan peli. Aluksi siivotaan kaikki, jotta voidaan aloittaa uusi peli
    /// </summary>
    public override void Begin()
    {
        Level.Background.CreateGradient(Color.Blue, Color.White);
        TileMap tiles = TileMap.FromStringArray(lines);

        Color oletus = Color.Yellow;

        tiles.SetTileMethod('X', LuoSeina, Color.Wheat);
        tiles.SetTileMethod('Y', LuoSeina, Color.Wheat);
        tiles.SetTileMethod('=', LuoKatto, Color.Red);
        tiles.SetTileMethod('/', LuoMaila, Color.Black);
        tiles.SetTileMethod('*', LuoVihollinen, Color.Pink);

        tiles.Execute(tileWidth, tileHeight);

        PhoneBackButton.Listen(ConfirmExit, "Lopeta peli");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
    }


    /// <summary>
    /// Luodaan seinäelementti
    /// </summary>
    private void LuoSeina(Vector paikka, double leveys, double korkeus, Color vari)
    {
        PhysicsObject seina = new PhysicsObject(leveys, korkeus);
        seina.Position = paikka;
        seina.Color = vari;
        seina.Tag = "rakenne";
        Add(seina);
    }


    /// <summary>
    /// Luodaan kattoelementti.  Luodaan hieman ylisuureksi, jolloin liimautuu
    /// naapuriin kiinni.
    /// </summary>
    private void LuoKatto(Vector paikka, double leveys, double korkeus, Color vari)
    {
        PhysicsObject katto = new PhysicsObject(leveys, korkeus);
        katto.Position = paikka;
        katto.Color = vari;
        katto.Tag = "rakenne";
        Add(katto);
    }


    /// <summary>
    /// Luodaan maila, jolla palloja lyödään
    /// </summary>
    private void LuoMaila(Vector paikka, double leveys, double korkeus, Color vari)
    {
        PhysicsObject maila = PhysicsObject.CreateStaticObject(leveys * 6, korkeus);
        maila.Position = paikka;
        maila.Color = vari;
       // maila.Image = LoadImage("maila3");
        Add(maila);
    }

    /// <summary>
    /// Luodaan vihollinen, joka hajoaa osuessaan rekenteeseen
    /// </summary>
    private void LuoVihollinen(Vector paikka, double leveys, double korkeus, Color vari)
    {
        PhysicsObject vihu = new PhysicsObject(leveys / 2, leveys / 2, Shape.Circle);
        vihu.Position = paikka;
        vihu.Color = vari;
        // vihu.Collided += new NewCollisionHandler(VihuunOsui);
        //AddCollisionHandler(vihu, "rakenne", VihuunOsui);
        vihu.Tag = "vihu";
        // vihu.Image = LoadImage("Baby");
        Add(vihu);
    }


}

