﻿using Jypeli;

/// @author  Vesa Lappalainen
/// @version 09.10.2011
///
/// <summary>
/// Kaksi lumiukkoa aliohjelman avulla
/// </summary>
public class LumiukkoAli : PhysicsGame
{
    /// <summary>
    /// Aliohjelmassa piirretään ja zoomataan kamera siten, 
    /// että kenttä näkyy hyvin ruudulla.
    /// </summary>
    public override void Begin()
    {
        Level.Size = Screen.Size;
        Level.Background.Color = Color.Black;

        PiirraLumiukko(this, 0, Level.Bottom + 200.0);
        PiirraLumiukko(this, 200.0, Level.Bottom + 300.0);

        Level.CreateBorders();
        Gravity = new Vector(10, -200);

        PhoneBackButton.Listen(ConfirmExit, "Lopeta peli");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
    }


    /// <summary>
    /// Aliohjelma piirtää lumiukon
    /// annettuun paikkaan.
    /// </summary>
    /// <param name="peli">Peli, johon lumiukko tehdään.</param>
    /// <param name="x">Lumiukon alimman pallon x-koordinaatti.</param>
    /// <param name="y">Lumiukon alimman pallon y-koordinaatti.</param>
    public static void PiirraLumiukko(PhysicsGame peli, double x, double y)
    {
        PhysicsObject p1, p2, p3;
        double r1 = 100;
        double r2 = r1 * 0.50;
        double r3 = r1 * 0.30;

        p1 = PiirraPallo(peli, x, y, r1);
        p2 = PiirraPallo(peli, x, p1.Y + r1 + r2, r2);
        p3 = PiirraPallo(peli, x, p2.Y + r2 + r3, r3);
    }


    /// <summary>
    /// Piirretään yksi pallo, jolle annetaan keskipiste ja säde
    /// </summary>
    /// <param name="peli">Peli, johon pallo piirretään</param>
    /// <param name="x">pallon keksipisteen x</param>
    /// <param name="y">pallon keskipisteen y</param>
    /// <param name="r">pallon säde</param>
    /// <returns>lisätty pallo</returns>
    public static PhysicsObject PiirraPallo(PhysicsGame peli, double x, double y, double r)
    {
        PhysicsObject p = new PhysicsObject(2 * r, 2 * r, Shape.Circle);
        p.X = x;
        p.Y = y;
        peli.Add(p);
        return p;
    }
}