using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Widgets;

/// @author vesal
/// @version 25.9.2020
/// <summary>
///  Peli jossa metsästetään pienempiä kolmioita ja lihotaan
/// </summary>
public class KolmionMetsastys : PhysicsGame
{
    /// <summary>
    /// Alustetaan peli
    /// </summary>
    public override void Begin()
    {
        MasterVolume = 0.1;
        Level.Background.Color = Color.Black;
        BoundingRectangle alaosa = new BoundingRectangle(new Vector(Level.Left, 0), Level.BoundingRect.BottomRight);
        BoundingRectangle ylaosa = new BoundingRectangle(Level.BoundingRect.TopLeft, new Vector(Level.Right, 0));

        Level.CreateBorders();

        for(int i = 0; i < 50; i++) 
            LuoSattunnainenKolmio(this, alaosa, 60, "vihu");

        PhysicsObject pelaaja = LuoSattunnainenKolmio(this, ylaosa, 50, "pelaaja1");
        pelaaja.Color = Color.Red;
        pelaaja.Image = LoadImage("leijona"); // Lisää leijona.png Content-hakemistoon ja vaihda sille Copy if newer 

        AddCollisionHandler(pelaaja, "vihu", PelaajaTormasi);

        Keyboard.Listen(Key.F1, ButtonState.Pressed, ShowControlHelp, "Näytä avustus");
        Keyboard.Listen(Key.Up, ButtonState.Pressed, LyoKolmiota, "Pelaaja ylös", pelaaja, new Vector(0, 200));
        Keyboard.Listen(Key.Down, ButtonState.Pressed, LyoKolmiota, "Pelaaja alas", pelaaja, new Vector(0, -200));
        Keyboard.Listen(Key.Left, ButtonState.Pressed, LyoKolmiota, "Pelaaja vasemmalle", pelaaja, new Vector(-200, 0));
        Keyboard.Listen(Key.Right, ButtonState.Pressed, LyoKolmiota, "Pelaaja oikealle", pelaaja, new Vector(200, 0));

        // Voidaan lisätä esim XBox-ohjain: 
        ControllerOne.Listen(Button.DPadUp, ButtonState.Down, LyoKolmiota, "Pelaaja ylös", pelaaja, new Vector(0, 50));
        ControllerOne.Listen(Button.DPadDown, ButtonState.Down, LyoKolmiota, "Pelaaja alas", pelaaja, new Vector(0, -50));
        ControllerOne.Listen(Button.DPadLeft, ButtonState.Down, LyoKolmiota, "Pelaaja vasemmalle", pelaaja, new Vector(-50, 0));
        ControllerOne.Listen(Button.DPadRight, ButtonState.Down, LyoKolmiota, "Pelaaja oikealle", pelaaja, new Vector(50, 0));

        PhoneBackButton.Listen(ConfirmExit, "Lopeta peli");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
    }


    /// <summary>
    /// Metodi johon tullaan kun pelaaja ja joku kolmio törmäävät.
    /// Mikäli pelaajan ala on suurempi kuin vihollisen ala,
    /// luodaan vihollisen kohdalla räjähdys ja poistetaan vihollinen pelistä
    /// sekä siirretään vihollisen ala pelaajalle.
    /// Muuten lopetetaan peli.  TODO: keksittävä parempi lopetus
    /// </summary>
    /// <param name="pelaaja">pelaaja joka törmäsi</param>
    /// <param name="vihu">vihollinen johon pelaaja törmäsi</param>
    private void PelaajaTormasi(PhysicsObject pelaaja, PhysicsObject vihu)
    {
        double pelaajanAla = KolmionAla(pelaaja.Width, pelaaja.Height);
        double vihunAla = KolmionAla(vihu.Width, vihu.Height);

        if (pelaajanAla > vihunAla)
        {
            Explosion rajahdys = new Explosion(vihu.Width * 2);
            rajahdys.Position = vihu.Position;
            rajahdys.UseShockWave = false; // kannattaa kokeilla mitä taphatuu true -arvolla. 
            this.Add(rajahdys);
            Remove(vihu);
            double kasvukerroin = Math.Sqrt((pelaajanAla + vihunAla) / pelaajanAla);
            pelaaja.Width *= kasvukerroin;
            pelaaja.Height *= kasvukerroin;
            return;
        }

        Exit(); // TODO: lisää parempi loppu 
    }


    /// <summary>
    /// Lyödään kolmiota voimavektorilla
    /// </summary>
    /// <param name="kolmio">lyötävä kolmio</param>
    /// <param name="suunta">voimavektori</param>
    public static void LyoKolmiota(PhysicsObject kolmio, Vector suunta)
    { 
        kolmio.Hit(kolmio.Mass * suunta);
    }


    /// <summary>
    /// Luodaan satunnainen kolmio suorakaiteen sisälle. Myös nopeus, väri ja kulma arvotaan.
    /// </summary>
    /// <param name="peli">Mihin peliin kolmio luodaan</param>
    /// <param name="rect">Minkä suorakaiteen sisälle luodaan</param>
    /// <param name="vauhti">Kolmion vauhti alkutilanteessa</param>
    /// <param name="tunniste">Kolmiolle annettava tunniste</param>
    /// <returns>Viite luotuun kolmioon</returns>
    public static PhysicsObject LuoSattunnainenKolmio(PhysicsGame peli, BoundingRectangle rect, double vauhti, string tunniste)
    {
        double leveys = RandomGen.NextDouble(10, 100);
        double korkeus = RandomGen.NextDouble(10, 100);
        PhysicsObject kolmio = new PhysicsObject(leveys, korkeus, Shape.Triangle);
        kolmio.Position = RandomGen.NextVector(rect);
        kolmio.Angle = RandomGen.NextAngle();
        kolmio.Color = RandomGen.NextColor();
        Vector suunta = RandomGen.NextVector(0, vauhti);
        kolmio.Hit(suunta);
        kolmio.Tag = tunniste;
        peli.Add(kolmio);
        return kolmio;
    }


    /// <summary>
    /// Laskee kolmion alan
    /// </summary>
    /// <param name="kanta">kolmion kanta</param>
    /// <param name="korkeus">kolmion korkeus</param>
    /// <returns>kolmion ala</returns>
    /// <example>
    /// <pre name="test">
    ///    KolmionAla(0,0) ~~~ 0;
    ///    KolmionAla(1,0) ~~~ 0;
    ///    KolmionAla(1,1) ~~~ 0.5;
    ///    KolmionAla(3,5) ~~~ 7.5;
    /// </pre>
    /// </example>
    public static double KolmionAla(double kanta, double korkeus)
    {
        return kanta * korkeus * 1 / 2;
    }
}

