﻿using System;
using System.Text;
using System.Linq;
using System.Collections.Generic;
using System.Diagnostics;

/// @author vesal
/// @version 11.10.2014
/// <summary>
/// Tutkitaan onko kalliimpaa kutsua kaksi kertaa silmukkaa vai yhden kerran
/// 
/// int KOKO = 100*1000*1000;
///                                   debug                    release
///  ---------------------------------------------------------------------------------------------------------------   
///     kone                        1 kutsu   2 kutsua       1 kutsu   2 kutsua  1 ei if   LinQ Avg   LinQ Avg if
///  ---------------------------------------------------------------------------------------------------------------   
///   Core i7-3770K  CPU @3.50 GHz    0.839     1.672           0.355     0.737   0.086     0.600       1.036
///   Core i7-2720QM CPU @2.20 GHz    1.014     1.968           0.410     0.845  
///  ---------------------------------------------------------------------------------------------------------------
/// </summary>
public class YksVaiKaksSilmukkaa
{
    /// <summary>
    /// 
    /// </summary>
    public static void Main()
    {
        int KOKO = 100 * 1000 * 1000;
        int RAJA = 50;
        int xn = 0;
        int a = 134775813; // http://en.wikipedia.org/wiki/Linear_congruential_generator Delphi
        int c = 1;

        int[] luvut = new int[KOKO];
        for (int i = 0; i < luvut.Length; i++)
        {
            luvut[i] = xn;
            xn = (a * xn + c);
        }


        Stopwatch sw = Stopwatch.StartNew();
        double ka = Keskiarvo(luvut, RAJA);
        sw.Stop();
        double milliseconds = sw.Elapsed.TotalMilliseconds;
        Console.WriteLine("Yksi silmukka   {0,7:##0.000} s", milliseconds / 1000);

        sw.Reset();
        sw.Start();
        double sum = Summa(luvut, RAJA);
        double milliseconds1 = sw.Elapsed.TotalMilliseconds;
        int lkm = Maara(luvut, RAJA);

        sw.Stop();
        milliseconds = sw.Elapsed.TotalMilliseconds;
        Console.WriteLine("Kaksi silmukkaa {0,7:##0.000} s", milliseconds1 / 1000);
        Console.WriteLine("Kaksi silmukkaa {0,7:##0.000} s", milliseconds / 1000);
        Console.WriteLine("Määrä: {0} / {1}", lkm, luvut.Length);

        sw.Reset();
        sw.Start();
        // sum = Summa(luvut);
        // lkm = Maara(luvut);
        double ka3 = Keskiarvo(luvut);
        sw.Stop();
        milliseconds = sw.Elapsed.TotalMilliseconds;
        Console.WriteLine("Ilman iffiä     {0,7:##0.000} s", milliseconds / 1000);

        sw.Reset();
        sw.Start();
        // sum = Summa(luvut);
        // lkm = Maara(luvut);
        double ka4 = luvut.Average();
        sw.Stop();
        milliseconds = sw.Elapsed.TotalMilliseconds;
        Console.WriteLine("LinQ Avrerage   {0,7:##0.000} s", milliseconds / 1000);

        sw.Reset();
        sw.Start();
        double ka5 = ka= 1.0 * luvut.Where(x => x > RAJA).Average();
        sw.Stop();
        milliseconds = sw.Elapsed.TotalMilliseconds;
        Console.WriteLine("LinQ Avg where  {0,7:##0.000} s", milliseconds / 1000);
        Console.WriteLine("{0:0.00} {1:0.00} d. {2:0.0}",ka, ka5, ka-ka5);
    }


    public static double Keskiarvo(int[] luvut, int raja)
    {
        double sum = 0;
        int lkm = 0;
        foreach (int luku in luvut)
            if (luku > raja)
            {
                sum += luku;
                lkm++;
            }
        if (lkm <= 0) return 0;
        return sum / lkm;
    }


    public static double Summa(int[] luvut, int raja)
    {
        double sum = 0;
        foreach (int luku in luvut) if (luku > raja) sum += luku;
        return sum;
    }


    public static int Maara(int[] luvut, int raja)
    {
        int lkm = 0;
        foreach (int luku in luvut) if (luku > raja) lkm++;
        return lkm;
    }


    public static double Summa(int[] luvut)
    {
        double sum = 0;
        foreach (int luku in luvut) sum += luku;
        return sum;
    }


    // Tässähän eiole mitään järkeä sinällään.  Silmukka on vain siksi, että saadaan
    // pyöritettyä vertailutulos if-lauseen sisältävään verrattuna ja voidaan arvioida
    // if lauseen osuutta suoritusajasta suhteessa silmukan suoritukseen. 
    public static int Maara(int[] luvut)
    {
        int lkm = 0;
        foreach (int luku in luvut) lkm++;
        return lkm;
    }


    public static double Keskiarvo(int[] luvut)
    {
        double sum = 0;
        int lkm = 0;
        /*
        foreach (int luku in luvut)
        {
            sum += luku;
            lkm++;
        }
         */
        for (int i = 0; i < luvut.Length ; i++)
        {
            // int luku = luvut[i];
            // sum += luku;
            sum += luvut[i];
            lkm++;
        }
        if (lkm <= 0) return 0;
        return sum / lkm;
    }
}
