﻿using System;
using System.Linq;

/// @author  Vesa Lappalainen
/// @version 12.10.2012
/// <summary>
/// Esimerkkjä aliohjelmien kutsumisesta
/// </summary>
public class Kutsuminen
{
    /// <summary>
    /// Kutsutaan aliohjelmia
    /// </summary>
    /// <param name="args">ei käytössä</param>
    public static void Main(string[] args)
    {

        int[,] array2D = { { 1, 2 }, { 3, 4 }, { 5, 6 }, { 7, 8 } };
        Console.WriteLine(String.Join(" ", array2D.Cast<int>()));

        int[] luvut = { 4, 9, 6, 2, 5 };

        VaihdaSuurin(luvut, 5);

        Console.WriteLine(String.Join(" ", luvut));

        int paikka = EtsiPaikka(luvut, 6);

        Console.WriteLine("Löytyi paikasta: {0}", paikka);
    }


    /// <summary>
    /// Vaihdetaan taulukon suurin alkio halutuksi.
    /// </summary>
    /// <param name="t">taulukko josta suurinta etsitään ja missä suurin vaihdetaan</param>
    /// <param name="korvaavaArvo">suurimman alkion uusi arvo</param>
    /// <example>
    /// <pre name="test">
    ///   int[] luvut = {2,5,4,3};
    ///   VaihdaSuurin(luvut,1);
    ///   String.Join(" ",luvut) === "2 1 4 3";
    ///   VaihdaSuurin(luvut,1);
    ///   String.Join(" ",luvut) === "2 1 1 3";
    ///   VaihdaSuurin(luvut,9);
    ///   String.Join(" ",luvut) === "2 1 1 9";
    ///   VaihdaSuurin(luvut,1);
    ///   String.Join(" ",luvut) === "2 1 1 1";
    ///   VaihdaSuurin(luvut,-1);
    ///   String.Join(" ",luvut) === "-1 1 1 1";
    ///   luvut = new int[]{5};
    ///   VaihdaSuurin(luvut,2);
    ///   String.Join(" ",luvut) === "2";
    ///   luvut = new int[0];
    ///   VaihdaSuurin(luvut,2);
    ///   String.Join(" ",luvut) === "";
    /// </pre>
    /// </example>
    public static void VaihdaSuurin(int[] t, int korvaavaArvo)
    {
        int paikka = SuurimanPaikka(t);
        if (paikka < 0) return;
        t[paikka] = korvaavaArvo;
    }


    /// <summary>
    /// Etsii taulukon suurimman luvun paikan.  Mikäli monta
    /// yhtäsuurta, palautetaan ensimmäisen paikka.  Mikäli
    /// suurinta ei löydy (tyhjä taulukko), palautetaan -1
    /// </summary>
    /// <param name="t">tutkittava taulukko</param>
    /// <returns>suurimman alkion paikan indeksi tai -1</returns>
    /// <example>
    /// <pre name="test">
    ///    SuurimanPaikka(new int[]{1,2,5,3}) === 2;
    ///    SuurimanPaikka(new int[]{7,2,5,3}) === 0;
    ///    SuurimanPaikka(new int[]{1,2,5,8}) === 3;
    ///    SuurimanPaikka(new int[]{}) === -1;
    /// </pre>
    /// </example>
    public static int SuurimanPaikka(int[] t)
    {
        int paikka = -1;
        int suurin = int.MinValue;
        for (int i = 0; i < t.Length; i++)
        {
            int luku = t[i];
            if (luku > suurin)
            {
                suurin = luku;
                paikka = i;
            }
        }
        return paikka;
    }


    /// <summary>
    /// Etsii taulukosta luvun paikan.  Mikäli monta samaa, palautetaan 1. paikka.
    /// Mikäli ei löydy, palautetaan -1
    /// </summary>
    /// <param name="t">tutkittava taulukko</param>
    /// <param name="luku">etsittävä luku</param>
    /// <returns>luvun indeksi taulukossa tai -1</returns>
    /// <example>
    /// <pre name="test">
    ///    EtsiPaikka(new int[]{1,2,5,3}, 5) === 2;
    ///    EtsiPaikka(new int[]{7,2,5,3}, 7) === 0;
    ///    EtsiPaikka(new int[]{1,2,5,8}, 8) === 3;
    ///    EtsiPaikka(new int[]{1,2,2,8}, 2) === 1;
    ///    EtsiPaikka(new int[]{1,2,2,8}, 3) === -1;
    ///    EtsiPaikka(new int[]{}, 4) === -1;
    /// </pre>
    /// </example>
    public static int EtsiPaikka(int[] t, int luku)
    {
        for (int i = 0; i < t.Length; i++)
        {
            if (t[i] == luku) return i;
        }
        return -1;
    }
}
