﻿using System;
using System.Text;

/// @author  Vesa Lappalainen
/// @version 22.10.2012
/// <summary>
/// Harjoitellaan merkkijonojen pilkkomista.
/// </summary>
public class MjPilkkominen
{
    /// <summary>
    /// Kysellaan kayttajalta merkkijonoja ja 
    /// tehdaan niista taulukkoja, lasketaan lukuja 
    /// yhteen ja tulostellaan.
    /// </summary>
    public static void Main()
    {
        char[] erottimet = new char[] { ' ', ';', ',' };
        Console.Write("Erotinmerkit ovat: ");
        foreach (char merkki in erottimet)
        {
            Console.Write($"'{merkki}' ");
        }
        Console.WriteLine();
        Console.Write("Kirjoita jono jossa sanoja > ");
        string mj = Console.ReadLine();

        // Split()-metodin avulla saadaan tehtya merkkijonosta
        // merkkijonotaulukko. Split ottaa parametrina merkkitaulukon
        // jossa erotinmerkit on ilmoitettu.
        // String[] pilkottu = mj.Split(erottimet, StringSplitOptions.RemoveEmptyEntries); 
        string[] pilkottu = mj.Split(erottimet);
        TulostaPalat(pilkottu);

        Console.WriteLine("Poistetaan tyhjat merkit alusta ja lopusta Trim()-aliohjelmalla...");
        Console.WriteLine("Ei myoskaan tulosteta tyhjia alkioita");
        pilkottu = mj.Trim().Split(erottimet);

        Console.Write("Anna kokonaislukuja > ");
        string lukusyote = Console.ReadLine();

        int[] luvut = MerkkijonoLuvuiksi(lukusyote, erottimet);
        if ( luvut != null )
            Console.WriteLine("Antamiesi lukujen summa on : " + Summa(luvut));
        else
            Console.WriteLine("Annoit jotain muuta kuin lukuja, tai tyhjan merkkijonon");

        Console.WriteLine("Merkkijonotaulukko voidaan myos yhdistaa:");
        string yhdistetty = String.Join(":", pilkottu);
        Console.WriteLine(yhdistetty);

        int[] t1 = MjPilkkominen.MerkkijonoLuvuiksi("1 2 3", new char[] { ' ' });
        string luvutPotkossa = String.Join(",", t1);
        Console.WriteLine(luvutPotkossa);
    }


    /// <summary>
    /// Aliohjelma muuttaa annetun merkkijonon kokonaislukutaulukoksi
    /// siten, etta eri luvut erotellaan annetun merkkitaulukon (erotinmerkkien)
    /// perusteella. Alkuperäisen syötteen alusta ja lopusta poistetaan välilyönnit.
    /// </summary>
    /// <param name="lukusyote">Muunnettava merkkijono</param>
    /// <param name="erottimet">Sallitut erotinmerkit merkkitaulukossa</param>
    /// <returns>Merkkijonosta selvitetty kokonaislukutaulukko. null jos yksikin luku väärin</returns>
    /// @example
    /// <pre name="test">
    /// int[] t1 = MerkkijonoLuvuiksi("1 2 3", new char[] {' '});
    /// String.Join(" ", t1) === "1 2 3";
    /// int[] t2 = MerkkijonoLuvuiksi(" ,,1;;3  ;5  ", new char[] {' ', ',', ';'});
    /// String.Join(" ", t2) === "0 0 1 0 3 0 0 5";
    /// int[] t3 = MerkkijonoLuvuiksi("1;;3  ;5", new char[] {' ', ',', ';'});
    /// String.Join(" ", t3) === "1 0 3 0 0 5";
    /// MerkkijonoLuvuiksi("1 3k 5", new char[] {' '}) === null;
    /// </pre>
    /// @endexample
    public static int[] MerkkijonoLuvuiksi(string lukusyote, char[] erottimet)
    {
        // Tyhjat pois edesta ja lopusta (Trim())            
        string[] pilkottu = lukusyote.Trim().Split(erottimet);
        int[] luvut = new int[pilkottu.Length]; // luvut[] saa kookseen saman kuin pilkottu[]
        for (int i = 0; i < pilkottu.Length; i++)
        {
            if (String.IsNullOrEmpty(pilkottu[i]))
            {
                // Jos taulukosta loytyy tyhja merkkijono (tai null-arvo), 
                // asetetaan luvut[i] arvoksi 0 
                luvut[i] = 0;
            }
            else
            {
                if (!int.TryParse(pilkottu[i], out luvut[i])) return null;
            }
        }
        return luvut;
    }


    /// <summary>
    /// Laskee kokonaislukutaulukon alkiot yhteen ja palauttaa alkioiden summan.
    /// </summary>
    /// <param name="luvut">Tutkittava kokonaislukutaulukko</param>
    /// <returns>Taulukon alkioiden summa</returns>
    /// <example>
    /// <pre name="test">
    ///    Summa(new int[]{1,2,3}) === 6;
    ///    Summa(new int[]{2}) === 2;
    /// </pre>
    /// </example>
    public static int Summa(int[] luvut)
    {
        int summa = 0;
        foreach (int luku in luvut) 
            summa += luku;
        return summa;
    }


    /// <summary>
    /// Tulostetaan merkkijonotaulukon palat
    /// </summary>
    /// <param name="palat">Tulostettava merkkijonotaulukko</param>
    public static void TulostaPalat(string[] palat)
    {
        foreach (string pala in palat)
            Console.WriteLine("Pala: \"{0}\"", pala);
        Console.WriteLine("-------------------------------");
    }


}
