﻿using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Effects;
using Jypeli.Widgets;

/// @author  Vesa Lappalainen
/// @version 22.10.2012
/// 
/// <summary>
/// Piirretään näyttöön pisteitä ja lasketaan niiden välisiä etäisyyksiä
/// </summary>
public class LaskeMatka : PhysicsGame
{

    /// <summary>Koordinaatistoakselit</summary>
    private Axis akselit; // Atribuutti


    /// <summary>Merkki paikalle jota viimeksi klikattu</summary>
    private PhysicsObject klikattuPiste;

    private const double x1 = -5;
    private const double x2 = 20;
    private const double y1 = -5;
    private const double y2 = 20;
    private const double pallonKoko = (y2 - y1) / 300;

    // Vaihda kommentteja vaihtamalla pisteet joko taulukoksi (joka tarvii lkm)
    // tai listaksi.  Eli jos haluat käyttää taulukoita, kommentoi
    // kaikki rivit joiden perässä on listalle ja poista kommentit riveistä
    // joissa lukee taulukolle.  Ja päinvastoin jos haluat käyttää listaa.

    /// <summary>Koordinaatit piirretyille pisteille</summary>

    // private Vector[] pisteet = new Vector[10];  // taulukolle
    // private int lkm = 0;                        // taulukolle  

    private List<Vector> pisteet = new List<Vector>(); // listalle

    private DoubleMeter yhteismatka = new DoubleMeter(200);
    private Label matkanytto;

    public override void Begin()
    {
        Level.Background.Color = Color.White;
        // IsFullScreen = true; // ota tämä pois kommentista jos haluat että hiiren klikkaus osuu kohdalleen
        // tai aja vaihtamalla kohteeksi Debug sijasta Release

        akselit = new Axis(x1, y1, x2, y2);

        klikattuPiste = LuoPallo(this, Vector.Zero, 1.5 * pallonKoko, Color.Blue);

        matkanytto = new Label("0.0");
        matkanytto.BindTo(yhteismatka);
        Add(matkanytto);
        matkanytto.Position = new Vector(400, 300);
        matkanytto.DoubleFormatString = "{0:0.000} m";

        AsetaOhjaimet();
        Camera.ZoomTo(x1, y1, x2, y2);

        LaskeMatkaNayttoon();
    }


    /// <summary>
    /// Lasketaan matka ja laitetaan tulos näyttöön
    /// </summary>
    private void LaskeMatkaNayttoon()
    {
        // double dist = SummaaEtaisyydet(pisteet, lkm); // Taulukolle
        double dist = SummaaEtaisyydet(pisteet);   // Listalle
        yhteismatka.Value = dist;
    }


    /// <summary>
    /// Lasketaan listassa olevien pisteiden vöisten etäisyyksien summa
    /// </summary>
    /// <param name="pisteet"></param>
    /// <returns></returns>
    // public static double SummaaEtaisyydet(Vector[] pisteet, int lkm) // taulukolle
    public static double SummaaEtaisyydet(List<Vector> pisteet) // listalle
    {
        double d = 0;
        int lkm = pisteet.Count;   // listalle
        if (lkm < 2) return 0;
        Vector p1 = pisteet[0];
        for (int i = 1; i < lkm; i++)
        {
            Vector p2 = pisteet[i];
            double s = Vector.Distance(p1, p2);
            d += s;
            p1 = p2;
        }
        return d;
    }


    /// <summary>
    /// Piirretään viivat pisteiden välille.
    /// </summary>
    /// <param name="canvas"></param>
    private void PiirraPisteidenValisteViivat(Canvas canvas)
    {
        int lkm = pisteet.Count; // listalle
        if (lkm < 2) return;
        canvas.BrushColor = Color.Green;
        Vector p1 = pisteet[0];
        for (int i = 1; i < lkm; i++)
        {
            Vector p2 = pisteet[i];
            canvas.DrawLine(p1, p2);
            p1 = p2;
        }
    }


    /// <summary>
    /// Asetetaan ohjaimet peliä varten. F1 ja ESC.
    /// Kuunnellaan hiiren paikkaa ja siirretään merkkipisteen sen mukaan mihin klikattu
    /// Samoin kuunnellaan puhelimelta sormella tökkäisyä.
    /// </summary>
    private void AsetaOhjaimet()
    {
        Keyboard.Listen(Key.F1, ButtonState.Pressed, ShowControlHelp, "Näytä ohjeet");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, Exit, "Poistu");

        Mouse.IsCursorVisible = true;
        Keyboard.Listen(Key.Delete, ButtonState.Pressed, PoistaPisteet, "Poistaa kaikki pisteet");
        Mouse.Listen(MouseButton.Left, ButtonState.Pressed, delegate () { LisaaPiste(Mouse.PositionOnWorld); }, "Laita piste");
        Mouse.Listen(MouseButton.Right, ButtonState.Pressed, PoistaPisteet, "Laita piste");

        // TouchPanel.Listen(ButtonState.Pressed, delegate(Touch kosketus) { SiirraMerkkipisteet(kosketus.PositionOnWorld); }, null);
        // TouchPanel.ListenGesture(GestureType.VerticalDrag, delegate(Touch touch) { KysyKoordinaatti(); }, "Kysy");
        PhoneBackButton.Listen(Exit, "Lopeta peli");
    }


    /// <summary>
    /// Poistetaan kaikki pisteet 
    /// </summary>
    private void PoistaPisteet()
    {
        ClearGameObjects();
        Add(matkanytto);

        // pisteet = new Vector[100]; lkm = 0; // taulukolle
        pisteet.RemoveRange(0, pisteet.Count); // listalle

        LaskeMatkaNayttoon();
    }


    /// <summary>
    /// Lisätään piste näyttöön ja listaan
    /// </summary>
    /// <param name="piste">paikka johon sininen piste</param>
    private void LisaaPiste(Vector piste)
    {
        // siirretään sininen pisteen kohdalle
        klikattuPiste.Position = piste;
        // if ( lkm < pisteet.Length ) { pisteet[lkm] = piste;  lkm++;  } // taulukolle
        pisteet.Add(piste);  // listalle
        LuoPallo(this, piste, pallonKoko, Color.Black);
        LaskeMatkaNayttoon();
    }


    /// <summary>
    /// Lisätään peliin pallo
    /// </summary>
    /// <param name="game">peli johon pallo lisätään</param>
    /// <param name="p">pallon keskipiste</param>
    /// <param name="r">pallon säde</param>
    /// <param name="color">pallon väri</param>
    /// <returns>lisätyn pallon viite</returns>
    public static PhysicsObject LuoPallo(PhysicsGame game, Vector p, double r, Color color)
    {
        PhysicsObject pallo = new PhysicsObject(r * 2, r * 2, Shape.Circle);
        pallo.Position = p;
        pallo.Color = color;
        game.Add(pallo);
        return pallo;
    }


    /// <summary>
    /// Tämä tehdään aina kun ruutua pitää päivittää
    /// </summary>
    /// <param name="canvas">Piirtoalue, johon voi piirtää</param>
    protected override void Paint(Canvas canvas)
    {
        akselit.Draw(canvas);
        PiirraPisteidenValisteViivat(canvas);
    }

}


/// <summary>
/// Luokka koordinaatiston piirtämistä varten
/// </summary>
public class Axis
{
    private double x1, x2, y1, y2;
    public Color Color { set; get; }

    /// <summary>
    /// Alustetaan koordinaatisto haluttuihin rajoihin
    /// </summary>
    /// <param name="x1">alueen vasemman alakulman x</param>
    /// <param name="y1">alueen vasemman alakulman y</param>
    /// <param name="x2">alueen vasemman yläkulman x</param>
    /// <param name="y2">alueen vasemman yläkulman y</param>
    public Axis(double x1, double y1, double x2, double y2)
    {
        this.x1 = x1;
        this.x2 = x2;
        this.y1 = y1;
        this.y2 = y2;
        Color = Color.Black;
    }


    /// <summary>
    /// Piirretään koordinaatisto niin, että tulee x ja y akselit ja niihin
    /// yhden välein pikkutikkuja.
    /// </summary>
    /// <param name="canvas">Canvas-olio, johon piirto tehdään</param>
    public void Draw(Canvas canvas)
    {
        canvas.BrushColor = Color;
        double t = (y2 - y1) / 200; // tikkujen koko

        // X-akseli
        canvas.DrawLine(new Vector(x1, 0), new Vector(x2, 0));
        // Tikut X-akselille 0:sta lähtien kumpaankin suuntaan
        for (double x = 0; x >= x1; x -= 1.0) canvas.DrawLine(new Vector(x, -t), new Vector(x, t));
        for (double x = 0; x <= x2; x += 1.0) canvas.DrawLine(new Vector(x, -t), new Vector(x, t));

        // Y    -akseli
        canvas.DrawLine(new Vector(0, y1), new Vector(0, y2));
        // Tikut Y-akselille 0:sta lähtien kumpaankin suuntaan
        for (double y = 0; y >= y1; y -= 1.0) canvas.DrawLine(new Vector(-t, y), new Vector(t, y));
        for (double y = 0; y <= y2; y += 1.0) canvas.DrawLine(new Vector(-t, y), new Vector(t, y));
    }
}


