﻿using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
// using Jypeli.Effects;
using Jypeli.Widgets;
using System.IO;

using FSJoint = FarseerPhysics.Dynamics.Joints.DistanceJoint;
using FarseerPhysics.Factories;
using FarseerPhysics.Dynamics;
using Jypeli.Farseer;


/// @author vesal
/// @version 2.11.2014
/// <summary>
/// Hirsipuupeli
/// 
/// Jos haluat enemmän sanoja, tee tiedostos sanat.txt johon luettelet
/// Alekkain sanoja tai sitten hae valmis Kotus-sanasto osoitteesta
///   https://kurssit.it.jyu.fi/ITKP102/content/Hirsipuu/sanat.txt 
/// ja tallenna se samaan paikkaan mihin .exe tulee
/// </summary>
public class Hirsipuu : PhysicsGame
{
    private static readonly string[] Kirjaimet =
    {
        "QWERTYUIOPÅ",
        "ASDFGHJKLÖÄ",
        "ZXCVBNM -"
    };

    private Aakkoset aakkoset;
    private Sana sana;
    private Hirsi hirsi;
    private PhysicsObject kuu;
    private bool loppunut = false; // jottei tule montaa loppujuttua kerralla
    private readonly string[] luetutRivit = LueSanat("sanat.txt");


    public static string[] LueSanat(string nimi)
    {
        try
        {
            return File.ReadAllLines(nimi);
        }
        catch (IOException)
        {
            return null;
        }
    }

    public override void Begin()
    {
        SetWindowSize(1280, 800);  // Tätä voi joutua käyttämään jos näytön resoluution on kovin suuri
        // SetWindowSize((int)Screen.Width, (int)Screen.Height); 

        string teksti = "LUMIUKKO"; 
        if (luetutRivit != null) teksti = RandomGen.SelectOne(luetutRivit).ToUpper();

        ClearAll();
        loppunut = false;

        Level.Background.CreateGradient(Color.Brown, Color.Black);

        aakkoset = new Aakkoset(this, Kirjaimet, new Vector(-200, 0), 800);
        sana = new Sana(this, new Vector(-200, 300), teksti, 100);
        hirsi = new Hirsi(this, new Vector(-400, 200), PeliLoppui, aakkoset.KirjaimenKoko, 6);

        kuu = PhysicsObject.CreateStaticObject(150, 150);
        kuu.Image = LoadImage("kuu");
        kuu.Position = new Vector(-790, 350);
        Add(kuu);

        Gravity = new Vector(0, -400);

        Mouse.IsCursorVisible = true;
        Mouse.Listen(MouseButton.Left, ButtonState.Pressed, TutkiKirjain, "Valitse kirjain");

        PhoneBackButton.Listen(ConfirmExit, "Lopeta peli");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");
        Keyboard.Listen(Key.F5, ButtonState.Pressed, Begin, "Uusi peli");

        Keyboard.TextInput += KirjainPainettu;

        Camera.ZoomToAllObjects();

        Timer.SingleShot(0.1, Camera.ZoomToAllObjects); // Muuten uuden pelin aluksi zoomaa väärin 
    }


    private void KirjainPainettu(char key)
    {
        string c = key.ToString().ToUpper();
        Kirjain k = aakkoset.AnnaKirjain(c);
        if (k == null) return;
        sana.Tutki(k, SanaTutkittu);
    }


    private void TutkiKirjain()
    {
        Kirjain k = aakkoset.AnnaKirjain(Mouse.PositionOnWorld);
        if (k == null) return;
        // Remove(k);  Add(k, 0);  // ei toiminut ???
        sana.Tutki(k, SanaTutkittu);
    }


    private void SanaTutkittu(Kirjain k, int n, bool valmis)
    {
        if (valmis)
        {
            aakkoset.PudotaKaikki();
            k.IgnoresGravity = false;
            hirsi.PoistaNaru();
            KysyUusi("Voitit");
            return;
        }

        if (n > 0) { Remove(k); return; }

        k.MoveTo(hirsi.Position + new Vector(0, 200), 400, delegate () { hirsi.Tutki(k); });
    }


    private void PeliLoppui()
    {
        if (loppunut) return; // vältetään mahdollinen toinen loppukutsu

        loppunut = true;
        aakkoset.PudotaKaikki();

        // Ajetaan Maan varjo kuun eteen
        var varjo = new PhysicsObject(kuu.Width, kuu.Height, Shape.Circle);
        varjo.Position = kuu.Position + new Vector(kuu.Width, 0);
        varjo.Color = Color.Black;
        varjo.IgnoresCollisionResponse = true;
        varjo.IgnoresGravity = true;
        Add(varjo, 2);
        varjo.MoveTo(kuu.Position - new Vector(kuu.Width, 0), 2);

        // Pois Zoomaus
        double z = Camera.ZoomFactor;
        Timer timer = new Timer();
        timer.Interval = 0.1;
        timer.Timeout += () =>
        {
            Camera.ZoomFactor = z;
            z *= 0.999;
        };
        timer.Start();
        timer.Interval = 0.1;

        // Pimennys
        Timer timer2 = new Timer();
        Color c1 = Color.Brown;
        double colorFactor = 0.9;
        timer2.Timeout += () =>
        {
            Level.Background.CreateGradient(c1, Color.Black);
            c1.GreenComponent = (byte)(c1.GreenComponent * colorFactor);
            c1.RedComponent = (byte)(c1.RedComponent * colorFactor);
            c1.BlueComponent = (byte)(c1.BlueComponent * colorFactor);
        };
        timer2.Interval = 1;
        timer2.Start();
        KysyUusi("Hävisit");
    }


    private void KysyUusi(string viesti)
    {
        MultiSelectWindow valikko = new MultiSelectWindow("\n\n   " + viesti + "\n\nOtetaanko uusi peli?", "Uusi peli", "Lopeta");
        valikko.ItemSelected += delegate (int valinta)
        {
            if (valinta == 0) Begin();
            else Exit();
        };
        Add(valikko);
    }
}


/// <summary>
/// Hiris-luokka pitämään ukkoa hirressä ja ottamaan "koppia" ohi menneistä kirjaimista.
/// </summary>
public class Hirsi : PhysicsObject
{
    /*
    public class AxleJoint : Jypeli.AxleJoint
    {

        internal FSJoint InnerJoint
        {
            get
            {
                return (FSJoint)innerJoint;
            }
            set
            {
                innerJoint = value;
            }

        }

        /// <summary>
        /// Tällä voi säätää näytetäänkö liitospisteet
        /// </summary>
        public static bool DrawJointPoints { get; set; }

        /// <summary>
        /// Luo uuden akseliliitoksen kahden olion välille.
        /// </summary>
        /// <param name="firstObject">Ensimmäinen olio</param>
        /// <param name="secondObject">Toinen olio</param>
        /// <param name="axlePosition">Liitoksen akselin paikka</param>
        /// <summary>
        public AxleJoint(PhysicsObject firstObject, PhysicsObject secondObject, Vector axlePosition)
            : base(firstObject, secondObject, axlePosition)
        {
            World world = PhysicsGame.Instance.Engine as World;
            var first = firstObject.Body as PhysicsBody;
            var second = secondObject.Body as PhysicsBody;
            var ap2 = firstObject.Position + axlePosition - secondObject.Position;
            InnerJoint = JointFactory.CreateDistanceJoint(world,
                first.FSBody, second.FSBody,
                axlePosition * FSConvert.DisplayToSim, ap2 * FSConvert.DisplayToSim);
            // axlePosition2 * FSConvert.DisplayToSim);
            InnerJoint.DampingRatio = 0.5f;
            InnerJoint.Frequency = 50;
            if (!DrawJointPoints) return;

            var dot = new GameObject(15, 10, Shape.Circle);
            dot.Color = Color.Red;
            dot.Position = firstObject.Position + axlePosition;
            firstObject.Add(dot);
            dot = new GameObject(10, 10, Shape.Circle);
            dot.Color = Color.Green;
            dot.Position = secondObject.Position + ap2;
            secondObject.Add(dot);
            //InnerJoint.Enabled = false;
        }

    }

    */


    private static readonly Image Kuva = Game.LoadImage("puupalikka");
    private readonly PhysicsGame peli;
    private readonly IntMeter palikoita = new IntMeter(0);
    private readonly PhysicsObject taso;
    private readonly PhysicsObject taso2;
    private readonly PhysicsObject r1;
    private readonly PhysicsObject r2;
    private PhysicsObject paa;
    private PhysicsObject naru;
    private PhysicsObject jakkara;
    private readonly Action peliLoppui;
    public int Raja { get; set; }

    public static PhysicsObject LuoOlio(PhysicsGame peli, Vector p, double w, double h, Image kuva, int taso = 0, bool makeStatic = false, int ignoreGroup = 100)
    {
        PhysicsObject olio = new PhysicsObject(w, h);
        olio.Position = p;
        if (makeStatic) olio.MakeStatic();
        olio.CollisionIgnoreGroup = ignoreGroup;
        olio.Image = kuva;
        peli.Add(olio, taso);
        return olio;
    }


    private PhysicsObject LuoOlio(Vector p, double w, double h, Image kuva, int taso = 0, bool makeStatic = false, int ignoreGroup = 100)
    {
        return LuoOlio(peli, p, w, h, kuva, taso, makeStatic, ignoreGroup);
    }


    public Hirsi(PhysicsGame peli, Vector p, Action peliLoppui, double kirjaimenKoko, int raja) : base(200, 400)
    {
        // AxleJoint.DrawJointPoints = true;  // Ota tämä käyttöön jos haluat nähdä saranapisteet
        Position = p;
        this.peli = peli;
        this.peliLoppui = peliLoppui;
        this.Raja = raja;
        double tolppanKorkeus = kirjaimenKoko * (raja - 1);

        // TODO: liikaa numeerisia kokeiltuja arvoja
        taso = LuoOlio(p + new Vector(-200, -600), 600, 30, Kuva, 1, false, 1);
        taso2 = LuoOlio(taso.Position - new Vector(0, 30), 600, 30, Kuva, 1, true, 0);
        r1 = LuoOlio(new Vector(taso.Right - kirjaimenKoko * 1.4 - 40 - 20, taso.Top + tolppanKorkeus / 2 - 20 + 1), 40, tolppanKorkeus + 20, Kuva, -1, false, 1);
        r2 = LuoOlio(new Vector(taso.Right - 20, taso.Top + tolppanKorkeus / 2 - 20 +1), 40, tolppanKorkeus + 20, Kuva, -1);
        r1.CanRotate = r2.CanRotate = false;

        peli.Add(new AxleJoint(r1, taso, new Vector(0, -r1.Height / 2 + 10 + 15)));
        peli.Add(new AxleJoint(r2, taso, new Vector(0, -r2.Height / 2 + 10 + 15)));

        PiirraHirsiJaUkko(p + new Vector(-500, -600));
    }


    private void PiirraHirsiJaUkko(Vector p)
    {
        // TODO: liikaa numeerisia kokeiltuja arvoja
        PhysicsObject tolppa = LuoOlio(p + new Vector(10, 300), 20, 600, Kuva, 0, true);
        peli.Add(new AxleJoint(tolppa, taso, new Vector(0, -tolppa.Height/2)));
        PhysicsObject orsi = LuoOlio(p + new Vector(150, 600), 300, 20, Kuva, 1, true);

        PhysicsObject tuki = LuoOlio(p + new Vector(50, 560), 100, 20, Kuva, -1, true);
        tuki.Angle = Angle.FromDegrees(45);

        naru = LuoOlio(p + new Vector(280, 550), 10, 100, Kuva, 0, false);
        peli.Add(new AxleJoint(naru, orsi, new Vector(0, +naru.Height / 2)));

        paa = LuoUkko(peli, naru.Position + new Vector(0, -50 - 50));
        peli.Add(new AxleJoint(naru, paa, new Vector(0, -naru.Height/2)));

        jakkara = LuoOlio(new Vector(naru.X, p.Y + 50), 200, 70, Kuva, 0, false, 0);
    }


    public static PhysicsObject LuoUkko(PhysicsGame peli, Vector p)
    {
        // TODO: liikaa numeerisia kokeiltuja arvoja

        PhysicsObject paa = LuoOlio(peli, p, 80, 140, Game.LoadImage("paa"), 1);

        PhysicsObject torso = LuoOlio(peli, paa.Position + new Vector(0, -90), 130, 200, Game.LoadImage("torso"));
        peli.Add(new AxleJoint(torso, paa,new Vector(7, torso.Height/2-10)));

        PhysicsObject vasenKasi = LuoOlio(peli, paa.Position + new Vector(50 + 75, -20), 150, 40, Game.LoadImage("kasi"));
        peli.Add(new AxleJoint(torso, vasenKasi, new Vector(+torso.Width / 2 * 0.85, torso.Height / 2 - 30)));
        vasenKasi.Angle = Angle.FromDegrees(-90);


        PhysicsObject oikeaKasi = LuoOlio(peli, paa.Position + new Vector(-50 + -65, -20), 150, 40, Game.LoadImage("kasi"));
        // oikeaKasi.Angle = Angle.FromDegrees(180);
        oikeaKasi.MirrorImage();
        peli.Add(new AxleJoint(torso, oikeaKasi, new Vector(-torso.Width/2*0.8, torso.Height/2-30)));

        PhysicsObject vasenJalka = LuoOlio(peli, torso.Position + new Vector(40, -110 - 75), 75, 190, Game.LoadImage("jalka"));
        peli.Add(new AxleJoint(vasenJalka, torso, new Vector(0, vasenJalka.Height/2)));

        PhysicsObject oikeaJalka = LuoOlio(peli, torso.Position + new Vector(-40, -110 - 75), 75, 190, Game.LoadImage("jalka"));
        oikeaJalka.MirrorImage();
        peli.Add(new AxleJoint(oikeaJalka, torso, new Vector(0, oikeaJalka.Height/2)));

        return paa;
    }


    public void Tutki(PhysicsObject k)
    {
        palikoita.Value++;
        k.MoveTo(Position + new Vector(0, 200), 400, delegate () { k.IgnoresGravity = false; k.IgnoresCollisionResponse = false; });
        peli.AddCollisionHandler(k, "palikka", TarkistaLoppu);
    }

    private void TarkistaLoppu(PhysicsObject k1, PhysicsObject k2)
    {
        if (palikoita.Value < Raja) return;
        // peli.Remove(taso2);
        taso2.Position -= new Vector(0, 100);
        paa.Image = Game.LoadImage("kuollutpaa");
        r1.CanRotate = r2.CanRotate = true;

        Explosion rajahdys = new Explosion(jakkara.Width);
        rajahdys.Position = new Vector(taso.Right, taso.Bottom);
        rajahdys.UseShockWave = true;
        peli.Add(rajahdys, 2);

        peliLoppui();
    }

    public void PoistaNaru()
    {
        peli.Remove(naru);
        // peli.Remove(jakkara);
        jakkara.Position -= new Vector(0, 35);
        // r1.CollisionIgnoreGroup = 0;
    }

}


/// <summary>
/// Käytössä olevat kirjaimet "kirjoituskonemuodossa". 
/// </summary>
public class Aakkoset
{
    private readonly List<Kirjain> kirjaimet = new List<Kirjain>();
    public double KirjaimenKoko { get; }

    public Aakkoset(PhysicsGame peli, string[] kirjaimet, Vector p, double w)
    {
        if (kirjaimet.Length == 0) return;
        int n = kirjaimet[0].Length;  // TODO: etsi pisin rivi
        if (n == 0) return;
        double d = w / n;
        KirjaimenKoko = d;
        Vector koko = new Vector(d, d);
        Vector rivisiirtyma = new Vector(d * 0.2, -d * 1.1);
        Vector sarakesiirtyma = new Vector(d * 1.1, 0);
        for (int rivi = 0; rivi < kirjaimet.Length; rivi++)
        {
            Vector paikka = p + rivisiirtyma * rivi;
            string jono = kirjaimet[rivi];
            for (int sarake = 0; sarake < jono.Length; sarake++)
            {
                char kirjain = jono[sarake];
                Kirjain palikka = new Kirjain(peli, kirjain, paikka, koko);
                this.kirjaimet.Add(palikka);
                paikka += sarakesiirtyma;
            }

        }
    }

    /// <summary>
    /// Etsitään kirjain, joka on pisteen p kohdalla.  Poistetaan kirjain omasta listasta.
    /// palautetaan 1. löytynyt.
    /// </summary>
    /// <param name="p">paikka jota tutkitaan</param>
    /// <returns>löytynyt pala tai null jos ei löydy</returns>
    public Kirjain AnnaKirjain(Vector p)
    {
        foreach (Kirjain k in kirjaimet)
            if (k.IsInside(p)) { kirjaimet.Remove(k); return k; }
        return null;
    }


    public Kirjain AnnaKirjain(string c)
    {
        foreach (Kirjain k in kirjaimet)
            if (k.Merkki.ToString() == c) { kirjaimet.Remove(k); return k; }
        return null;
    }


    public void PudotaKaikki()
    {
        foreach (Kirjain k in kirjaimet) k.IgnoresGravity = false;
    }

}


/// <summary>
/// Luokka arvattavalle sanalle. Kirjain voidaan tuoda tutkittavaksi ja oikeat kirjaimet käännetään.
/// </summary>
public class Sana
{
    private readonly string teksti;
    private readonly Vector position;
    public Vector Position { get { return position; } }
    private readonly List<Kirjain> kirjaimet = new List<Kirjain>();
    private readonly IntMeter kaannettyja = new IntMeter(0);
    private readonly PhysicsGame peli;

    public Sana(PhysicsGame peli, Vector p, string teksti, double d)
    {
        this.teksti = teksti;
        this.position = p;
        Vector sarakesiirtyma = new Vector(d * 1.1, 0);
        Vector paikka = p;
        this.peli = peli;
        Vector koko = new Vector(d, d);
        for (int sarake = 0; sarake < teksti.Length; sarake++)
        {
            char kirjain = teksti[sarake];
            Kirjain palikka = new Kirjain(peli, kirjain, paikka, koko, false);
            this.kirjaimet.Add(palikka);
            paikka += sarakesiirtyma;
        }
    }

    public delegate void SanaTutkittu(Kirjain k, int n, bool valmis);

    public void Tutki(Kirjain k, SanaTutkittu tutkittu)
    {
        k.MoveTo(kirjaimet[0].Position + new Vector(0, -kirjaimet[0].Height), 300, delegate () { Tutki(k, tutkittu, 0, 0); });
    }

    public void Tutki(Kirjain k, SanaTutkittu tutkittu, int i, int n)
    {
        if (i >= teksti.Length) { tutkittu(k, n, kaannettyja.Value == teksti.Length); return; }
        Kirjain oma = kirjaimet[i];
        if (k.Merkki == oma.Merkki)
        {
            n++;
            oma.Kaanna();
            kaannettyja.Value++;
            Explosion rajahdys = new Explosion(oma.Width * 2);
            rajahdys.Position = oma.Position;
            rajahdys.UseShockWave = false;
            peli.Add(rajahdys, 2);

        }
        if (i < teksti.Length - 1) oma = kirjaimet[i + 1];
        k.MoveTo(new Vector(oma.X, k.Y), 300, delegate () { Tutki(k, tutkittu, i + 1, n); });
    }
}


/// <summary>
/// Ykis kirjain, joka voidaan kääntää.
/// </summary>
public class Kirjain : PhysicsObject
{
    private static readonly Font Fontti = new Font(70, true);
    private static readonly Vector siirto = new Vector(0, 5);
    private static readonly Image Kuva = PhysicsGame.LoadImage("puupalikka");
    public char Merkki { get; }
    private readonly Label label;

    public Kirjain(PhysicsGame peli, char k, Vector p, Vector koko, bool nakyy = true) : base(10, 10)
    {
        this.Merkki = k;
        this.Image = Kuva;
        peli.Add(this);
        Position = p;
        Size = koko;
        label = new Label("" + k);
        label.Font = Fontti;
        Tag = "palikka";

        label.Position = p + siirto;
        Add(label);

        label.IsVisible = nakyy;
        IgnoresCollisionResponse = true;
        IgnoresGravity = true;
    }


    public void Kaanna()
    {
        label.IsVisible = true;
    }
}