﻿using System;
using System.Text;
using System.Linq;
using System.Collections.Generic;
using System.IO;
using System.Text.RegularExpressions;
using System.Globalization;
using System.Collections;

/// @author vesal
/// @version 1.11.2016
/// <summary>
/// Yhdistetään tiedostoja joissa demoihin käytettyjä aikoja ja dempisteitä.
/// Yhdistetään käyttäjätunnuksen perusteella.  Jos tunntusta ei ole kummassakin
/// tiedostossa, ko tunnus hylätään.
/// <pre>
/// Tiedostojen muoto:
///    aajat.txt:
///     Olli Opiskelija; op25; 117356.tunnit; 2016-09-12 16:09:45.784837+00:00; 1; 
///     4
///     
///     ----------------------------------------------------------------------------------
///     Olli Opiskelija; op21; 117356.tunnit; 2016-09-18 21:21:05.440732+00:00; 1; 
///     1
///     
///     ----------------------------------------------------------------------------------
///     Olli Opiskelija; op3693; 117356.tunnit; 2016-09-11 19:21:19.272020+00:00; 1; 
///     3
///     
///     ----------------------------------------------------------------------------------
///     Olli Opiskelija; op19; 117356.tunnit; 2016-09-11 16:01:48.799028+00:00; 1; 
///     5
///
///  apist.txt
///     op19;d1;7
///     op21;d1;4
///     op25;d1;10
/// 
///  Nämä tulostetaan muodossa
///     op;demo;pist;aika
///     op25;d1;10;4
///     op21;d1;4;1
///     op19;d1;7;5
/// </pre>
/// </summary>
public class Aika
{
    /// <summary>
    /// Käsitellään demotiedostot
    /// </summary>
    /// <param name="args">ajat pist n</param>
    public static void Main(string[] args)
    {
        string ajat = "aajat{0}.txt";
        string pist = "apist{0}.txt";
        int n = 7;
        if (args.Length >= 1) ajat = args[0];
        if (args.Length >= 2) pist = args[1];
        if (args.Length >= 3) int.TryParse(args[2], out n);

        Console.WriteLine("op; demo; pist; aika");
        for (int i = 1; i <= n; i++)
            Yhdista(String.Format(ajat,i), String.Format(pist, i));
    }


    /// <summary>
    /// Yhdistetään kahden demotiedoston tiedot ja tulostetaan näyttöön
    /// </summary>
    /// <param name="ajat">aika-tiedoston nimi</param>
    /// <param name="pisteet">pistetiedoston nimi</param>
    public static void Yhdista(string ajat, string pisteet)
    {
        string[] aikarivit = File.ReadAllLines(ajat);
        string[] pisterivit = File.ReadAllLines(pisteet);

        foreach (var rivi in Yhdista(aikarivit, pisterivit)) Console.WriteLine(rivi);
    }


    /// <summary>
    /// Yhdistetään kahdessa taulukossa olevat tiedot
    /// </summary>
    /// <param name="aikarivit">taulukko jossa ona aikoja</param>
    /// <param name="pisterivit">taulukko jossa on demopisteitä</param>
    /// <returns>lista yhdistetyistä tiedoista</returns>
    /// <example>
    /// <pre name="test">
    ///   string[] aikarivit = {
    ///     "Olli Opiskelija; op25; 117356.tunnit; 2016-09-12 16:09:45.784837+00:00; 1; ",
    ///     "4",
    ///     "",
    ///     "----------------------------------------------------------------------------------",
    ///     "Olli Opiskelija; op21; 117356.tunnit; 2016-09-18 21:21:05.440732+00:00; 1; ",
    ///     "1 h",
    ///     "",
    ///     "----------------------------------------------------------------------------------",
    ///     "Olli Opiskelija; op3693; 117356.tunnit; 2016-09-11 19:21:19.272020+00:00; 1; ",
    ///     "3",
    ///     "",
    ///     "----------------------------------------------------------------------------------",
    ///     "Olli Opiskelija; op19; 117356.tunnit; 2016-09-11 16:01:48.799028+00:00; 1; ",
    ///     "5"
    ///  };
    ///
    ///  string[] pisterivit = {
    ///     "op19;d1;7",
    ///     "op21;d1;4",
    ///     "op25;d1;10",
    ///     "op13;d1;7",
    ///  };
    /// 
    ///  string tulos = 
    ///     "op25;d1;10;4\n" +
    ///     "op21;d1;4;1\n" +
    ///     "op19;d1;7;5";
    /// 
    ///  var s = Yhdista(aikarivit, pisterivit);
    ///  String.Join("\n", s) === tulos; 
    ///  
    /// </pre>
    /// </example>
    public static List<string> Yhdista(string[] aikarivit, string[] pisterivit)
    {
        List<string> tulos = new List<string>();
        char[] erottimet = { ';' };
        Dictionary<string, string> aikadict = new Dictionary<string, string>();

        int i = 0;
        while (i < aikarivit.Length - 1)
        {
            string rivi = aikarivit[i];
            string[] palat = rivi.Split(erottimet);
            if (palat.Length < 2) { i++; continue; }
            string tunnus = palat[1].Trim();
            string aika = aikarivit[i + 1].Trim();
            aika = aika.Replace(',', '.');
            i += 2;
            double tunnit;
            // if (!double.TryParse(aika, out tunnit)) continue; // paranna tätä
            tunnit = ErotaDouble(aika, -1);
            if (tunnit < 0) continue;
            aika = "" + tunnit;
            if (aikadict.ContainsKey(tunnus)) aikadict[tunnus] = aika;
            else aikadict.Add(tunnus, aika);
        }

        foreach (string rivi in pisterivit)
        {
            string[] palat = rivi.Split(erottimet);
            string tunnus = palat[0];
            if (!aikadict.ContainsKey(tunnus)) continue;
            string aika = aikadict[tunnus];
            aikadict[tunnus] = rivi + ";" + aika;
        }

        foreach (var pair in aikadict)
        {
            if (pair.Value.IndexOf(";") >= 0)
                tulos.Add(pair.Value);
        }
        return tulos;
    }


    // Demo 7:

    /// <summary>
    /// Otetaan jonossa oleva reaaliluku.  Jos luku ei ole
    /// mielekäs luku, niin palautetaan oletus. Desimaalina on aina .
    /// </summary>
    /// <param name="jono">jono josta reaaliluku otetaan</param>
    /// <param name="oletus">mikä arvo annetaan jos jonosta ei saada lukua</param>
    /// <returns>otettu reaaliluku</returns>
    /// <example>
    /// <pre name="test">
    ///   ErotaDouble("") ~~~ 0.0;
    ///   ErotaDouble("",2) ~~~ 2.0;
    ///   ErotaDouble(" 2.3 ") ~~~ 2.3;
    ///   ErotaDouble("5 3") ~~~ 5;
    ///   ErotaDouble("5k3") ~~~ 5;
    ///   ErotaDouble("5e3") ~~~ 5000;
    ///   ErotaDouble("5E-3") ~~~ 0.005;
    ///   ErotaDouble("k") ~~~ 0.0;
    ///   ErotaDouble("k",1.0) ~~~ 1.0;
    ///   ErotaDouble("2..3") ~~~ 0.0;
    /// </pre>
    /// </example>
    public static double ErotaDouble(string jono, double oletus = 0.0)
    {
        string reg = @"^([-0-9\.eE]+)(.*)$";
        Match m = Regex.Match(jono.Trim(), reg);
        string tjono = m.Groups[1].Value;
        double tulos;
        NumberFormatInfo nfi = new NumberFormatInfo();
        nfi.NumberDecimalSeparator = ".";
        if (double.TryParse(tjono, NumberStyles.Any, nfi, out tulos)) return tulos;
        return oletus;
    }

}
