﻿using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Effects;
using Jypeli.Widgets;
using System.IO;
using System.Text.RegularExpressions;
using System.Globalization;

/// <summary>
/// Kuvataan miten demoajat ovat muuttuneet
/// </summary>
public class Demoajat : PhysicsGame
{
    /// <summary>Koordinaatistoakselit</summary>
    private Axis akselit;

    /// <summary>Merkki paikalle jota viimeksi klikattu</summary>
    private PhysicsObject klikattuPiste;

    /// <summary>Merkki pisteelle joka on lähinnä klikattua paikkaa</summary>
    private PhysicsObject merkkipiste;

    private static Vector z = new Vector(-1, -1);

    public class Opiskelija
    {
        public string id;
        public Vector[] demot;
        public double avg;

        public Opiskelija(string id, int n, int dnro, double tunnit, double demoja)
        {
            this.id = id;
            this.avg = -1;
            demot = new Vector[n+1];
            for (int i = 0; i < demot.Length; i++)
            {
                demot[i] = z;
            }
            demot[dnro] = new Vector(tunnit, demoja);
        }
    }


    private const double x1 = -1;
    private const double x2 = 20;
    private const double y1 = -1;
    private const double y2 = 18;
    private const double pallonKoko = (y2 - y1) / 300;
    private Dictionary<string, Opiskelija> opiskelijat = new Dictionary<string, Opiskelija>();
    private List<Opiskelija> opiskelijaLista = new List<Opiskelija>();
    private List<PhysicsObject> pallot = new List<PhysicsObject>();

    private readonly IntMeter opnro = new IntMeter(0);

    private int Opnro
    {
        get { return opnro.Value; }
        set { opnro.Value = value; }
    }
    private Label opname;
    private Label opN;

    public string[] args;

    /// <summary>
    /// Alustetaan rajat ja luodaan pisteet ja 
    /// niiden paikat pisteet-taulukkoon.
    /// </summary>
    public override void Begin()
    {
        string ajat = "aajat{0}.txt";
        string pist = "apist{0}.txt";
        int n = 7;
        if (args.Length >= 1) ajat = args[0];
        if (args.Length >= 2) pist = args[1];
        if (args.Length >= 3) int.TryParse(args[2], out n);

        Console.WriteLine("op; demo; pist; aika");
        for (int i = 1; i <= n; i++)
            Yhdista(String.Format(ajat, i), String.Format(pist, i), i, n);
        foreach (var pair in opiskelijat) opiskelijaLista.Add(pair.Value);

        Level.Background.Color = Color.White;
        // IsFullScreen = true;
        Level.Width = x2 - x1;
        Level.Height = y2 - y1;

        akselit = new Axis(x1, y1, x2, y2, 6, 6);

        klikattuPiste = LuoPallo(this, Vector.Zero, 1.5 * pallonKoko, Color.Blue, 1);
        merkkipiste = LuoPallo(this, Vector.Zero, 1.5 * pallonKoko, Color.Red, 1);

        Camera.ZoomTo(x1-1, y1-1, x2+1, y2+1);
        AsetaOhjaimet();

        LuoNaytto("Nro: {0:000}", opnro, 0);
        opname = LuoNaytto("nimi", null, 1);
        opN = LuoNaytto("n", null, 2);

        PiirraOpiskelija();
    }

    private void ClearPallot()
    {
        foreach (var p in pallot)
        {
            p.Destroy();
        }
        pallot.Clear();
    }




    private void PiirraOpiskelija(int onro=-2)
    {
        if (onro != -2) Opnro = onro;
        Opnro = Math.Max(0, Math.Min(opnro, opiskelijaLista.Count - 1));
        ClearPallot();
       
        var op = opiskelijaLista[opnro];
        opname.Text = op.id;
        var color = Color.Red;
        PhysicsObject obj = null;
        int cnt = 0;
        double sum = 0;
        int lasti = 0;
        int idx = 1;
        foreach (var v in op.demot) {
            if ( v.X >= 0 ) {
                cnt++;
                sum += v.X;
                lasti = idx;
            }
            idx++;
        }
        double r = 0.15;
        double avg = -1;
        if (cnt > 0) avg = sum / cnt;
        op.avg = avg;
        double x = -1;
        int opn = op.demot.Length-1;
        int mn = 0;
        for (int i = 1; i < op.demot.Length; i++)
        {
            Vector p = op.demot[i];
            x = p.X;
            if (x < 0)
            {
                p.X = avg;
                color = Color.LightGray;
                opn--;
            }
            else mn = i;
            obj = LuoPallo(this, p, r, color);
            pallot.Add(obj);
            color = Color.Black;
            r = 0.10;
        }
        obj.Size = new Vector(2*r*1.5, 2*r*1.5);
        obj.Color = Color.Green;
        if (x < 0) obj.Color = Color.Lime;
        if (obj.Y < 0)
        {
            obj.Color = Color.Brown;
            obj.Shape = Shape.Star;
        }
        opN.Text = "" + opn + "/" + mn;
    }


    /// <summary>
    /// Tämä tehdään aina kun ruutua pitää päivittää
    /// </summary>
    /// <param name="canvas">Piirtoalue, johon voi piirtää</param>
    protected override void Paint(Canvas canvas)
    {
        var op = opiskelijaLista[opnro];
        akselit.Draw(canvas);
        Vector p1 = pallot[0].Position;

        for (int i = 1; i < pallot.Count; i++)
        {
            Vector p = pallot[i].Position;
            canvas.DrawLine(p1, p);
            p1 = p;
        }
    }

    
    /// <summary>
    /// Yhdistetään kahden demotiedoston tiedot ja tulostetaan näyttöön
    /// </summary>
    /// <param name="ajat">aika-tiedoston nimi</param>
    /// <param name="pisteet">pistetiedoston nimi</param>
    /// <param name="n">Demojen lkm</param>
    private void Yhdista(string ajat, string pisteet, int dnro, int n)
    {
        string[] aikarivit = File.ReadAllLines(ajat);
        string[] pisterivit = File.ReadAllLines(pisteet);

        Yhdista(aikarivit, pisterivit, n, dnro); // Console.WriteLine(rivi);
    }


    private void Yhdista(string[] aikarivit, string[] pisterivit, int n, int dnro)
    {
        List<string> tulos = new List<string>();
        int i = 0;
        char[] erottimet = { ';' };

        while (i < aikarivit.Length - 1)
        {
            string rivi = aikarivit[i];
            string[] palat = rivi.Split(erottimet);
            if (palat.Length < 2) { i++; continue; }
            string tunnus = palat[1].Trim();
            string aika = aikarivit[i + 1].Trim();
            aika = aika.Replace(',', '.');
            i += 2;
            double tunnit = 0;
            // if (!double.TryParse(aika, out tunnit)) continue; // paranna tätä
            tunnit = ErotaDouble(aika, -1);
            if (opiskelijat.ContainsKey(tunnus)) opiskelijat[tunnus].demot[dnro].X = tunnit;
            else
            {
                Opiskelija op = new Opiskelija(tunnus, n, dnro, tunnit, -1);
                opiskelijat.Add(tunnus, op);
            }
        }

        foreach (string rivi in pisterivit)
        {
            string[] palat = rivi.Split(erottimet);
            if (palat.Length < 3) continue;
            string tunnus = palat[0];
            double pisteet = ErotaDouble(palat[2], -1);
            if (opiskelijat.ContainsKey(tunnus)) opiskelijat[tunnus].demot[dnro].Y = pisteet;
            else
            {
                Opiskelija op = new Opiskelija(tunnus, n, dnro, -1, pisteet);
                opiskelijat.Add(tunnus, op);
            }
        }
    }


    /// <summary>
    /// Luo yksittäisen labelin näyttämään laskurin arvoa n:n ilmoittamaan paikkaan
    /// </summary>
    /// <param name="format">missä muodossa laskurin arvo näytetään</param>
    /// <param name="mittari">mihin laskuriin kiinitetään</param>
    /// <param name="n">monenteenko paikkaan kokrkeussuunnassa</param>
    /// <returns></returns>
    private Label LuoNaytto(string format, IntMeter mittari, int n)
    {
        Label naytto = new Label(format);
        naytto.Width = 150;
        naytto.SizeMode = TextSizeMode.None;

        naytto.IntFormatString = format;
        if (mittari != null) naytto.BindTo(mittari);
        naytto.Position = new Vector(Screen.Right - naytto.Width / 2 - 20, Screen.Top - naytto.Height / 2 - n * naytto.Height);
        naytto.Color = Color.Yellow;
        naytto.BorderColor = Color.Black;
        naytto.PreferredSize = new Vector(150, naytto.Height);
        Add(naytto);
        return naytto;
    }


    // Demo 7:

    /// <summary>
    /// Otetaan jonossa oleva reaaliluku.  Jos luku ei ole
    /// mielekäs luku, niin palautetaan oletus. Desimaalina on aina .
    /// </summary>
    /// <param name="jono">jono josta reaaliluku otetaan</param>
    /// <param name="oletus">mikä arvo annetaan jos jonosta ei saada lukua</param>
    /// <returns>otettu reaaliluku</returns>
    /// <example>
    /// <pre name="test">
    ///   ErotaDouble("") ~~~ 0.0;
    ///   ErotaDouble("",2) ~~~ 2.0;
    ///   ErotaDouble(" 2.3 ") ~~~ 2.3;
    ///   ErotaDouble("5 3") ~~~ 5;
    ///   ErotaDouble("5k3") ~~~ 5;
    ///   ErotaDouble("5e3") ~~~ 5000;
    ///   ErotaDouble("5E-3") ~~~ 0.005;
    ///   ErotaDouble("k") ~~~ 0.0;
    ///   ErotaDouble("k",1.0) ~~~ 1.0;
    ///   ErotaDouble("2..3") ~~~ 0.0;
    /// </pre>
    /// </example>
    public static double ErotaDouble(string jono, double oletus = 0.0)
    {
        string reg = @"^([-0-9\.eE]+)(.*)$";
        Match m = Regex.Match(jono.Trim(), reg);
        string tjono = m.Groups[1].Value;
        double tulos = oletus;
        NumberFormatInfo nfi = new NumberFormatInfo();
        nfi.NumberDecimalSeparator = ".";
        if (double.TryParse(tjono, NumberStyles.Any, nfi, out tulos)) return tulos;
        return oletus;
    }

    
    /// <summary>
    /// Asetetaan ohjaimet peliä varten. F1 ja ESC.
    /// Kuunnellaan hiiren paikkaa ja siirretään merkkipisteen sen mukaan mihin klikattu
    /// Samoin kuunnellaan puhelimelta sormella tökkäisyä.
    /// </summary>
    private void AsetaOhjaimet()
    {
        Keyboard.Listen(Key.F1, ButtonState.Pressed, ShowControlHelp, "Näytä ohjeet");
//        Keyboard.Listen(Key.Escape, ButtonState.Pressed, Exit, "Poistu");
        Keyboard.Listen(Key.Left, ButtonState.Pressed, () => PiirraOpiskelija(opnro-1) , "Poistu");
        Keyboard.Listen(Key.Right, ButtonState.Pressed, () => PiirraOpiskelija(opnro + 1), "Poistu");

        Mouse.IsCursorVisible = true;
        Mouse.Listen(MouseButton.Left, ButtonState.Pressed, () => PiirraOpiskelija(opnro - 1), "Laita piste");
        Mouse.Listen(MouseButton.Right, ButtonState.Pressed, () => PiirraOpiskelija(opnro + 1), "Laita piste");

        PhoneBackButton.Listen(Exit, "Lopeta peli");
    }


    /// <summary>
    /// Siirretään sininen piste paikan p kohdalle 
    /// ja siirretään punainen piste lähimmän alueen pisteen päälle
    /// </summary>
    /// <param name="p">paikka johon sininen piste tulee</param>
    public void SiirraMerkkipisteet(Vector p)
    {
        klikattuPiste.Position = p;

    }


    /// <summary>
    /// Luodaan peliin taulukossa olevien koordinaattien kohdalle pienet pallot
    /// </summary>
    /// <param name="game">peli, johon pisteet lisätään</param>
    /// <param name="pisteet">piirrettävien pisteiden koordinaatit</param>
    /// <param name="ala">raja jota pienemmät piirretään sinisellä</param>
    /// <param name="ala">raja jota suuremmat piirretään punaisella</param>
    /// <param name="r">säde kullekin pisteelle</param>
    public static List<PhysicsObject> LuoPisteet(PhysicsGame game, IEnumerable<Vector> pisteet, double r, double ala = double.MinValue, double yla = double.MaxValue)
    {
        List<PhysicsObject> pallot = new List<PhysicsObject>();
        foreach (Vector p in pisteet)
        {
            Color color = Color.Black;
            if (p.Y <= ala) color = Color.Blue;
            if (p.Y >= yla) color = Color.Red;
            pallot.Add(LuoPallo(game, p, r, color));
        }
        return pallot;
    }


    /// <summary>
    /// Lisätään peliin pallo
    /// </summary>
    /// <param name="game">peli johon pallo lisätään</param>
    /// <param name="p">pallon keskipiste</param>
    /// <param name="r">pallon säde</param>
    /// <param name="color">pallon väri</param>
    /// <param name="taso">mille tasolle pallo lisätään</param>
    /// <returns>lisätyn pallon viite</returns>
    public static PhysicsObject LuoPallo(PhysicsGame game, Vector p, double r, Color color, int taso = 0)
    {
        PhysicsObject pallo = new PhysicsObject(r * 2, r * 2, Shape.Circle);
        pallo.Position = p;
        pallo.Color = color;
        game.Add(pallo, taso);
        return pallo;
    }


    /// <summary>
    /// Arvotaan joukko 2D-pisteitä halutulle alueelle
    /// </summary>
    /// <param name="n">montako pistettä arvotaan</param>
    /// <param name="x1">alueen vasemman alakulman x</param>
    /// <param name="y1">alueen vasemman alakulman y</param>
    /// <param name="x2">alueen oikean yläkulman x</param>
    /// <param name="y2">alueen oikean yläkulman y</param>
    /// <returns></returns>
    public static List<Vector> ArvoPisteet(int n, double x1, double y1, double x2, double y2)
    {
        List<Vector> pisteet = new List<Vector>(n);
        for (int i = 0; i < n; i++)
        {
            double x = RandomGen.NextDouble(x1, x2);
            double y = RandomGen.NextDouble(y1, y2);
            pisteet.Add(new Vector(x, y));
            // pisteet[i] = new Vector(x, y);
        }
        return pisteet;
    }


    /// <summary>
    /// Etsitään pistettä lähimmän taulukon alkion indeksi.
    /// </summary>
    /// <param name="pisteet">taulukko jossa pisteet, joista lähintä etsitään</param>
    /// <param name="piste">piste johon muita verrataan</param>
    /// <returns>lähimmän pisteen indeksi, -1 jos taulukko tyhjä</returns>
    /// <example>
    /// <pre name="test">
    ///   Vector[] t = { new Vector(1,2),new Vector(3,4),new Vector(5,2),new Vector(5,5) };
    ///   List<Vector> luvut = new List<Vector>(t);
    ///   Kuvaaja.LahimmanIndeksi(luvut,new Vector(3,5)) === 1;
    ///   Kuvaaja.LahimmanIndeksi(luvut,new Vector(0,0)) === 0;
    ///   Kuvaaja.LahimmanIndeksi(luvut,new Vector(5,3)) === 2;
    ///   Kuvaaja.LahimmanIndeksi(luvut,new Vector(15,5)) === 3;
    /// </pre>
    /// </example>
    public static int LahimmanIndeksi(List<Vector> pisteet, Vector piste)
    {
        int paikka = -1;
        double ero = double.MaxValue;
        for (int i = 0; i < pisteet.Count; i++)
        {
            Vector p = pisteet[i];
            double d = Vector.Distance(piste, p);
            if (d < ero)
            {
                paikka = i;
                ero = d;
            }
        }
        return paikka;
    }



}


/// <summary>
/// Luokka koordinaatiston piirtämistä varten
/// </summary>
public class Axis
{
    private double x1, x2, y1, y2, zx, zy;
    public Color Color { set; get; }

    /// <summary>
    /// Alustetaan koordinaatisto haluttuihin rajoihin
    /// </summary>
    /// <param name="x1">alueen vasemman alakulman x</param>
    /// <param name="y1">alueen vasemman alakulman y</param>
    /// <param name="x2">alueen vasemman yläkulman x</param>
    /// <param name="y2">alueen vasemman yläkulman y</param>
    public Axis(double x1, double y1, double x2, double y2, double zx=0, double zy=0)
    {
        this.x1 = x1;
        this.x2 = x2;
        this.y1 = y1;
        this.y2 = y2;
        this.zx = zx;
        this.zy = zy;
        Color = Color.Black;
    }


    /// <summary>
    /// Piirretään koordinaatisto niin, että tulee x ja y akselit ja niihin
    /// yhden välein pikkutikkuja.
    /// </summary>
    /// <param name="canvas">Canvas-olio, johon piirto tehdään</param>
    public void Draw(Canvas canvas)
    {
        canvas.BrushColor = Color;
        double t = (y2 - y1) / 200; // tikkujen koko
        double t2 = (y2 - y1) / 400; // tikkujen koko

        // X-akseli
        canvas.DrawLine(new Vector(x1, zy), new Vector(x2, zy));
        // Tikut X-akselille 0:sta lähtien kumpaankin suuntaan
        for (double x = zx; x >= x1; x -= 1.0) canvas.DrawLine(new Vector(x, zy-t), new Vector(x, zy+t));
        for (double x = zx; x <= x2; x += 1.0) canvas.DrawLine(new Vector(x, zy-t), new Vector(x, zy+t));

        for (double x = zx; x >= x1; x -= 0.2) canvas.DrawLine(new Vector(x, zy-t2), new Vector(x, zy+t2));
        for (double x = zx; x <= x2; x += 0.2) canvas.DrawLine(new Vector(x, zy-t2), new Vector(x, zy+t2));

        // Y-akseli
        canvas.DrawLine(new Vector(zx, y1), new Vector(zx, y2));
        // Tikut Y-akselille 0:sta lähtien kumpaankin suuntaan
        for (double y = zy; y >= y1; y -= 1.0) canvas.DrawLine(new Vector(zx-t, y), new Vector(zx + t, y));
        for (double y = zy; y <= y2; y += 1.0) canvas.DrawLine(new Vector(zx-t, y), new Vector(zx + t, y));

        for (double y = zy; y >= y1; y -= 0.2) canvas.DrawLine(new Vector(zx-t2, y), new Vector(zx + t2, y));
        for (double y = zy; y <= y2; y += 0.2) canvas.DrawLine(new Vector(zx-t2, y), new Vector(zx+t2, y));
    }



}

