using System;
using Jypeli;

/// @author  Vesa Lappalainen
/// @version 06.11.2011
/// <summary>
/// Sierpinskin kolmio piirtämällä sisältäpäin
/// </summary>
public class SierpinskinKolmio2 : PhysicsGame
{
    const double PIENIN_KORKEUS = 5.0;

    /// <summary>
    /// Isoimman korkeus ja ensimmäinen aliohjelmakutsu.
    /// </summary>
    public override void Begin()
    {
        double korkeus = 300;
        double y = (-1) * korkeus / 2;
        Level.Background.Color = Color.White;
        SierpinskinKolmio(this, 0, -korkeus, korkeus);
        Camera.Zoom(0.8);
        Keyboard.Listen(Key.Space, ButtonState.Pressed, () => Gravity = new Vector(0, -100), "Pudota");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, Exit, "Lopeta");
        Level.CreateBorders();
        Gravity = new Vector(0, -50);
    }


    /// <summary>
    /// Piirtää Sierpinskin kolmion.
    /// </summary>
    /// <param name="peli">Peli johon piirretään</param>
    /// <param name="x">kolmion kärjen keskipisteen x-koord</param>
    /// <param name="y">kolmion kärjen keskipisteen y-koord</param>
    /// <param name="h">kolmion korkeus</param>
    public static void SierpinskinKolmio(PhysicsGame peli, double x, double y, double h)
    {
        if (h < PIENIN_KORKEUS) return;

        double s2 = h / Math.Sqrt(3); // sivun pituus s/2

        GameObject kolmio = new PhysicsObject(2 * s2, h, Shape.Triangle);
        kolmio.Color = Color.Black;
        kolmio.X = x;
        kolmio.Y = y + h / 2;
        kolmio.Angle = Angle.FromDegrees(180);
        peli.Add(kolmio);
        kolmio.Tag = "kolmio";
        // peli.AddCollisionHandler(kolmio, "kolmio", tormays);

        SierpinskinKolmio(peli, x - s2, y, h / 2); // Vasen alakolmio
        SierpinskinKolmio(peli, x + s2, y, h / 2); // Oikea alakolmio
        SierpinskinKolmio(peli, x, y + h, h / 2); // Yläkolmio
    }
}
