using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Effects;
using Jypeli.Widgets;

/// @author  Vesa Lappalainen
/// @version 06.11.2011
/// <summary>
/// 
/// Piirretään Sierpinskin kolmiota
///  - Space puolittaa kolmion minimikoon
///  - Enter lisää yhden kolmion
///  - Del palaa alkuun
/// </summary>
public class SierpinskinKolmio3 : Game
{
    private static double PIENIN_KORKEUS = 400.0;
    private static int KOLMIOIDEN_MAARA = 1;
    private static int kolmiota = 0;
    private static int OLETUS = -1;
    private DoubleMeter pienin = new DoubleMeter(PIENIN_KORKEUS);
    private IntMeter maara = new IntMeter(KOLMIOIDEN_MAARA);
    private int ALKUY = 0;


    /// <summary>
    /// Vaihdetaan taustaväri ja asetetaan kuuntelijat
    /// </summary>
    public override void Begin()
    {
        Level.Background.Color = Color.White;
        Keyboard.Listen(Key.Space, ButtonState.Pressed, MuutaKokoa, "Seuraava taso", 0.5);
        Keyboard.Listen(Key.Back, ButtonState.Pressed, MuutaKokoa, "Edellinen taso", 2.0);
        Keyboard.Listen(Key.Enter, ButtonState.Pressed, KasvataMaaraa, "Lisää kolmio");
        Keyboard.Listen(Key.Right, ButtonState.Pressed, KasvataMaaraa, "Lisää kolmio");
        Mouse.Listen(MouseButton.Left, ButtonState.Pressed, KasvataMaaraa, "Lisää kolmio");
        Mouse.Listen(MouseButton.Right, ButtonState.Pressed, VahennaMaaraa, "Poista kolmio");
        //Mouse.ListenWheel(HiirenRulla, "Muuta kokoa");
        Keyboard.Listen(Key.Left, ButtonState.Pressed, VahennaMaaraa, "Poista kolmio");
        Keyboard.Listen(Key.Down, ButtonState.Pressed, MuutaKokoa, "Pienennä kolmiota", 0.5);
        Keyboard.Listen(Key.Up, ButtonState.Pressed, MuutaKokoa, "Suurenna kolmiota", 2.0);
        Keyboard.Listen(Key.W, ButtonState.Down, delegate { ALKUY++; }, "Siirrä kuvaa ylös");
        Keyboard.Listen(Key.X, ButtonState.Down, () => ALKUY--, "Siirrä kuvaa alas");
        Keyboard.Listen(Key.Delete, ButtonState.Pressed, delegate { PaivitaNaytot(1, 400); }, "Alusta");
        Keyboard.Listen(Key.Add, ButtonState.Pressed, delegate { PaivitaNaytot(100000, OLETUS); }, "Valmis");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, Exit, "Exit");
        Keyboard.Listen(Key.F1, ButtonState.Pressed, ShowControlHelp, "Help");
        LuoNaytto("Määrä: {0:0}", maara, 1);
        LuoNaytto("Pienin: {0:0.0}", pienin, 2);
        IsMouseVisible = true;
    }


    /// <summary>
    /// Suurennetaan tai pienennetään pienimmän kolmion kokoa rullan perusteella
    /// </summary>
    private void HiirenRulla()
    {
        int suunta = Mouse.WheelChange;
        double kerroin = 2;
        if (suunta < 0) kerroin = 0.5;
        MuutaKokoa(kerroin);
    }


    /// <summary>
    /// Suurennetaan tai pienennetään pienimmän kolmion kokoa kertoimen perusteella
    /// </summary>
    private void MuutaKokoa(double kerroin)
    {
        PaivitaNaytot(OLETUS, PIENIN_KORKEUS * kerroin);
    }


    /// <summary>
    /// Päivitetään laskurinäytöt annetuilla arvoilla
    /// </summary>
    /// <param name="n">uusi määrä, OLETUS=ei muuteta</param>
    /// <param name="koko">uusi koko, OLETUS=ei muuteta</param>
    private void PaivitaNaytot(int n = -1, double koko = -1)
    {
        if (n != OLETUS) KOLMIOIDEN_MAARA = n;
        if (koko != OLETUS) PIENIN_KORKEUS = koko;
        if (PIENIN_KORKEUS < 0.5) PIENIN_KORKEUS = 0.5;
        maara.Value = KOLMIOIDEN_MAARA;
        pienin.Value = PIENIN_KORKEUS;
    }


    /// <summary>
    /// Luo yksittäisen labelin näyttämään laskurin arvoa n:n ilmoittamaan paikkaan
    /// </summary>
    /// <param name="format">missä muodossa laskurin arvo näytetään</param>
    /// <param name="mittari">mihin laskuriin kiinitetään</param>
    /// <param name="n">monenteenko paikkaan kokrkeussuunnassa</param>
    /// <returns></returns>
    private Label LuoNaytto(string format, Meter mittari, int n)
    {
        Label naytto = new Label(format);
        naytto.Width = 250;
        naytto.PreferredSize = new Vector(150, naytto.Height);
        naytto.SizeMode = TextSizeMode.None;
        naytto.Position = new Vector(Screen.Right - naytto.Width / 2 - 20, Screen.Top - naytto.Height / 2 - n * naytto.Height);
        naytto.IntFormatString = format;
        naytto.DoubleFormatString = format;
        naytto.BindTo(mittari);
        naytto.Color = Color.Yellow;
        naytto.BorderColor = Color.Black;
        Add(naytto);
        return naytto;
    }


    /// <summary>
    /// Kasvatetaan kolmioiden määrää, jolloin kuvasta tulee valmiimpi
    /// </summary>
    public void KasvataMaaraa()
    {
        KOLMIOIDEN_MAARA++;
        if (KOLMIOIDEN_MAARA > 200) KOLMIOIDEN_MAARA *= 2;
        PaivitaNaytot();
    }


    /// <summary>
    /// Kasvatetaan kolmioiden määrää, jolloin kuvasta tulee valmiimpi
    /// </summary>
    public void VahennaMaaraa()
    {
        PaivitaNaytot(KOLMIOIDEN_MAARA - 1, OLETUS);
    }


    /// <summary>
    /// Tämä tehdään aina kun ruutua pitää päivittää
    /// </summary>
    /// <param name="canvas">Piirtoalue, johon voi piirtää</param>
    protected override void Paint(Canvas canvas)
    {
        base.Paint(canvas);
        double korkeus = 300;
        kolmiota = 0;
        SierpinskinKolmio(canvas, 0, -korkeus + ALKUY, korkeus);
    }


    /// <summary>
    /// Piirtää Sierpinskin kolmion.
    /// </summary>
    /// <param name="canvas">Piirtoalue johon piirretään</param>
    /// <param name="x">kolmion kärjen keskipisteen x-koord</param>
    /// <param name="y">kolmion kärjen keskipisteen y-koord</param>
    /// <param name="h">kolmion korkeus</param>
    public static void SierpinskinKolmio(Canvas canvas, double x, double y, double h)
    {
        if (kolmiota >= KOLMIOIDEN_MAARA) return;

        double s2 = h / Math.Sqrt(3); // sivun pituus s/2

        Vector p1 = new Vector(x, y);
        Vector p2 = new Vector(x - s2, y + h);
        Vector p3 = new Vector(x + s2, y + h);

        canvas.DrawLine(p1, p2);
        canvas.DrawLine(p2, p3);
        canvas.DrawLine(p3, p1);


        kolmiota++;
        if (h < PIENIN_KORKEUS) return;

        SierpinskinKolmio(canvas, x - s2, y, h / 2); // Vasen alakolmio
        SierpinskinKolmio(canvas, x + s2, y, h / 2); // Oikea alakolmio
        SierpinskinKolmio(canvas, x, y + h, h / 2); // Yläkolmio
    }

}
