﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Data;

namespace AutolaskuriBind
{
    /// @author Vesa Lappalainen
    /// @version 24.11.2012
    /// <summary>
    /// Laskuri, joka voidaan sitoa kiinni johonkin kontrollin ominaisuuteen
    /// </summary>
    public class BindLaskuri : UserControl, INotifyPropertyChanged
    {
        private PropertyChangedEventArgs args = new PropertyChangedEventArgs("Count");
        private int count;

        public int Count
        {
            get { return count; }
            set
            {
                count = value;
                if (PropertyChanged != null) PropertyChanged(this, args);
            }
        }

        public static BindLaskuri operator ++(BindLaskuri laskuri)
        {
            laskuri.Count++;
            return laskuri;
        }

        public event PropertyChangedEventHandler PropertyChanged;


        /// <summary>
        /// Selvittää olion Content-ominaisuuteen liitetyn laskurin
        /// </summary>
        /// <param name="sender">Olio jonka liitosta selvitetään</param>
        /// <returns>liitetty laskuri tai null jos ei liitetty</returns>
        public static BindLaskuri GetLaskuri(object sender)
        {
            if (!(sender is Control)) return null;
            Control ctr = (Control)sender;
            Object obj = null;
            if (ctr is Button) obj = ctr.ReadLocalValue(Button.ContentProperty);
            if (ctr is TextBox) obj = ctr.ReadLocalValue(TextBox.TextProperty);
            if (obj == null) return null;
            if (!(obj is BindingExpression)) return null;
            BindingExpression bind = (BindingExpression)obj;
            Object target = bind.DataItem;
            if (!(target is BindLaskuri)) return null;
            return (BindLaskuri)target;
        }

    }
}