using System;
using System.Text;
using Jypeli;
using Jypeli.Widgets;

/// @author Hanna & Heini Kinnunen
/// @version 1.11.2013
/// @author vesal
/// @version 2.11.2013
/// 
/// <summary>
/// Ohjelmassa amebat tanssivat.
/// Ota osoitteesta
///    https://kurssit.it.jyu.fi/ITKP102/content/AmebaDanceWars/Content/
/// seuraavat kuvat Content-kansioon:
///  - oma_tausta.png
///  - taustakuva_lattia.png
///  - vastustajan_tausta.png
/// </summary>
public class AmebaDanceWars : PhysicsGame
{
    // TODO: pitää estää oman ameban tanssiminen kun vastustaja näyttää liikkeitä.
    // TODO: aikalaskurit
    // TODO: Kaikki käyttäjän ohjaaminen
    // TODO: taso vaihtuu kun painelee
    // TODO: omien luokkien kommentointi
    private Image taustakuva = LoadImage("taustakuva_lattia");
    private Image valokeila = LoadImage("valokeila");

    private static string liikkeidenNimet = "voya";
    private static Image[] vastuksenKuvat = { LoadImage("vastus_odotus"), LoadImage("vastus_vasen"), LoadImage("vastus_oikea"), LoadImage("vastus_ylos"), LoadImage("vastus_alas") };
    private static Image[] omatKuvat = { LoadImage("oma_odotus"), LoadImage("oma_vasen"), LoadImage("oma_oikea"), LoadImage("oma_ylos"), LoadImage("oma_alas") };
    /*
    private TanssiAmeba vastusAmeba = new TanssiAmeba(350, 300, liikkeidenNimet, vastuksenKuvat);
    private OmaAmeba omaAmeba = new OmaAmeba(350, 300, liikkeidenNimet, omatKuvat);

    private GameObject spotti = new GameObject(500, 800);
    private IntMeter tasolaskuri = new IntMeter(1);
    */
    private TanssiAmeba vastusAmeba;
    private OmaAmeba omaAmeba;

    private GameObject spotti;
    private IntMeter tasolaskuri = new IntMeter(1);

    public override void Begin()
    {
       vastusAmeba = new TanssiAmeba(350, 300, liikkeidenNimet, vastuksenKuvat);
       omaAmeba = new OmaAmeba(350, 300, liikkeidenNimet, omatKuvat);

       spotti = new GameObject(500, 800);

       Camera.ZoomToLevel();

        Level.Background.Image = taustakuva;
        Level.Background.FitToLevel();
        //   Surfaces reunat = Level.CreateBorders(0.2, true, Color.Black);

        // aloitusseremoniat

        spotti.Position = new Vector(0, 100);
        spotti.Image = valokeila;
        Add(spotti, -1);

        vastusAmeba.Position = new Vector(-230, -100);
        Add(vastusAmeba, 3);
        vastusAmeba.Valmis = delegate { OmaVuoro(); };

        omaAmeba.Position = new Vector(190, -100);
        Add(omaAmeba, 3);
        omaAmeba.Valmis = delegate {
            tasolaskuri.Value++; VastustajanVuoro();
            if (tasolaskuri.Value > 4) Gravity = new Vector(0, -300);
        };

        Label pisteNaytto = new Label();
        pisteNaytto.Position = new Vector(Screen.Right - 350, Screen.Top - 100);
        pisteNaytto.Color = Color.White;
        pisteNaytto.IntFormatString = "Pisteitä: {0,2}";
        pisteNaytto.BindTo(omaAmeba.Pistelaskuri);
        Add(pisteNaytto);

        Label tasoNaytto = new Label(pisteNaytto.Width, pisteNaytto.Height);
        tasoNaytto.Position = new Vector(Screen.Right - 350, pisteNaytto.Bottom - tasoNaytto.Height / 2 - 2);
        tasoNaytto.Color = Color.White;
        tasoNaytto.IntFormatString = "Taso:     {0,2}";
        tasoNaytto.BindTo(tasolaskuri);
        Add(tasoNaytto);

        Keyboard.Listen(Key.F1, ButtonState.Pressed, ShowControlHelp, "Näytä avustus");
        Keyboard.Listen(Key.Space, ButtonState.Pressed, delegate { vastusAmeba.ArvoLiikkeetJaAloita(tasolaskuri.Value); }, "Paina välilyöntiä nähdäksesi vastustajan liikkeet");

        Keyboard.Listen(Key.Left, ButtonState.Pressed, omaAmeba.TeeLiike, "Liike vasemmalle", 'v');
        Keyboard.Listen(Key.Right, ButtonState.Pressed, omaAmeba.TeeLiike, "Liike oikealle", 'o');
        Keyboard.Listen(Key.Up, ButtonState.Pressed, omaAmeba.TeeLiike, "Liike ylös", 'y');
        Keyboard.Listen(Key.Down, ButtonState.Pressed, omaAmeba.TeeLiike, "Liike alas", 'a');

        PhoneBackButton.Listen(ConfirmExit, "Lopeta peli");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");

        for (int i = 0; i < 10; i++) // luodaan taustatanssijoita
        {
            TanssiAmeba tanssiAmeba = new TanssiAmeba(RandomGen.NextDouble(40, 150), RandomGen.NextDouble(30, 130), liikkeidenNimet, vastuksenKuvat);
            tanssiAmeba.Position = RandomGen.NextVector(-500, 0, 500, 300);
            Add(tanssiAmeba);
            tanssiAmeba.Valmis = delegate { tanssiAmeba.ArvoLiikkeetJaAloita(4); };
            tanssiAmeba.ArvoLiikkeetJaAloita(4);
        }

        VastustajanVuoro();
    }


    /// <summary>
    /// Kääntää pelissä taustalla olevan spotin osoittamaan vastustajaa ja käskee käyttäjää käynnistämään vastustajan liikkeet
    /// </summary>
    private void VastustajanVuoro()
    {
        spotti.Angle = Angle.FromDegrees(-25.0);
        MessageDisplay.Add("Paina välilyöntiä");
        MessageDisplay.MessageTime = new TimeSpan(0, 0, 3);
    }


    /// <summary>
    /// Kääntää pelissä taustalla olevan spotin osoittamaan omaa amebaa ja kertoo vuoron olevan pelaajalla
    /// </summary>
    private void OmaVuoro()
    {
        spotti.Angle = Angle.FromDegrees(25.0);
        MessageDisplay.Add("Sinun vuorosi");
        MessageDisplay.MessageTime = new TimeSpan(0, 0, 3);
        omaAmeba.Aloita(vastusAmeba.ArvotutLiikkeet);
    }
}


public class Ameba : PhysicsObject
{
    public delegate void SuoritusValmis(Ameba kuka);

    private Image[] kuvat;
    private string liikkeidenNimet;
    public SuoritusValmis Valmis { get; set; }
    public string LiikkeidenNimet { get { return liikkeidenNimet; } }


    public Ameba(double w, double h, string liikkeidenNimet, Image[] kuvat) : base(h, w)
    {
        this.kuvat = kuvat;
        this.Image = kuvat[0];
        this.liikkeidenNimet = liikkeidenNimet;
        this.Valmis = Tyhja;
    }


    public virtual void TeeLiike(char liike)
    {
        int i = liikkeidenNimet.IndexOf(liike) + 1;
        if (i >= kuvat.Length) return;
        Image = kuvat[i];
        Timer.SingleShot(0.5, delegate { Odotusliike(); });
    }


    protected virtual void Odotusliike()
    {
        this.Image = kuvat[0];
    }


    private void Tyhja(Ameba kuka) { }
}


public class OmaAmeba : Ameba
{
    private string matkittavatLiikkeet = "";
    private string painetutLiikkeet = "";
    private IntMeter pistelaskuri = new IntMeter(0);

    public IntMeter Pistelaskuri { get { return pistelaskuri; } }

    public OmaAmeba(double w, double h, string nimet, Image[] kuvat) : base(h, w, nimet, kuvat) { }


    public override void TeeLiike(char liike)
    {
        if (painetutLiikkeet.Length >= matkittavatLiikkeet.Length) return;
        base.TeeLiike(liike);
        painetutLiikkeet += liike;
    }


    protected override void Odotusliike()
    {
        base.Odotusliike();
        if (painetutLiikkeet.Length > matkittavatLiikkeet.Length) return;
        int i = painetutLiikkeet.Length - 1;
        if (i < 0) return;
        if (matkittavatLiikkeet[i] == painetutLiikkeet[i]) Pistelaskuri.Value += 1;
        if (matkittavatLiikkeet == painetutLiikkeet) Valmis(this);
    }


    public void Aloita(string matkittavatLiikkeet)
    {
        painetutLiikkeet = "";
        this.matkittavatLiikkeet = matkittavatLiikkeet;
    }
}



public class TanssiAmeba : Ameba
{
    private int seuraavaLiikenumero = 0;
    private string arvotutLiikkeet;
    public string ArvotutLiikkeet { get { return arvotutLiikkeet; } }

    public TanssiAmeba(double w, double h, string nimet, Image[] kuvat) : base(h, w, nimet, kuvat) { }


    protected override void Odotusliike()
    {
        base.Odotusliike();
        Timer.SingleShot(0.5, delegate { KaynnistaLiike(seuraavaLiikenumero); });
    }


    private void KaynnistaLiike(int liikenumero)
    {
        if (liikenumero >= arvotutLiikkeet.Length) { Valmis(this); return; }
        TeeLiike(arvotutLiikkeet[liikenumero]);
        seuraavaLiikenumero = liikenumero + 1;
    }


    public void ArvoLiikkeetJaAloita(int maara)
    {
        StringBuilder arvotutLiikkeet = new StringBuilder();
        for (int i = 0; i < maara; i++)
        {
            int indeksi = RandomGen.NextInt(LiikkeidenNimet.Length);
            arvotutLiikkeet.Append(LiikkeidenNimet[indeksi]);
        }
        this.arvotutLiikkeet = arvotutLiikkeet.ToString();
        KaynnistaLiike(0);
    }
}
