﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Lajittelu
{
    /// @author  Vesa Lappalainen
    /// @version 05.11.2011
    /// <summary>
    /// Tutkitaan kokonaislukujen lajittelemiseen kuluvaa aikaa.
    /// Mittaukset tehty vesah3-koneella 6.11.2011.
    /// Verrattu Java ja C#:n lajiteluaikaa erilaisille taulukoille, joissa
    /// on ollut luvut järjestettynä etu ja takaperin, vähän erilaisia
    /// alkiota ja satunnainen täysin satunnainen määrä alkioita.
    /// Javassa on käytetty Arrays-luokan sort-metodia.
    /// C#:issa Array-luokan Sort-metodia.
    /// Lisäksi on tehty viritetty QuickSort, joka pyrkii nopeuteen
    /// taulukon lajitelluilla osilla. Tämä on merkitty testissä QS.
    /// n = taulukon koko. m = erilaisten taulukon alkioiden lukumäärä.
    /// Kaikissa testeissä on käytetty omatekoista generaattoria, joka
    /// tuottaa kielestä riippumatta samanlaisen taulukon.
    /// 
    /// Vertailtava metodi vaihdetaan Jarjesta-aliohjelmassa
    /// </summary>
    /// <remarks>
    /// <code>
    ///      n      m          Java Arrays    Java QS        C# Array        C# QS
    /// Järjestetty: ==================================================================
    ///        5         :       0      0      0      0        0      0       1      0
    ///       50         :       0      0      0      0        0      0       0      0
    ///      500         :       0      0      0      0        0      0       0      0
    ///     5000         :       7      1      1      0        0      0       0      0
    ///    50000         :       2      3      5      0        1      2       1      0
    ///   500000         :      21     21      2      0       48     12       1      1
    ///  5000000         :     240    240      5      5      232    141       8      8
    /// 50000000         :    2554   2557     91     41     1649   1660      83     80
    /// 
    /// Vähän erilaisia alkioita: =====================================================
    ///        5        5:       0             0               0              0
    ///       50        5:       0             0               0              0
    ///      500        5:       0             0               0              0
    ///     5000        5:       0             0               0              0
    ///    50000        5:       1             1               4              1
    ///   500000        5:       8            10              29              9
    ///  5000000        5:      83           100             333             90
    /// 50000000        5:     961          1025            3572            936
    /// 
    /// Satunnaisia lukuja: ===========================================================
    ///        5        5:       0             0               0              0
    ///       50       50:       0             0               0              0
    ///      500      500:       0             0               0              0
    ///     5000     5000:       1             1               1              1
    ///    50000    50000:       6            12               5              4
    ///   500000   500000:      74            60              57             54
    ///  5000000  5000000:     864           711             640            613
    /// 50000000 50000000:    9919          7852            7181           6956
    /// 
    /// Erilaisten alkioiden määrän vaikutus ==========================================
    /// 50000000        1:     140            41            3119             81
    /// 50000000        5:     952          1043            3586            933
    /// 50000000       50:    1980          1971            4203           1753
    /// 50000000      500:    3086          2963            4500           2569
    /// 50000000     5000:    4260          3785            4908           3323
    /// 50000000    50000:    5446          4771            5288           4136
    /// 50000000   500000:    6702          5912            5822           5269
    /// 50000000  5000000:    8376          7223            6696           6331
    /// 50000000 50000000:    9840          7838            7213           6914
    /// 
    /// vesah3 8.11.2015: (i7-3770k, 3.5GHz, Java 1.8.0_60)
    /// Erilaisten alkioiden määrän vaikutus: ======================================
    /// 50000000        1:      42            20            2614             54
    /// 50000000        5:     432           631            2962            758
    /// 50000000       50:    1063          1278            3357           1442
    /// 50000000      500:    1592          1835            3721           2149
    /// 50000000     5000:    2230          2408            4025           2734
    /// 50000000    50000:    2819          3053            4399           3555
    /// 50000000   500000:    3474          3736            4737           4284
    /// 50000000  5000000:    4299          4626            5189           5394
    /// 50000000 50000000:    4545          5032            5300           5921
    /// 
    /// volga2 8.11.2015: ((i7-5600U, 2.6GHz)
    ///     Erilaisten alkioiden määrän vaikutus: ========
    /// 50000000        1:                                  2336             47
    /// 50000000        5:                                  2832            891
    /// 50000000       50:                                  3359           1702
    /// 50000000      500:                                  3823           2562
    /// 50000000     5000:                                  4272           3312
    /// 50000000    50000:                                  4844           4254
    /// 50000000   500000:                                  5297           5153
    /// 50000000  5000000:                                  5957           6572
    /// 50000000 50000000:                                  6078           7219    
    /// 
    /// </code>
    /// Huomioita: C#:in valmis lajittelu häviää Javalle jos aineisto koostuu
    /// suuresta määrästä alkiota, joissa erilaisia ei ole kovin montaa ( n >> m ).
    /// Omatekoinen QS voittaa ylivoimaisesti jos taulukko
    /// on jo järjestyksessä. 
    /// </remarks>
    /// 
    public class IntLajittelu
    {
        /// <summary>
        /// Kokeillaan miten kauan lajittelu kestää
        /// </summary>
        /// <param name="args"></param>
        public static void IntMain(string[] args)
        {
            /*
            */
            Console.WriteLine("\nJärjestetty: =================================");
            AjoitaJarjestetty(5);
            AjoitaJarjestetty(50);
            AjoitaJarjestetty(500);
            AjoitaJarjestetty(5000);
            AjoitaJarjestetty(50000);
            AjoitaJarjestetty(500000);
            AjoitaJarjestetty(5000000);
            AjoitaJarjestetty(50000000);

            Console.WriteLine("\nVähän erilaisia alkioita: ====================");
            Ajoita(5, 5);
            Ajoita(50, 5);
            Ajoita(500, 5);
            Ajoita(5000, 5);
            Ajoita(50000, 5);
            Ajoita(500000, 5);
            Ajoita(5000000, 5);
            Ajoita(50000000, 5);

            Console.WriteLine("\nSatunnaisia lukuja: ==========================");
            Ajoita(5, 5);
            Ajoita(50, 50);
            Ajoita(500, 500);
            Ajoita(5000, 5000);
            Ajoita(50000, 50000);
            Ajoita(500000, 500000);
            Ajoita(5000000, 5000000);
            Ajoita(50000000, 50000000);
            //Ajoita(500000000, 500000000);

            Console.WriteLine("\nErilaisten alkioiden määrän vaikutus: ========");
            Ajoita(50000000, 1);
            Ajoita(50000000, 5);
            Ajoita(50000000, 50);
            Ajoita(50000000, 500);
            Ajoita(50000000, 5000);
            Ajoita(50000000, 50000);
            Ajoita(50000000, 500000);
            Ajoita(50000000, 5000000);
            Ajoita(50000000, 50000000);
            Console.ReadKey();
        }


        /// <summary>
        /// Otetaan aikaa kun lajitellaan taulukko, jossa n kpl nousevasti
        /// ja laskevasti järjestettyjä alkioita.
        /// </summary>
        /// <param name="n">kuinka iso taulukko</param>
        public static void AjoitaJarjestetty(int n)
        {
            Console.Write("{0,8} {1,8}: ", n, " ");
            System.GC.Collect();
            int[] luvut = Utilities.RandomGen.Tayta(n, 0, -1, 1, 1);
            Utilities.StopTimer timer = new Utilities.StopTimer();
            Jarjesta(luvut);
            Console.Write("{0,7:0}", timer.Lap());
            Tarkista(luvut);
            luvut = null;
            System.GC.Collect();
            luvut = Utilities.RandomGen.Tayta(n, 0, -1, 1, 1);
            timer.Start();
            Jarjesta(luvut);
            Console.WriteLine("{0,7:0} ", timer.Lap());
            Tarkista(luvut);
        }


        /// <summary>
        /// Otetaan aikaa kun lajitellaan taulukko, jossa n kpl m erilaista alkiota
        /// </summary>
        /// <param name="n">kuinka iso taulukko</param>
        /// <param name="m">montako </param>
        public static void Ajoita(int n, int m)
        {
            Console.Write("{0,8} {1,8}: ", n, m);
            System.GC.Collect();
            int[] luvut = Utilities.RandomGen.Tayta(n, m);
            Utilities.StopTimer timer = new Utilities.StopTimer();
            Jarjesta(luvut);
            Console.WriteLine("{0,7:0} ", timer.Lap());
            Tarkista(luvut);
        }


        /// <summary>
        /// Lajitellaan taulukko
        /// </summary>
        /// <param name="luvut">lajiteltava taulukko</param>
        public static void Jarjesta(int[] luvut)
        {
            //Array.Sort(luvut);
            QuickSort(luvut);
        }


        /// <summary>
        /// Tarkistaa onko taulukko järjestyksessä
        /// </summary>
        /// <param name="luvut">tarkistettava taulukko</param>
        public static void Tarkista(int[] luvut)
        {
            for (int i = 0; i < luvut.Length - 1; i++)
                if (luvut[i] > luvut[i + 1])
                {
                    Console.WriteLine("VÄÄRIN: {0} > {1}", luvut[i], luvut[i + 1]);
                    return;
                }
        }


        /// <summary>
        /// Lajitellaan taulukko käyttäen rekursiivista QuickSorttia
        /// </summary>
        /// <param name="a">lajiteltava taulukko, taulukko muuttuu</param>
        /// <returns>viite lajiteltuun taulukkoon</returns>
        /// <example>
        /// <pre name="test">
        ///   int[] luvut = {3,6,2,9,3,1};
        ///   IntLajittelu.QuickSort(luvut);
        ///   String.Join(" ",luvut) === "1 2 3 3 6 9";
        /// </pre>
        /// </example>
        public static void QuickSort(int[] a)
        {
            QuickSort(a, 0, a.Length - 1);
        }


        /// <summary>
        /// Lajitellaan osataulukko väliltä [alku,loppu]
        /// </summary>
        /// <param name="a">lajiteltava taulukko, muuttuu</param>
        /// <param name="alku">lajitteltavan alueen alku</param>
        /// <param name="loppu">alueen loppu</param>
        /// <returns>viite lajiteltuun taulukkoon</returns>
        /// <example>
        /// <pre name="test">
        ///   int[] luvut = {3,6,2,9,3,1};
        ///   IntLajittelu.QuickSort(luvut,0,2);
        ///   String.Join(" ",luvut) === "2 3 6 9 3 1";
        /// </pre>
        /// </example>
        public static void QuickSort(int[] a, int alku, int loppu)
        {
            while (alku < loppu)
            {
                if (Check(a, alku, loppu)) return;
                int q = Partition(a, alku, loppu);
                QuickSort(a, alku, q);
                //QuickSort(a, q + 1, loppu); // Vaihdetaan häntärekursioksi, niin pino riittää
                alku = q + 1;
            }
        }


        /// <summary>
        /// Tarkistetaan että onko taulukon osa järjestyksessä.
        /// Aluksi katsotaan 2,3,4 kokoiset erikoistapauksena ja järjestettään ne.
        /// Isomista palautetaan vain tieto että onko taulukko järjestetty vai ei.
        /// </summary>
        /// <param name="a">tutkittava taulukko, sisältä voi muuttua</param>
        /// <param name="alku">tarkastelun alkuindeksi</param>
        /// <param name="loppu">tarkastelun loppuindeksi</param>
        /// <returns>true jos taulukko järjestyksessä.</returns>
        /// <example>
        /// <pre name="test">
        ///   int[] luvut = {1,2};
        ///   IntLajittelu.Check(luvut,0,1) === true;
        ///   luvut = new int[]{2,1};
        ///   IntLajittelu.Check(luvut,0,1) === true;
        ///   string.Join(" ",luvut) === "1 2";
        ///   luvut = new int[]{1,2,3};
        ///   IntLajittelu.Check(luvut,0,2) === true;
        ///   luvut = new int[]{3,1,2};
        ///   IntLajittelu.Check(luvut,0,2) === true;
        ///   string.Join(" ",luvut) === "1 2 3";
        ///   luvut = new int[]{1,2,3,4};
        ///   IntLajittelu.Check(luvut,0,3) === true;
        ///   luvut = new int[]{3,1,4,2};
        ///   IntLajittelu.Check(luvut,0,3) === true;
        ///   string.Join(" ",luvut) === "1 2 3 4";
        ///   luvut = new int[]{3,1,5,4,2};
        ///   IntLajittelu.Check(luvut,0,4) === false;
        ///   string.Join(" ",luvut) === "3 1 5 4 2";
        /// </pre>
        /// </example>
        public static bool Check(int[] a, int alku, int loppu)
        {
            switch (loppu - alku)
            {
                case 1:
                    {
                        int a1 = a[alku];
                        int a2 = a[loppu];
                        if (a1 <= a2) return true;
                        a[alku] = a2;
                        a[loppu] = a1;
                        return true;
                    }
                case 2:
                    {
                        int a1 = a[alku];
                        int a2 = a[alku + 1];
                        int a3 = a[loppu];
                        if (a1 <= a2 && a2 <= a3) return true;
                        if (a1 > a2) { a1 = a2; a2 = a[alku]; }
                        if (a2 > a3) { a[loppu] = a2; a2 = a3; }
                        if (a1 > a2) { a[alku] = a2; a[alku + 1] = a1; }
                        else { a[alku] = a1; a[alku + 1] = a2; }
                        return true;
                    }
                case 3:
                    {
                        int a1 = a[alku];
                        int a2 = a[alku + 1];
                        int a3 = a[loppu - 1];
                        int a4 = a[loppu];
                        if (a1 <= a2 && a2 <= a3 && a3 <= a4) return true;
                        if (a1 > a2) { a1 = a2; a2 = a[alku]; }
                        if (a2 > a3) { a3 = a2; a2 = a[loppu - 1]; }
                        if (a3 > a4) { a[loppu] = a3; a3 = a4; }
                        a[loppu - 1] = a3;
                        if (a1 > a2) { int tmp = a1; a1 = a2; a2 = tmp; }
                        if (a2 > a3) { a[loppu - 1] = a2; a2 = a3; }
                        if (a1 > a2) { a[alku] = a2; a[alku + 1] = a1; }
                        else { a[alku] = a1; a[alku + 1] = a2; }
                        return true;
                    }
            }
            for (int i = alku; i < loppu; i++)
                if (a[i] > a[i + 1]) return false;
            return true;
        }

        /// <summary>
        /// Etsitään jakopaikka (pivot).  Eli alkio, jonka vasemmalla
        /// puolella kaikki ovat alkiota pienempiä ja oikealla puolella
        /// suurempia. Etsinnän aikana vaihdetaan alkioiden paikkoja.
        /// </summary>
        /// <param name="a">tutkittava taulukko</param>
        /// <param name="vasen">osan alku</param>
        /// <param name="oikea">osan loppu</param>
        /// <returns>jakopaikka</returns> 
        /// <example>
        /// <pre name="test">
        ///   int[] luvutj = {1,2,3,4,5,6,7,8,9};
        ///   IntLajittelu.Partition(luvutj,0,8) === 4;
        ///   String.Join(" ",luvutj) === "1 2 3 4 5 6 7 8 9";
        ///   int[] luvut = {3,7,8,5,2,1,9,5,4};
        ///   IntLajittelu.Partition(luvut,0,8) === 1;
        ///   String.Join(" ",luvut) === "1 2 8 5 7 3 9 5 4";
        /// </pre>
        /// </example>
        public static int Partition(int[] a, int vasen, int oikea)
        {
            int pivotIndex = (vasen + oikea) / 2;
            // int pivotIndex = vasen; // huono järjestetylle joukolle
            // int pivotIndex = MiidiIndex(a, vasen, oikea); // nostaa kerrointa koska hidasta
            /* // Etistään kolmen luvun mediaani, ei paljonkaan auta
            int i0 = vasen;
            int i1 = pivotIndex;
            int i2 = oikea;
            if (a[i0] > a[i1]) { i0 = pivotIndex; i1 = vasen; }
            if (a[i1] > a[i2]) { i2 = i1; i1 = oikea; }
            if (a[i0] > a[i1]) { i1 = i0; }
            pivotIndex = i1;
            */

            int pivotValue = a[pivotIndex];
            int iv = vasen - 1;
            int io = oikea + 1;
            int tmp = 0;
            while (true)
            {
                do
                {
                    io--;
                } while (a[io] > pivotValue);
                do
                {
                    iv++;
                } while (a[iv] < pivotValue);

                if (iv >= io) return io;

                tmp = a[iv];
                a[iv] = a[io];
                a[io] = tmp;
            }
        }

        /*
        private static int MiidiIndex(int[] a, int vasen, int oikea)
        {
            double summa = 0;
            for (int i = vasen; i <= oikea; i++) summa += a[i];
            int ind = vasen;
            double lahinEt = double.MaxValue;
            double ka = summa / (oikea - vasen + 1);
            for (int i = vasen; i <= oikea; i++)
            {
                int alkio = a[i];
                double et = Math.Abs(ka - alkio); 
                if ( et < lahinEt)
                {
                    ind = i;
                    lahinEt = et;
                }
            }
            return ind;
        }
         */

    }
}
