using System;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Widgets;
using Jypeli.WP7;


/// @author  Vesa Lappalainen
/// @version 14.08.2011
/// <summary>
/// Peli, jossa kaksi tankkia ampuu päättömästi toisiaan.  
/// Puut ja muut esteet haittaavat keskittymistä.
/// </summary>
public class Tankkipeli : PhysicsGame
{
    private Tank tankki1; // Tankit
    private Tank tankki2;
    private Label point1; // Pelaajien pisteet
    private Label point2;
    private Label ammo1;  // Ammusten määrät
    private Label ammo2;
    private int vaantoVoima = 90000; // millä voimalla moottori vääntää


    /// <summary>
    /// Pelin alustus
    /// </summary>
    public override void Begin()
    {
        Camera.Reset();
        LuoTaso();
        Camera.ZoomToLevel(); // zoomaa kameran siten, että koko taso näkyy ruudussa isona
#if !WINDOWS_PHONE
        ShowControlHelp();
#endif
    }


    /// <summary>
    /// Luodaan ensimmäinen taso
    /// </summary>
    /// <returns>luotu taso</returns>
    private Level LuoTaso()
    {
        Level taso = Level;
#if WINDOWS_PHONE
        taso.Width = 800;
        taso.Height = 480;
        Phone.DisplayResolution = DisplayResolution.Large;
#else
        taso.Width = 800;
        taso.Height = 600;
#endif
        taso.CreateBorders();
        //SetWindowSize(800,600);
        taso.Background.Image = LoadImage("tausta3");
        double alinMaanKorkeus = 10.0;
        double ylinMaanKorkeus = 50.0;

        tankki1 = LuoTankki(-350, ylinMaanKorkeus, 0);
        tankki2 = LuoTankki(350, ylinMaanKorkeus, 180);

        double alkuY = Level.Bottom + ylinMaanKorkeus + 150.0;

        // taso.CreateGround(alinMaanKorkeus, ylinMaanKorkeus, 20, null ); //LoadImage("maasto")); // TÄTÄ EI ONNISTU PIIRTÄMÄÄN?
        LuoMaasto(alinMaanKorkeus, ylinMaanKorkeus);

        LuoEste("seina", 200, 100, 1, 0, alkuY, "tiiliseina", 30);
        Este talo = LuoEste("talo", 70, 70, 1, 0, alkuY + 100, "talo", 30);
        talo.Shape = Shape.FromImage(talo.Image);
        LuoKuulia(0, alkuY + 200, 3, 5, 30);
        LuoPuita(0, taso.Bottom + 150, 10, 20, 50, 100, "isopuu", 4);
        LuoPuita(0, taso.Bottom + 250, 12, 10, 25, 100, "puu", 2);

        LuoNaytot();

        AsetaOhjaimet();

        return taso;
    }


    /// <summary>
    /// Luodaan pistelaskurit elämästä, ammuksista ja liukupalkit tankin voimasta
    /// </summary>
    private void LuoNaytot()
    {
        int koko = 130;
        int h = 22;
        point1 = LuoNaytto("Life: ", tankki1.HitPoints, new Vector(Screen.Left, Screen.Top - h / 2), koko);
        point2 = LuoNaytto("Life: ", tankki2.HitPoints, new Vector(Screen.Right, Screen.Top - h / 2), -koko);
        ammo1 = LuoNaytto("Ammo: ", tankki1.Ammo, new Vector(Screen.Left, Screen.Top - 3 * h / 2), koko);
        ammo2 = LuoNaytto("Ammo: ", tankki2.Ammo, new Vector(Screen.Right, Screen.Top - 3 * h / 2), -koko);
        LuoLiuku(tankki1.Cannon.Power, new Vector(Screen.Left, Screen.Top - 5 * h / 2 + h / 4), h / 2, koko);
        LuoLiuku(tankki2.Cannon.Power, new Vector(Screen.Right, Screen.Top - 5 * h / 2 + h / 4), h / 2, -koko);
    }


    /// <summary>
    /// Luodaan yksi liukunäytin
    /// </summary>
    /// <param name="mittari">mittari jonka arvoa liuku näyttää</param>
    /// <param name="position">liun paikka</param>
    /// <param name="korkeus">kuinka korkea liuku on</param>
    /// <param name="offset">paljonko paikasta siirretään liukua</param>
    /// <returns>luotu liuku</returns>
    private ProgressBar LuoLiuku(DoubleMeter mittari, Vector position, int korkeus, int offset)
    {
        ProgressBar liuku = new ProgressBar(Math.Abs(offset), korkeus);
        liuku.Position = position + new Vector(offset / 2, 0);
        liuku.BindTo(mittari);
        //liuku.BorderColor = Color.Yellow;
        liuku.Color = Color.LightYellow;
        // liuku.Direction = BarGauge.BarDirection.BarHorizontalLeft;
        // liuku.Direction = BarGauge.BarDirection.BarVerticalDown;
        Add(liuku);
        return liuku;
    }


    /// <summary>
    /// Luodaan yksi tekstinäyttö
    /// </summary>
    /// <param name="teksti">Mikä teksti tulee näyttöön</param>
    /// <param name="mittari">Minkä mittarin arvoa näytetään</param>
    /// <param name="position">Mihin paikkaan teksti laitetaan</param>
    /// <param name="offset">paljonko siirrytään paikasta</param>
    /// <returns>luotu tekstinäyttö</returns>
    private Label LuoNaytto(string teksti, IntMeter mittari, Vector position, int offset)
    {
        Label naytto = new Label("000");
        naytto.SizeMode = TextSizeMode.None;
        naytto.Width = Math.Abs(offset);
        // naytto.Font = LoadFont("Segoe UI Mono");
        // naytto.Font = Font.DefaultSmall;
        naytto.IntFormatString = " " + teksti + " {0,2:D} ";
        naytto.HorizontalAlignment = HorizontalAlignment.Left;
        naytto.Position = position + new Vector(offset / 2, 0);
        naytto.TextColor = Color.Black;
        naytto.Color = Color.Yellow;
        naytto.BindTo(mittari);

        mittari.MinValue = 0;
        mittari.AddTrigger(2, TriggerDirection.Down, delegate() { naytto.TextColor = Color.Red; });

        Add(naytto);
        return naytto;
    }


    /// <summary>
    /// Mitä tehdään kun ammus on osunut.  Ammus räjähtää joka tapauksessa.
    /// Miinaan törmättäessä miina räjähtää.
    /// Muusta esteestä vähennetään kestoa ja jos se on loppu, niin kohdekin häviää.
    /// Tankin tapauksessa tankin kestämät osumat vähenevät
    /// </summary>
    /// <param name="ammus">ammus joka osuu</param>
    /// <param name="kohde">kohde johon ammus osui</param>
    public void AmmusOsui(PhysicsObject ammus, PhysicsObject kohde)
    {
        ammus.Destroy();
        Explosion rajahdys = new Explosion(30);
        rajahdys.Position = ammus.Position;
        Add(rajahdys);
        if (kohde is CannonBall)
        {
            kohde.Destroy();
        }
        if (kohde is Este)
        {
            Este e = (Este)kohde;
            e.Kesto--;
            if (e.Kesto <= 0) kohde.Destroy();
        }
        if (kohde is Tank)
        {
            Explosion rajahdys2 = new Explosion(60);
            rajahdys2.Position = ammus.Position;
            rajahdys2.Speed = 70;
            Add(rajahdys2);
            (((Tank)kohde).HitPoints.Value)--;
        }
    }


    /// <summary>
    /// Käsitellään kuulan osuma.  Jos kuula osuu esteeseen,
    /// menettää este kestoaan jos se on jo heikentynyt.
    /// Keston lopuessa este räjähtää.
    /// Tankin tapauksessa kuula räjähtää ja tankki menettää osumakestävyyttään
    /// kahdella pisteellä.
    /// </summary>
    /// <param name="kuula"></param>
    /// <param name="kohde"></param>
    public void KuulaOsui(PhysicsObject kuula, PhysicsObject kohde)
    {
        if (kohde is Este)
        {
            Este e = (Este)kohde;
            if (e.Kesto > 10) return;
            e.Kesto -= 1;
            if (e.Kesto <= 0)
            {
                Explosion rajahdys = new Explosion(50);
                rajahdys.Position = kuula.Position;
                Add(rajahdys);
                kuula.Destroy();
                kohde.Destroy();
            }
        }
        if (kohde is Tank)
        {
            Explosion rajahdys = new Explosion(50);
            rajahdys.Position = kuula.Position;
            rajahdys.Speed = 100;

            Add(rajahdys);
            kuula.Destroy();
            ((Tank)kohde).HitPoints.Value -= 2;
        }
    }


    /// <summary>
    /// Luodaan yksi tankki
    /// </summary>
    /// <param name="x">tankin x-koordinaatti</param>
    /// <param name="y">tankin y-koodrinaatti</param>
    /// <param name="kulma">putke suuntauskulma asteina</param>
    /// <returns>luotu tankki</returns>
    private Tank LuoTankki(double x, double y, double kulma)
    {
        Tank tankki = new Tank(100, 50);
        tankki.X = x;
        tankki.Y = Level.Bottom + y + 20.0;
        tankki.Cannon.Angle = Angle.FromDegrees(kulma);
        tankki.Mass = 1e3;
        tankki.Cannon.ProjectileCollision += AmmusOsui;
        tankki.Cannon.CanHitOwner = true;
        tankki.Cannon.Power.DefaultValue = 0;
        tankki.Cannon.Power.MaxValue = 15000;
        tankki.Cannon.Power.MinValue = 0;
        tankki.Cannon.Power.Reset();
        tankki.Ammo.Value = 50;
        Add(tankki);
        return tankki;
    }


    /// <summary>
    /// Luodaan yksi este
    /// </summary>
    /// <param name="nimi">esteelle annettava tunnistenimi</param>
    /// <param name="leveys">esteen leveys</param>
    /// <param name="korkeus">esteen korkeus</param>
    /// <param name="massa">esteen massa</param>
    /// <param name="x">paikan x</param>
    /// <param name="y">paikan y</param>
    /// <param name="texture">ladattavan kuvan nimi</param>
    /// <param name="kesto">esteen kesto alkutilanteessa</param>
    /// <returns>luotu este</returns>
    private Este LuoEste(string nimi, double leveys, double korkeus, double massa, double x, double y, string texture, int kesto)
    {
        Este este = new Este(nimi, leveys, korkeus, massa, kesto);
        este.Image = LoadImage(texture);
        este.Y = y;
        este.Y = y;
        Add(este);
        return este;
    }


    /// <summary>
    /// Luodaan maasto
    /// </summary>
    /// <param name="pohja">maaston pohjan korkeus</param>
    /// <param name="korkeus">maaston yläreunan korkeus</param>
    private void LuoMaasto(double pohja, double korkeus)
    {
        double leveys = Level.Width;
        double[] heights = {
             pohja,
             pohja,
             pohja,
             korkeus * 0.3,
             korkeus * 0.5,
             korkeus * 0.2,
             korkeus * 0.5,
             korkeus * 0.9,
             korkeus * 1.35,
        };
        Surface ground = new Surface(leveys, Peilaa(heights, 0), 1.0);
        ground.Y = Level.Bottom;
        ground.Color = Color.DarkGreen;
        Add(ground);
    }


    /// <summary>
    /// Luodaan joukko kuulia kentälle
    /// </summary>
    /// <param name="keskipisteX">kuula-alueen keskipiste</param>
    /// <param name="alkuY">kuula-alueen alareuna</param>
    /// <param name="maaraX">montako kuulaa X-suunnassa</param>
    /// <param name="maaraY">montako kuulaa Y-suunnassa</param>
    /// <param name="koko">kuulan halkaisija</param>
    private void LuoKuulia(double keskipisteX, double alkuY, int maaraX, int maaraY, int koko)
    {
        for (int x = 0; x < maaraX; x++)
        {
            for (int y = 0; y < maaraY; y++)
            {
                CannonBall kuula = new CannonBall(koko / 2);
                //kuula.ExplodeCondition = Projectile.IsBattleObject;
                //kuula.Explosion.Force = 20;
                AddCollisionHandler(kuula, KuulaOsui);
                kuula.Y = alkuY + y * (koko);
                kuula.X = keskipisteX - koko * (maaraX - 1) / 2 + x * koko;
                kuula.Tag = "kuula";
                Add(kuula);
            }
        }
    }


    /// <summary>
    /// Luodaan tasolle puita
    /// </summary>
    /// <param name="keskipisteX">puualueen x-keskipiste</param>
    /// <param name="alkuY">puualueen alalaita</param>
    /// <param name="puidenMaara">montako puuta luodaan</param>
    /// <param name="kokoX">puun leveys</param>
    /// <param name="kokoY">puun korkeus</param>
    /// <param name="puidenVali">puiden välimatka</param>
    /// <param name="tag">puille annettava nimi</param>
    /// <param name="kesto">alkuarvo kestolle</param>
    private void LuoPuita(double keskipisteX, double alkuY, int puidenMaara, int kokoX, int kokoY, int puidenVali, string tag, int kesto)
    {
        Image puuTexture = LoadImage("puu");
        Shape puuShape = Shape.FromImage(puuTexture);

        for (int i = 0; i < puidenMaara; i++)
        {
            Este puu = new Este("puu" + i + "-" + alkuY, kokoX, kokoY, 1, kesto);
            puu.Tag = tag;
            puu.Image = puuTexture;
            puu.Shape = puuShape;
            puu.Y = alkuY;
            puu.X = keskipisteX - puidenVali * (puidenMaara - 1) / 2 + i * puidenVali;
            Add(puu);
            AddCollisionHandler(puu, OsuPuuhun);
            puu.MomentOfInertia = 10000;
            puu.Body.IgnoresGravity = true;
        }
    }


    /// <summary>
    /// Luodaan pelissä tarvittavat ohjaimet näppäimistölle,
    /// XBox peliohjaimelle sekä kosketusnäytölle.
    /// </summary>
    private void AsetaOhjaimet()
    {
        double voima = vaantoVoima;
        Keyboard.Listen(Key.F1, ButtonState.Pressed, ShowControlHelp, "Tämä avustus");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, LopetaPeli, "Lopeta peli");
        Keyboard.Listen(Key.F5, ButtonState.Pressed, AloitaPeli, "Aloita peli");

        Keyboard.Listen(Key.E, ButtonState.Down, ViritaTykki, "Pelaaja1: Ammu", tankki1);
        Keyboard.Listen(Key.E, ButtonState.Released, Ammu, null, tankki1);
        Keyboard.Listen(Key.W, ButtonState.Down, KaannaPutkea, "Pelaaja1: Käännä tykkiä vasemmalle", tankki1, 1);
        Keyboard.Listen(Key.S, ButtonState.Down, KaannaPutkea, "Pelaaja1: Käännä tykkiä oikealle", tankki1, -1);
        Keyboard.Listen(Key.A, ButtonState.Down, AjaTankkia, "pelaaja1: Aja tankkia vasemmalle", tankki1, voima);
        Keyboard.Listen(Key.D, ButtonState.Down, AjaTankkia, "pelaaja1: Aja tankkia oikealle", tankki1, -voima);
        Keyboard.Listen(Key.R, ButtonState.Pressed, KaannaTankkiOikeinpain, "Pelaaja1: Käännä tankki ympäri", tankki1);
        asetaGamePadOhjaimet(ControllerOne, tankki1);

        Keyboard.Listen(Key.Delete, ButtonState.Down, ViritaTykki, "Pelaaja2: Ammu", tankki2);
        Keyboard.Listen(Key.Delete, ButtonState.Released, Ammu, null, tankki2);
        Keyboard.Listen(Key.Up, ButtonState.Down, KaannaPutkea, "Pelaaja2: Käännä tykkiä vasemmalle", tankki2, -1);
        Keyboard.Listen(Key.Down, ButtonState.Down, KaannaPutkea, "Pelaaja2: Käännä tykkiä oikealle", tankki2, 1);
        Keyboard.Listen(Key.Left, ButtonState.Down, AjaTankkia, "pelaaja2: Aja tankkia vasemmalle", tankki2, voima);
        Keyboard.Listen(Key.Right, ButtonState.Down, AjaTankkia, "pelaaja2: Aja tankkia oikealle", tankki2, -voima);
        Keyboard.Listen(Key.PageDown, ButtonState.Pressed, KaannaTankkiOikeinpain, "Pelaaja2: Käännä tankki ympäri", tankki2);
        asetaGamePadOhjaimet(ControllerTwo, tankki2);
        LisaaKosketusKontrollit();
    }


    /// <summary>
    /// Asetetaan XBox-ohjaimet
    /// </summary>
    /// <param name="controller"></param>
    /// <param name="tankki"></param>
    protected void asetaGamePadOhjaimet(GamePad controller, Tank tankki)
    {
        controller.Listen(Button.B, ButtonState.Pressed, ShowControlHelp, "Avustus");
        controller.Listen(Button.Start, ButtonState.Down, AloitaPeli, "Aloita peli");
        controller.Listen(Button.Back, ButtonState.Down, LopetaPeli, "Lopeta peli");
        controller.ListenAnalog(AnalogControl.LeftStick, 0.03, AjaTankkiaAnalogisesti, "Aja tankkia", tankki);
        controller.ListenAnalog(AnalogControl.RightStick, 0.03, KaannaPutkeaAnalogisesti, "Suuntaa tykki", tankki);
        controller.Listen(Button.Y, ButtonState.Pressed, KaannaTankkiOikeinpain, "Käännä tankki ympäri", tankki);
        controller.Listen(Button.DPadLeft, ButtonState.Down, KaannaPutkea, "Käännä tykkiä vasemmalle", tankki, 1);
        controller.Listen(Button.DPadRight, ButtonState.Down, KaannaPutkea, "Käännä tykkiä oikealle", tankki, -1);
        controller.ListenAnalog(AnalogControl.RightTrigger, 0.1, ViritaTykkiAnalogisesti, "Ammu", tankki);
        controller.Listen(Button.RightTrigger, ButtonState.Released, Ammu, "Ammu", tankki);
        controller.Listen(Button.A, ButtonState.Down, ViritaTykki, "Ammu", tankki);
        controller.Listen(Button.A, ButtonState.Released, Ammu, null, tankki);
        controller.Listen(Button.RightShoulder, ButtonState.Down, ViritaTykki, "Ammu", tankki);
        controller.Listen(Button.RightShoulder, ButtonState.Released, Ammu, null, tankki);
    }


    /// <summary>
    /// Lisätään Windows Phonen ohjaukset
    /// </summary>
    void LisaaKosketusKontrollit()
    {
#if WINDOWS_PHONE
        Label vasemmalleLiikutusPainike;
        Label oikealleLiikutusPainike;
        Label ammuPainike;

        TouchPanel.Clear();

        double marginaali = 0;
        Color vari = new Color(20, 20, 20, 15);

        vasemmalleLiikutusPainike = new Label(Screen.Width * 0.20, Screen.Height * 0.8, "<-") { Color = vari, BorderColor = Color.Gray, TextColor = Color.White };
        vasemmalleLiikutusPainike.X = Screen.Left + marginaali + vasemmalleLiikutusPainike.Width / 2;
        vasemmalleLiikutusPainike.Y = Screen.Bottom + marginaali + vasemmalleLiikutusPainike.Height / 2;
        Add(vasemmalleLiikutusPainike);

        oikealleLiikutusPainike = new Label(Screen.Width * 0.20, Screen.Height * 0.8, "->") { Color = vari, BorderColor = Color.Gray, TextColor = Color.White };
        oikealleLiikutusPainike.X = Screen.Right - marginaali - oikealleLiikutusPainike.Width / 2;
        oikealleLiikutusPainike.Y = Screen.Bottom + marginaali + oikealleLiikutusPainike.Height / 2;
        Add(oikealleLiikutusPainike);

        ammuPainike = new Label(Screen.Width * 0.60, Screen.Height * 0.8, "Shoot") { Color = vari, BorderColor = Color.Gray, TextColor = Color.White };
        ammuPainike.X = Screen.Center.X;
        ammuPainike.Y = Screen.Bottom + marginaali + ammuPainike.Height / 2;
        Add(ammuPainike);

        TouchPanel.ListenOn(vasemmalleLiikutusPainike, ButtonState.Down, delegate(Touch kosketus) { AjaTankkia(tankki1, vaantoVoima); }, null);
        TouchPanel.ListenOn(oikealleLiikutusPainike, ButtonState.Down, delegate(Touch kosketus) { AjaTankkia(tankki1, -vaantoVoima); }, null);
        TouchPanel.ListenOn(point1, ButtonState.Pressed, AloitaUusiPeli, null);
        TouchPanel.ListenOn(ammo1, ButtonState.Pressed, AloitaUusiPeli, null);
        TouchPanel.ListenOn(ammuPainike, ButtonState.Down, ViritaTykkiaTouch, null);
        TouchPanel.ListenOn(ammuPainike, ButtonState.Released, delegate(Touch kosketus) { Ammu(tankki1); ammuPainike.Text = ""; }, null);
        PhoneBackButton.Listen(Exit, "Lopeta peli");
#endif
    }


    /// <summary>
    /// Viritetään tykkiä sen mukaan kaunako on painettu
    /// </summary>
    /// <param name="kosketus">kosketusnäytön tila</param>
    void ViritaTykkiaTouch(Touch kosketus)
    {
        ViritaTykki(tankki1);
        Vector p = kosketus.PositionOnWorld;
        Vector suunta = (p - tankki1.Position).Normalize();
        tankki1.Cannon.Angle = suunta.Angle;
    }


    /// <summary>
    /// Aloitetaan uusi peli kun on painettu pelinaloitusaluetta
    /// </summary>
    /// <param name="kosketus">kosketusnäytön tila</param>
    void AloitaUusiPeli(Touch kosketus)
    {
        AloitaPeli();
    }


#if WINDOWS_PHONE
    /// <summary>
    /// Kun Windows Phonessa palataan takaisin, aloitetaan uusi peli
    /// </summary>
    public override void Continue()
    {
        Begin();
    }
#endif

    /// <summary>
    /// Jos joku osuu puuhun, niin puuhun alkaa vaikuttamaan painovoima
    /// </summary>
    /// <param name="lahde"></param>
    /// <param name="kohde"></param>
    public void OsuPuuhun(PhysicsObject lahde, PhysicsObject kohde)
    {
        if (lahde is Este) lahde.Body.IgnoresGravity = false;
    }


    private bool clearing = false; // estetään sisäkkäiset pyyhkimiset


    /// <summary>
    /// Aloitetaan uusi peli poistamalla kaikki ja sitten alustetaan uudelleen.
    /// Poistamisen ajaksi estetään tämän uusi kutsu
    /// </summary>
    protected void AloitaPeli()
    {
        if (clearing) return; // jos poistaminen menossa, ei tehdä uudestaan ennenkuin valmis
        clearing = true;
        ClearAll();
        Begin();
        this.Gravity = new Vector(0, -200);
    }


    /// <summary>
    /// Pelin lopettaminen
    /// </summary>
    protected void LopetaPeli()
    {
        this.Exit();
    }


    /// <summary>
    /// Viritetään valitun tankin tykin voimaa
    /// </summary>
    /// <param name="tankki">tankki, jonka tykkiä viritetään</param>
    public void ViritaTykki(Tank tankki)
    {
        tankki.Cannon.Power.Value += 20 * Game.Time.SinceLastUpdate.TotalMilliseconds;
    }


    /// <summary>
    /// Viritetään tykin voimaa ohjaimen tiedoista
    /// </summary>
    /// <param name="liipasimenTila">missä kohti liipasin menossa</param>
    /// <param name="tankki">minkä tankin tykkiä viritetään</param>
    public void ViritaTykkiAnalogisesti(AnalogState liipasimenTila, Tank tankki)
    {
        double voima = tankki.Cannon.Power.MaxValue * liipasimenTila.State;
        if (voima > tankki.Cannon.Power.Value)
        {
            // Suurin asento missä liipasin on käynyt
            tankki.Cannon.Power.Value = voima;
        }
    }


    /// <summary>
    /// Ammutaan valitulla tankilla
    /// </summary>
    /// <param name="tankki">tankki, jolla ammutaan</param>
    public void Ammu(Tank tankki)
    {
        tankki.Shoot();
        tankki.Cannon.Power.Reset();
        clearing = false;
    }


    /// <summary>
    /// Käännetään valitun tankin tykkiä lisää haluttuun suuntaan
    /// </summary>
    /// <param name="tankki">minkä tankin tykkiä käännetään</param>
    /// <param name="suunta">mihin suuntaan käännetään</param>
    protected void KaannaPutkea(Tank tankki, int suunta)
    {
        Angle pyorimisNopeus = Angle.FromRadians(1); // radiaania per sekunti
        double sekuntejaKulunut = Game.Time.SinceLastUpdate.TotalSeconds; // Aikaa edellisestä kutsusta
        tankki.Cannon.Angle += (suunta * pyorimisNopeus * sekuntejaKulunut);
    }


    /// <summary>
    /// Käännetään tykkiä peliohjaimen tatin asennon mukaan
    /// </summary>
    /// <param name="tatti">tatin tilanne</param>
    /// <param name="tankki">minkä tankin tykkiä käännetään</param>
    protected void KaannaPutkeaAnalogisesti(AnalogState tatti, Tank tankki)
    {
        double angle = tatti.StateVector.X;
        double newAngle = -angle * Math.PI / 1.5 + Math.PI / 2;

        tankki.Cannon.Angle = (Angle.FromRadians(newAngle));
    }


    /// <summary>
    /// Palauttaa tankin takaisin pystyyn
    /// </summary>
    /// <param name="tankki">tankki joka käännetään</param>
    private void KaannaTankkiOikeinpain(Tank tankki)
    {
        tankki.Stop();
        tankki.Angle = new Angle();
    }


    /// <summary>
    /// Ajetaan tankkia väännön määräämään suuntaan
    /// </summary>
    /// <param name="tankki">tankki jota ajetaan</param>
    /// <param name="vaanto">millä väännöllä tankki kiihdytetään</param>
    private void AjaTankkia(Tank tankki, double vaanto)
    {
        tankki.Accelerate(vaanto);
    }


    /// <summary>
    /// Ajetaan tankkia ohjaimen taitn asennon perusteella
    /// </summary>
    /// <param name="tatti">tatin tila</param>
    /// <param name="tankki">mitä tankkia ajetaan</param>
    private void AjaTankkiaAnalogisesti(AnalogState tatti, Tank tankki)
    {
        double torque = -(tatti.StateVector.X * 200);
        tankki.Accelerate(torque);
        //MessageDisplay.Clear();
        //MessageDisplay.Add(String.Format("{0:F0} {1:F0}", force.X, force.Y));
        //MessageDisplay.Add(String.Format("{0:F0}", torque));
    }


    /// <summary>
    /// Lyödään tankki kulkemaan tiettyyn suuntaan
    /// </summary>
    /// <param name="tankki">mitä tankkia lyödään</param>
    /// <param name="voima">millä voimalla lyödään</param>
    /// <param name="suunta">mihin suuntaan lyödään, jos ylöspäin, niin menetetään ammuksia</param>
    protected void LiikutaTankkia(Tank tankki, double voima, double suunta)
    {
        //  if ( tankki.tankkiTuhoutunut ) return false;

        if (1 <= suunta && suunta <= 2)
        { // Jos hypätään, menetetään ammuksia
            // if ( tankki.ammuksetLoppu ) return false; // ei voi enää hypätä, jos ammukset lopussa
            --tankki.Ammo.Value;
        }
        tankki.Hit(Vector.FromLengthAndAngle(voima, Angle.FromRadians(suunta)));

    }


    /// <summary>
    /// Luo taulukon jossa on alkuosassa on alkuperäinen taulukko ja loppuosassa tämän pelikuva.
    /// </summary>
    /// <param name="luvut">peilattava taulukko</param>
    /// <param name="pituusero">paljonko pituutta muutetaan 0=2x isompi, -1 = viimeinen alkio vain 1 kerran</param>
    /// <returns>peilattu taulukko</returns>
    /// @example
    /// <pre name="test">
    ///    double[] luvut = {1,4,2};
    ///    string.Join(",",Utils.peilaa(luvut,0)) === "1,4,2,2,4,1";
    ///    string.Join(",",Utils.peilaa(luvut,-1)) === "1,4,2,4,1";
    ///    string.Join(",",Utils.peilaa(luvut,1)) === "1,4,2,0,2,4,1";
    ///    string.Join(",",Utils.peilaa(luvut,2)) === "1,4,2,0,0,2,4,1";
    ///    string.Join(",",Utils.peilaa(luvut,-2)) === "1,4,4,1";
    ///    string.Join(",",Utils.peilaa(luvut,-5)) === "1";
    ///    string.Join(",",Utils.peilaa(luvut,-6)) === "";
    ///    string.Join(",",Utils.peilaa(luvut,-7)) === ""; // ei heitä # THROWS System.OverflowException
    ///    double[] luvut2 = {1,4};
    ///    string.Join(",",Utils.peilaa(luvut2,0)) === "1,4,4,1";
    ///    string.Join(",",Utils.peilaa(luvut2,-1)) === "1,4,1";
    /// </pre>
    /// @endexample
    public static double[] Peilaa(double[] luvut, int pituusero)
    {
        int koko = luvut.Length * 2 + pituusero;
        if (koko < 0) koko = 0;
        double[] result = new double[koko];
        for (int i = 0, j = result.Length - 1; i < luvut.Length && i <= j; i++, j--)
        {
            result[j] = result[i] = luvut[i];
        }
        return result;
    }
}


/// <summary>
/// Este, jolla on kestolaskuri
/// </summary>
public class Este : PhysicsObject
{
    /// <summary>
    /// Esteen kesto
    /// </summary>
    public int Kesto { get; set; }

    /// <summary>
    /// Esteen alustaminen
    /// </summary>
    /// <param name="name">esteelle annettava nimi</param>
    /// <param name="width">esteen leveys</param>
    /// <param name="heigth">esteen korkeus</param>
    /// <param name="mass">esteen massa</param>
    /// <param name="kesto">esteen kestoominaisuuden alkuarvo</param>
    public Este(string name, double width, double heigth, double mass, int kesto)
        : base(width, heigth)
    {
        this.Mass = mass;
        Kesto = kesto;
    }
}


