﻿using System;
using System.Collections.Generic;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Effects;
using Jypeli.Widgets;

/// @author  Vesa Lappalainen
/// @version 16.10.2011
/// @version 24.11.2013
/// <summary>
/// Osa, jolle voidaan asettaa suunta johon se pyrkii.
/// Suunta käännetään toisinpäin jos ollaan menossa vasemmalle.
/// </summary>
public class KaantyvaOsa : PhysicsObject
{
    private Angle oikea = Angle.FromDegrees(0);
    private Angle vasen = Angle.FromDegrees(180);
    private Angle suuntaero = Angle.FromDegrees(0);
    private int s = 1;
    /// <summary>
    /// Tavoitekulma, johon maila pyrkii osoittamaan
    /// </summary>
    public Angle Tavoite { get { return tavoite; } set { tavoite = value; saavutettu = false; } }
    private Angle tavoite;
    private bool saavutettu = true;
    private Direction suunta = Direction.Right;
    public Direction Suunta
    {
        get { return suunta; }
        set
        {
            if (value != Suunta) FlipImage();
            suunta = value;
            if (value == Direction.Left) { suuntaero = vasen; s = -1; } else { suuntaero = oikea; s = 1; }
            Tavoite = Tavoite;
        }
    }


    public KaantyvaOsa(double width, double height)
        : base(width, height)
    {
        // MakeStatic();
        Mass = 10000;
        IgnoresGravity = true;
        IsUpdated = true;
    }


    /// <summary>
    /// Uusi versio Update-metodista, jossa nyt käännetään osaa kohti tavoitekulmaa.
    /// </summary>
    /// <param name="time">Peliaika</param>
    public override void Update(Time time)
    {
        if (saavutettu) { base.Update(time); return; }
        Angle diff = s * Tavoite + suuntaero - Angle;
        double suunta = diff.Degrees;
        double d = 2;
        if (Math.Abs(suunta) > d)
        {
            if (suunta > 180) suunta = 360 - suunta;
            this.StopAngular();
            this.ApplyTorque(10000000 * 35.0 * suunta); // kokeiltu hihavakio
        }
        else
        {
            Stop();
            saavutettu = true;
        }
        base.Update(time);
    }
}

