﻿using System;
using Jypeli;

/// <summary>
/// "Taulukko" tiernapojan kuville
/// </summary>
public struct TiernapojanKuvat
{
    public Image vartalo;
    public Image vartaloTakaa;
    public Image polvillaan;
    public Image kasi;
    public Image miekka;
    public Image kahva;
    public Image paa;
    public Image paaTakaa;
    public Image suuAuki;
}



/// <summary>
/// Tiernapoika joka osaa heiluttaa kättä, päätä ja suuta.
/// Kädessä voi olla miekka tai miekan voi laittaa tuppeen.
/// </summary>
public class Tiernapoika : PlatformCharacter
{
    // Tiernapojan osat
    private KaantyvaOsa paa;
    private KaantyvaOsa kasi;
    private PhysicsObject miekka;
    private PhysicsObject miekankahva;


    // Alkuperäiset koot
    private Vector koko;
    private Vector paanKoko;
    private Vector kadenKoko;
    private Vector miekanKoko;
    private Vector miekanKahvanKoko;

    public Vector OriginalPos { get; set; }

    private TiernapojanKuvat kuvat;

    private Direction suunta;

    private PhysicsGame peli;

    public KaantyvaOsa Kasi
    {
        get { return kasi; }
    }

    public PhysicsObject Miekka
    {
        get { return miekka; }
    }

    public bool LaulaStick { get; set; }
    public void FlipLaulaStick() { LaulaStick = !LaulaStick; }

    public Tiernapoika(PhysicsGame peli, double leveys, double korkeus, TiernapojanKuvat kuvat)
        : base(leveys, korkeus)
    {
        this.kuvat = kuvat;
        double suhde = korkeus / leveys; // olion kuvasuhde
        Image = kuvat.vartalo;
        Height = Width * Image.Height / Image.Width * suhde; // tekstuurin kuvasuhde
        koko = Size;
        CollisionIgnoreGroup = 1;
        Mass = 5000.0;
        suunta = Direction.Right;
        DirectionChanged += new Action<Direction>(Kaantyi);

        // Pojan pää
        paa = new KaantyvaOsa(1, 1);
        paa.Image = kuvat.paa;
        paa.Height = paa.Image.Height * Size.Y / Image.Height;
        paa.Width = paa.Image.Width * Size.X / Image.Width;
        paa.IgnoresGravity = true;
        paa.CollisionIgnoreGroup = 1;
        paanKoko = paa.Size;
        paa.Mass = 10000;
        paa.Suunta = Direction.Left;
        Add(paa);

        // Pojan oikea käsi
        kasi = new KaantyvaOsa(Height, Height);
        kasi.Image = kuvat.kasi;
        kasi.Height = kasi.Image.Height * Height / Image.Height / suhde;
        kasi.Width = kasi.Height / kasi.Image.Height * kasi.Image.Width * suhde;
        // Poika ja käsi eivät saa törmätä toisiinsa
        kasi.CollisionIgnoreGroup = 1;
        kadenKoko = kasi.Size;
        kasi.Mass = 2000;
        Add(kasi);


        miekka = new PhysicsObject(Height * 0.7, 1);
        miekka.Image = kuvat.miekka;
        miekka.Height = miekka.Width * miekka.Image.Height / miekka.Image.Width;
        miekka.IgnoresGravity = true;
        miekka.CollisionIgnoreGroup = 1;
        miekanKoko = miekka.Size;
        miekka.Angle = Angle.FromDegrees(10);
        kasi.Add(miekka);

        miekankahva = new PhysicsObject(Height * 0.7, 1);
        miekankahva.Image = kuvat.kahva;
        miekankahva.Height = miekankahva.Image.Height * miekanKoko.Y / miekka.Image.Height;
        miekankahva.Width = miekankahva.Image.Width * miekanKoko.X / miekka.Image.Width;
        miekankahva.IgnoresGravity = true;
        miekankahva.CollisionIgnoreGroup = 1;
        miekanKahvanKoko = miekankahva.Size;
        miekankahva.Angle = Angle.FromDegrees(-20);
        Add(miekankahva);

        peli.Add(this, 1);
        this.peli = peli;
        MiekkaEsiin();
        SaadaKoko(koko);
    }

    private void Kaantyi(Direction suunta)
    {
        double suuntakerroin = 1;
        miekka.StopAngular();
        if (suunta == Direction.Left) suuntakerroin = -1;
        miekka.Angle = Angle.FromDegrees(10 * suuntakerroin);
        miekankahva.Angle = Angle.FromDegrees(-20 * suuntakerroin);
        kasi.Suunta = suunta;
        // paa.Suunta = suunta;
        this.suunta = suunta;
        paa.Tavoite = Angle.FromDegrees(suuntakerroin * 20);

        // Aina kääntäessä asetetaan paikka kaikille vehkeille
        kasi.Position = new Vector(Width / 2 * 0.85 * suuntakerroin, Height * 0.27);
        paa.Position = new Vector(Width * 0.1 * suuntakerroin, Height * 0.3);
        miekankahva.Position = new Vector(0, Height * 0.1);
        miekka.Position = new Vector(kasi.Width / 2 - 10, 0);
    }


    private void SaadaKoko(Vector uusikoko, bool myosMuut = true)
    {
        Size = uusikoko;
        if (myosMuut)
        {
            double kerroin = Size.Y / koko.Y;
            kasi.Size = kadenKoko * kerroin;
            miekka.Size = miekanKoko * kerroin;
            miekankahva.Size = miekanKahvanKoko * kerroin;
            paa.Size = paanKoko * kerroin;
        }
        Kaantyi(suunta);
    }


    public void Flip()
    {
        if (suunta == Direction.Left) Walk(0.1);
        else Walk(-0.1);
    }


    public void Nouse(double nopeus)
    {
        SaadaKoko(koko);
        Kaantyi(suunta);
        Image = kuvat.vartalo;
        paa.Image = kuvat.paa;
    }

    public void Polvistu(double nopeus)
    {
        SaadaKoko(new Vector(Width, koko.Y / 2), false);
        MiekkaTuppeen();
        Image = kuvat.polvillaan;
        if (Bottom < peli.Level.Bottom)
            Bottom = peli.Level.Bottom;
    }

    public void Pois(double nopeus)
    {
        if (Height < koko.Y / 2) return;
        if (Image == kuvat.polvillaan) Nouse(1);
        SaadaKoko(Size * 0.99);
        //  peli.Remove(this);
        //  peli.Add(this, -1);
        if (Image != kuvat.vartaloTakaa)
        {
            Image = kuvat.vartaloTakaa;
            paa.Image = kuvat.paaTakaa;
            Flip();
            miekankahva.IsVisible = false;
        }
    }

    public void Takaisin(double nopeus)
    {
        if (Height > koko.Y) Nouse(nopeus);
        SaadaKoko(Size * 1.01);
        // peli.Remove(this);
        // peli.Add(this, 1);
        if (Image != kuvat.vartalo)
        {
            Image = kuvat.vartalo;
            paa.Image = kuvat.paa;
            Flip();
            if (!miekka.IsVisible) miekankahva.IsVisible = true;
        }
    }

    public void MiekkaTuppeen()
    {
        miekka.IsVisible = false;
        miekankahva.IsVisible = true;
        Osoita(-125);
    }

    public void MiekkaEsiin()
    {
        Osoita(-110);
        miekka.IsVisible = true;
        miekankahva.IsVisible = false;
        miekka.Velocity = Vector.Zero;
    }

    public void Pyorita()
    {
        miekka.ApplyTorque(100000);
    }

    public void MiekkaSwap()
    {
        if (miekka.IsVisible) MiekkaTuppeen(); else MiekkaEsiin();
    }

    public void KyykyynYlos(double nopeus)
    {
        if (Height < koko.Y * 0.6) Nouse(nopeus); else Polvistu(nopeus);
    }

    public void Osoita(double kulma)
    {
        miekka.StopAngular();
        Kasi.Tavoite = Angle.FromDegrees(kulma);
    }

    public void KaannaPaa(double kulma)
    {
        paa.Tavoite = Angle.FromDegrees(kulma);
    }

    public void AvaaSuu()
    {
        if (paa.Image == kuvat.paa) paa.Image = kuvat.suuAuki;
    }

    public void SuljeSuu()
    {
        if (paa.Image == kuvat.suuAuki) paa.Image = kuvat.paa;
    }

    public void Suu(double kulma)
    {
        if (kulma <= 20) SuljeSuu(); else AvaaSuu();
    }

    public void Liiku(double speed)
    {
        Walk(speed);
    }


    private Color veri = new Color(200, 0, 0);

    public void TiputaVerta()
    {
        PhysicsObject pisara = new PhysicsObject(4, 6, Shape.Circle);
        pisara.Position = kasi.AbsolutePosition + Vector.FromLengthAndAngle(kasi.Width / 2, kasi.Angle) + Vector.FromLengthAndAngle(0.95 * miekka.Width / 2, kasi.Angle + Miekka.Angle);
        // pisara.Position = miekka.AbsolutePosition; // jos tämä on, niin verui tippuu mistä sattuu
        pisara.Color = veri;
        pisara.CollisionIgnoreGroup = 1; // Tämän jos kommentoi, niin kädet irtoavat
        peli.Add(pisara);
        pisara.MaximumLifetime = TimeSpan.FromMilliseconds(3000);
        pisara.Mass = 1;
        pisara.LinearDamping = 0.95;

        peli.AddCollisionHandler(pisara, "Lattia", delegate (PhysicsObject o1, PhysicsObject o2) {
            // pisara.Width = 30; 
            // pisara.Velocity = Vector.Zero; 
            PhysicsObject latakko = new PhysicsObject(30, 4, Shape.Ellipse);
            latakko.Color = veri;
            latakko.Position = pisara.Position;
            latakko.CollisionIgnoreGroup = 1;
            peli.Add(latakko);
            latakko.MaximumLifetime = TimeSpan.FromMilliseconds(2000);
        });

    }


    public void Start()
    {
        Position = OriginalPos;
        Nouse(100);
    }
}


