using System;
using System.Collections.Generic;
using System.IO;
using Jypeli;
using Jypeli.Assets;
using Jypeli.Controls;
using Jypeli.Effects;
using Jypeli.Widgets;

/// <summary>
/// Tiernapoikapeli joulujuhlia varten.
/// Katso ohjainten käyttöohje:
/// 
///  https://kurssit.it.jyu.fi/ITKP102/content/Tiernapojat/tiernapoika_ohjaimet.pptx
///  
/// GITissä ei ole seuraavia tiedostoja ja ne pitää hakea Content-kansioon
/// osoitteesta https://kurssit.it.jyu.fi/ITKP102/content/Tiernapojat/Content/
///   
///  - tiernapoika.wav
///  - verho2.png
///  
/// </summary>
public class Tiernapojat : PhysicsGame
{

    private static readonly string[] taso0 = {
"                   *    ",
"      *                 ",
"   *     *     *        ",
" *        *   *         ",
"                        ",
"                        ",
"                        ",
"                        ",
"                        ",
"                        ",
"                        ",
"                        ",
"                        ",
"                        ",
"                        ",
"                        ",
"                        ",
"   M    H      K      N ",
"=                       ",
                                             };


    const double nopeus = 200;
    const double hyppyNopeus = 750;
    const int RUUDUN_KOKO = 40;

    private List<Tiernapoika> pojat = new List<Tiernapoika>();

    private Image tahtiKuva = LoadImage("tahti");

    private TiernapojanKuvat herodeksenKuvat = new TiernapojanKuvat { vartalo = LoadImage("TP_Herodes"), vartaloTakaa = LoadImage("TP_HerodesBack"), polvillaan = LoadImage("TP_HerodesCrouch"), kasi = LoadImage("TP_HerodesHand"), miekka = LoadImage("TP_sword"), kahva = LoadImage("TP_handle"), paa = LoadImage("TP_HerodesHead"), paaTakaa = LoadImage("TP_HerodesHeadBack"), suuAuki = LoadImage("TP_HerodesHeadSpeak") };
    private TiernapojanKuvat murjaaninKuvat = new TiernapojanKuvat { vartalo = LoadImage("TP_Murjaani"), vartaloTakaa = LoadImage("TP_MurjaaniBack"), polvillaan = LoadImage("TP_MurjaaniCrouch"), kasi = LoadImage("TP_MurjaaniHand"), miekka = LoadImage("TP_sword"), kahva = LoadImage("TP_handle"), paa = LoadImage("TP_MurjaaniHead"), paaTakaa = LoadImage("TP_MurjaaniHeadBack"), suuAuki = LoadImage("TP_MurjaaniHeadSpeak") };
    private TiernapojanKuvat knihdinKuvat = new TiernapojanKuvat { vartalo = LoadImage("TP_Knihti"), vartaloTakaa = LoadImage("TP_KnihtiBack"), polvillaan = LoadImage("TP_HerodesCrouch"), kasi = LoadImage("TP_HerodesHand"), miekka = LoadImage("TP_sword"), kahva = LoadImage("TP_handle"), paa = LoadImage("TP_KnihtiHead"), paaTakaa = LoadImage("TP_KnihtiHeadBack"), suuAuki = LoadImage("TP_KnihtiHeadSpeak") };
    private TiernapojanKuvat mankinKuvat = new TiernapojanKuvat { vartalo = LoadImage("TP_Manki"), vartaloTakaa = LoadImage("TP_MankiBack"), polvillaan = LoadImage("TP_HerodesCrouch"), kasi = LoadImage("TP_MankiHand"), miekka = LoadImage("TP_star"), kahva = LoadImage("TP_star"), paa = LoadImage("TP_MankiHead"), paaTakaa = LoadImage("TP_MankiHeadBack"), suuAuki = LoadImage("TP_MankiHeadSpeak") };

    private SoundEffect maaliAani = LoadSoundEffect("maali");


    public override void Begin()
    {
        SetWindowSize((int)Screen.Width, (int)Screen.Height);
        GamePad[] ohjaimet = { ControllerOne, ControllerTwo, ControllerThree, ControllerFour };
        Gravity = new Vector(0, -1000);

        LuoKentta();

        int ci = 0;
        foreach (Tiernapoika poika in pojat)
        {
            LisaaNappaimet(ohjaimet[ci++], poika);
            if (ci >= ohjaimet.Length) break;
        }

        Verho verho = new Verho(Screen.Width, Screen.Height, LoadImage("verho2"));
        Add(verho, 3);


        Keyboard.Listen(Key.Space, ButtonState.Pressed, LiikutaVerho, null, verho);

        Keyboard.Listen(Key.F1, ButtonState.Pressed, ShowControlHelp, "Näytä ohjeet");
        Keyboard.Listen(Key.Escape, ButtonState.Pressed, ConfirmExit, "Lopeta peli");

        // Camera.Follow(poika1);
        Camera.ZoomFactor = 1;
        // Camera.ZoomToAllObjects(0);
        // Camera.StayInLevel = true;
    }

    void LiikutaVerho(Verho verho)
    {
        if (verho.Nostettu)
        {
            verho.MoveTo(new Vector(verho.X, verho.Y - verho.Height), 250);
            MediaPlayer.Stop();
        }
        else
        {
            verho.MoveTo(new Vector(verho.X, verho.Y + verho.Height), 250);
            try
            {
                MediaPlayer.Play("tiernapoika"); // TODO: eikö voi soittaa mp3???
            } catch (FileNotFoundException)
            {
                // jatketaan ilman musiikkia
            }
        }
        verho.Nostettu = !verho.Nostettu;
    }

    void LuoKentta()
    {
        // TileMap kentta = TileMap.FromLevelAsset("kentta1");
        TileMap kentta = TileMap.FromStringArray(taso0);
        kentta.SetTileMethod('#', LisaaTaso);
        kentta.SetTileMethod('=', LisaaPohjaTaso);
        kentta.SetTileMethod('*', LisaaTahti);
        kentta.SetTileMethod('H', (p, l, k) => LisaaPelaaja(p, l, k, herodeksenKuvat));
        kentta.SetTileMethod('M', (p, l, k) => LisaaPelaaja(p, l * 1.3, k * 1.1, murjaaninKuvat));
        kentta.SetTileMethod('K', (p, l, k) => LisaaPelaaja(p, l * 1.2, k * 0.9, knihdinKuvat));
        kentta.SetTileMethod('N', (p, l, k) => LisaaPelaaja(p, l, k, mankinKuvat));
        kentta.Execute(RUUDUN_KOKO, RUUDUN_KOKO);
        //Level.CreateBorders();
        Level.Background.CreateGradient(Color.SkyBlue, Color.Blue);

        PhysicsObject vasen = new PhysicsObject(100, 2000);
        vasen.Right = Level.Left - 500;
        vasen.MakeStatic();
        Add(vasen);

        PhysicsObject oikea = new PhysicsObject(100, 2000);
        oikea.Left = Level.Right + 500;
        oikea.MakeStatic();
        Add(oikea);
    }


    void LisaaTaso(Vector paikka, double leveys, double korkeus)
    {
        PhysicsObject taso = PhysicsObject.CreateStaticObject(leveys, korkeus);
        taso.Position = paikka;
        taso.Color = Color.Green;
        Add(taso);
    }


    void LisaaPohjaTaso(Vector paikka, double leveys, double korkeus)
    {
        PhysicsObject taso = PhysicsObject.CreateStaticObject(leveys * 100, korkeus);
        taso.Position = paikka;
        taso.Color = Color.Green;
        taso.Tag = "Lattia";
        Add(taso);
    }


    void LisaaTahti(Vector paikka, double leveys, double korkeus)
    {
        PhysicsObject tahti = PhysicsObject.CreateStaticObject(leveys, korkeus);
        tahti.IgnoresCollisionResponse = true;
        tahti.Position = paikka;
        tahti.Image = tahtiKuva;
        tahti.Tag = "tahti";
        Add(tahti);
    }


    void LisaaPelaaja(Vector paikka, double leveys, double korkeus, TiernapojanKuvat kuvat)
    {
        Tiernapoika poika = new Tiernapoika(this, leveys * 4, korkeus * 4, kuvat);
        poika.Position = paikka;
        poika.OriginalPos = paikka;
        poika.Color = Color.Red;

        AddCollisionHandler(poika, "tahti", TormaaTahteen);
        AddCollisionHandler(poika.Miekka, "tahti", TormaaTahteen);
        pojat.Add(poika);
    }


    void LisaaNappaimet(GamePad controller, Tiernapoika poika)
    {
        Keyboard.Listen(Key.Left, ButtonState.Down, () => poika.Liiku(-nopeus), "Liikkuu vasemmalle");
        Keyboard.Listen(Key.Right, ButtonState.Down, () => poika.Liiku(nopeus), "Liikkuu vasemmalle");
        Keyboard.Listen(Key.Up, ButtonState.Pressed, Hyppaa, "Pelaaja hyppää", poika, hyppyNopeus);
        Keyboard.Listen(Key.Multiply, ButtonState.Down, () => poika.Pyorita(), "Pyöritä tähteä");
        Keyboard.Listen(Key.Down, ButtonState.Pressed, () => poika.Polvistu(hyppyNopeus), "Pelaaja polvistuu");
        Keyboard.Listen(Key.Subtract, ButtonState.Down, () => poika.Pois(nopeus), "Kävele pois");
        Keyboard.Listen(Key.Add, ButtonState.Down, () => poika.Takaisin(nopeus), "Kävele takaisin");
        Keyboard.Listen(Key.Divide, ButtonState.Down, () => { poika.MiekkaEsiin(); poika.Osoita(80); }, "Miekka pystyyn");
        Keyboard.Listen(Key.Back, ButtonState.Pressed, () => poika.AvaaSuu(), "Avaa suu");
        Keyboard.Listen(Key.Back, ButtonState.Released, () => poika.SuljeSuu(), "Sulje suu");
        Keyboard.Listen(Key.V, ButtonState.Pressed, () => poika.TiputaVerta(), "Tiputa verta");

        // controller.Listen(Button.Back, ButtonState.Pressed, Exit, "Poistu pelistä");
        controller.Listen(Button.DPadLeft, ButtonState.Down, () => poika.Liiku(-nopeus), "Pelaaja liikkuu vasemmalle");
        controller.Listen(Button.DPadRight, ButtonState.Down, () => poika.Liiku(nopeus), "Pelaaja liikkuu oikealle");
        controller.Listen(Button.DPadUp, ButtonState.Down, () => poika.Pois(nopeus), "Pelaaja poispäin");
        controller.Listen(Button.DPadDown, ButtonState.Down, () => poika.Takaisin(nopeus), "Pelaaja takaisinpäin");
        controller.Listen(Button.Y, ButtonState.Pressed, () => poika.Nouse(nopeus), "Pelaaja nousee");
        controller.Listen(Button.RightStick, ButtonState.Down, () => { poika.MiekkaEsiin(); poika.Osoita(80); }, "Miekka pystyyn");
        //controller.Listen(Button.A, ButtonState.Down,  () => poika.Polvistu(hyppyNopeus), "Pelaaja polvistuu");
        controller.Listen(Button.A, ButtonState.Pressed, () => poika.KyykyynYlos(nopeus), "Kyykyyn/Ylös");
        controller.Listen(Button.RightTrigger, ButtonState.Pressed, Hyppaa, "Pelaaja hyppää", poika, hyppyNopeus);
        controller.Listen(Button.LeftTrigger, ButtonState.Pressed, () => poika.TiputaVerta(), "Tiputa verta");
        controller.Listen(Button.LeftShoulder, ButtonState.Pressed, () => poika.Pyorita(), "Pyöritä tähteä");
        controller.Listen(Button.RightShoulder, ButtonState.Pressed, () => poika.AvaaSuu(), "Avaa suu");
        controller.Listen(Button.RightShoulder, ButtonState.Released, () => poika.SuljeSuu(), "Sulje suu");
        controller.Listen(Button.X, ButtonState.Pressed, () => poika.MiekkaSwap(), "Miekka pois/takaisin");
        controller.Listen(Button.B, ButtonState.Pressed, () => poika.MiekkaEsiin(), "Miekka esiin");
        controller.Listen(Button.Back, ButtonState.Pressed, () => poika.FlipLaulaStick(), "Laula Stick on/off");
        controller.Listen(Button.Start, ButtonState.Pressed, () => poika.Start(), "Alkuperäinen paikka");

        controller.ListenAnalog(AnalogControl.RightStick, 0.03, (a) => poika.Osoita(a.StateVector.Y * 130), "Liikuta kättä");
        controller.ListenAnalog(AnalogControl.LeftStick, 0.03, (a) => poika.KaannaPaa(a.StateVector.X * 60), "Käännä pää");
        controller.ListenAnalog(AnalogControl.LeftStick, 0.03, (a) => { if (poika.LaulaStick) poika.Suu(a.StateVector.Y * 100); }, "Liikuta suuta");

        PhoneBackButton.Listen(ConfirmExit, "Lopeta peli");
    }


    void Hyppaa(Tiernapoika hahmo, double nopeus)
    {
        hahmo.Nouse(nopeus);
        hahmo.Jump(nopeus);
    }


    void TormaaTahteen(PhysicsObject hahmo, PhysicsObject tahti)
    {
        maaliAani.Play();
        MessageDisplay.Add("Keräsit tähden!");
        tahti.Destroy();
    }
}



