﻿
namespace Utilities
{
    public class Utils
    {
        /// <summary>
        /// Luo taulukon jossa on alkuosassa on alkuperäinen taulukko ja loppuosassa tämän pelikuva.
        /// </summary>
        /// <param name="luvut">peilattava taulukko</param>
        /// <param name="pituusero">paljonko pituutta muutetaan 0=2x isompi, -1 = viimeinen alkio vain 1 kerran</param>
        /// <returns>peilattu taulukko</returns>
        /// @example
        /// <pre name="test">
        ///    double[] luvut = {1,4,2};
        ///    string.Join(",",Utils.peilaa(luvut,0)) === "1,4,2,2,4,1";
        ///    string.Join(",",Utils.peilaa(luvut,-1)) === "1,4,2,4,1";
        ///    string.Join(",",Utils.peilaa(luvut,1)) === "1,4,2,0,2,4,1";
        ///    string.Join(",",Utils.peilaa(luvut,2)) === "1,4,2,0,0,2,4,1";
        ///    string.Join(",",Utils.peilaa(luvut,-2)) === "1,4,4,1";
        ///    string.Join(",",Utils.peilaa(luvut,-5)) === "1";
        ///    string.Join(",",Utils.peilaa(luvut,-6)) === "";
        ///    string.Join(",",Utils.peilaa(luvut,-7)) === ""; 
        ///    double[] luvut2 = {1,4};
        ///    string.Join(",",Utils.peilaa(luvut2,0)) === "1,4,4,1";
        ///    string.Join(",",Utils.peilaa(luvut2,-1)) === "1,4,1";
        /// </pre>
        /// @endexample
        public static double[] peilaa(double[] luvut, int pituusero)
        {
            int pituus = luvut.Length * 2 + pituusero;
            if (pituus <= 0) return new double[0];
            double[] result = new double[pituus];
            for (int i = 0, j = result.Length - 1; i < luvut.Length && i <= j; i++, j--)
            {
                result[j] = result[i] = luvut[i];
            }
            return result;
        }
    }
}
